package com.bocloud.cos.booter.controller;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.MapTools;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.cos.common.model.BasicDomainModel;
import com.bocloud.cos.engine.model.DefinitionModel;
import com.bocloud.cos.entity.FlowDefinition;
import com.bocloud.cos.intf.FlowDefinitionService;
import com.bocloud.cos.model.DefinitionParams;
import com.bocloud.cos.model.DefinitionPatchParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author hufy
 */
@RestController
@Api(value = "流程定义", tags = "流程定义")
@RequestMapping({"/v1/definitions"})
public class FlowDefinitionController extends FlowBaseController {

    protected final FlowDefinitionService definitionService;

    @Autowired
    public FlowDefinitionController(FlowDefinitionService definitionService) {
        this.definitionService = definitionService;
    }

    /**
     * 流程定义列表
     *
     * @return
     */
    @GetMapping
    @ApiOperation(tags = {"CMC", "CSC"}, value = "流程定义列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null != pager.getParams()) {
            pager.getParams().add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
        }
        int count = definitionService.count(context, pager, true);
        List<FlowDefinition> latestDefinitions = definitionService.getDefinitions(context, pager, true);
        GridBean gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), count, latestDefinitions);
        return new BocloudResult(true, gridBean, "成功");
    }

    /**
     * 新增流程定义
     */
    @PostMapping
    @ApiOperation(tags = {"CMC", "CSC"}, value = "新增流程定义", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult create(@Value(Common.REQ_CONTEXT) RequestContext context,
                                @RequestBody DefinitionParams params) {
        Long definitionId = definitionService.deploy(params, context);
        if (definitionId == null) {
            return new BocloudResult(false, "创建失败");
        }
        BasicDomainModel<FlowDefinition, DefinitionModel> definitionById = definitionService
                .getDefinition(definitionId);
        if (definitionById != null) {
            return new BocloudResult(true, "创建成功");
        } else {
            return new BocloudResult(false, "创建失败");
        }
    }

    /**
     * 修改流程定义
     *
     * @param id
     * @param params
     * @return
     */
    @PutMapping(value = "/{id}")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "修改流程定义", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult modify(@Value(Common.REQ_CONTEXT) RequestContext context, @PathVariable(Common.ID) Long id,
                                @RequestBody DefinitionParams params) {
        return definitionService.redeploy(id, params, context);
    }

    /**
     * 激活、终止流程定义
     *
     * @param id
     * @param param
     * @return
     */
    @PatchMapping(value = "/{id}")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "激活、终止流程定义", response = BocloudResult.class, httpMethod = "PATCH", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult patch(@Value(Common.REQ_CONTEXT) RequestContext context, @PathVariable(Common.ID) Long id,
                               @RequestBody DefinitionPatchParam param) {
        if ("terminate".equals(param.getAction())) {
            boolean success = definitionService.terminate(id, context, param.getRemark());
            return new BocloudResult(success, success ? "终止成功" : "终止失败");
        } else if ("active".equals(param.getAction())) {
            boolean success = definitionService.active(id, context, param.getRemark());
            return new BocloudResult(success, success ? "激活成功" : "激活失败");
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 删除流程定义
     *
     * @param id
     * @return
     */
    @DeleteMapping(value = "/{id}")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除流程定义", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult remove(@Value(Common.REQ_CONTEXT) RequestContext context, @PathVariable(Common.ID) Long id) {
        return definitionService.remove(id, context);
    }

    /**
     * 流程定义详情
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/{id}")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "流程定义详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult detail(@PathVariable(Common.ID) Long id) {
        BasicDomainModel<FlowDefinition, DefinitionModel> definition = definitionService.getDefinition(id);
        if (definition == null) {
            return new BocloudResult(false, "没有找到该流程定义");
        }
        FlowDefinition entity = definition.getEntity();
        if (entity != null) {
            return new BocloudResult(true, entity, "Success");
        } else {
            return new BocloudResult(false, "Failed");
        }
    }

    /**
     * 查看更新历史
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/{id}/history")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看更新历史", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult listHistory(@Value(Common.REQ_CONTEXT) RequestContext context,
                                     @PathVariable(Common.ID) Long id) {
        List<FlowDefinition> definitionHistory = definitionService.getDefinitionHistory(context, id);
        return new BocloudResult(true, definitionHistory, "Success");
    }

}
