package com.bocloud.cos.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.intf.BillAnalysisService;
import com.bocloud.cos.intf.OrderAnalysisService;
import com.bocloud.cos.intf.ResourceAnalysisService;
import com.bocloud.cos.intf.TenantAnalysisService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

/**
 * 运营分析
 *
 * @author tyl
 * @time 2020/6/29 周一 9:23
 */
@RestController @RequestMapping("/v1/soa") @Api(value = "运营分析管理", tags = "运营分析管理")
public class OperateAnalysisController {

    private final OrderAnalysisService orderAnalysisService;
    private final ResourceAnalysisService resourceAnalysisService;
    private final BillAnalysisService billAnalysisService;
    private final TenantAnalysisService tenantAnalysisService;

    public OperateAnalysisController(OrderAnalysisService orderAnalysisService,
        ResourceAnalysisService resourceAnalysisService, BillAnalysisService billAnalysisService,
        TenantAnalysisService tenantAnalysisService) {
        this.orderAnalysisService = orderAnalysisService;
        this.resourceAnalysisService = resourceAnalysisService;
        this.billAnalysisService = billAnalysisService;
        this.tenantAnalysisService = tenantAnalysisService;
    }

    @GetMapping("/resources/statistics") @ApiOperation(tags = {"CMC",
        "CSC"}, value = "资源分析统计", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult resourceStatistics(@RequestParam(value = "vendorId", required = false) Long vendorId,
        @RequestParam(value = "vdcId", required = false) Long vdcId,
        @RequestParam(value = "clusterId", required = false) Long clusterId,
        @RequestParam(value = Common.CONDITION, required = false) String condition) {
        return resourceAnalysisService.resourceStatistics(vendorId, vdcId, clusterId, condition);
    }

    @GetMapping("/resources/getAllCloudCount") @ApiOperation(tags = {"CMC",
        "CSC"}, value = "云平台资源统计", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult getAllCloudCount(@RequestParam(value = "vendorId", required = false) Long vendorId,
        @RequestParam(value = "vdcId", required = false) Long vdcId,
        @RequestParam(value = "clusterId", required = false) Long clusterId) {
        return resourceAnalysisService.getAllCloudCount(vendorId, vdcId, clusterId);
    }

    @GetMapping("/resources/status") @ApiOperation(tags = {"CMC",
        "CSC"}, value = "资源状态分析统计", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult resourceStatus(@RequestParam(value = "vendorId", required = false) Long vendorId,
        @RequestParam(value = "vdcId", required = false) Long vdcId,
        @RequestParam(value = "clusterId", required = false) Long clusterId,
        @RequestParam(value = Common.CONDITION, required = false) String condition) {
        return resourceAnalysisService.resourceStatus(vendorId, vdcId, clusterId, condition);
    }

    @GetMapping("/resources/capacity") @ApiOperation(tags = {"CMC",
        "CSC"}, value = "运营分析统计", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult resourceCapacity(@RequestParam(value = "vendorId", required = false) Long vendorId,
        @RequestParam(value = "vdcId", required = false) Long vdcId,
        @RequestParam(value = "clusterId", required = false) Long clusterId,
        @RequestParam(value = "time", required = false) String time,
        @RequestParam(value = "resourceType", required = false) String resourceType,
        @RequestParam(value = Common.CONDITION, required = false) String condition) {
        return resourceAnalysisService.resourceCapacity(vendorId, vdcId, clusterId, time, resourceType, condition);
    }

    @GetMapping("/orders/dashboard") @ApiOperation(tags = {"CMC",
        "CSC"}, value = "订单分析概览", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult orderAnalysisOverview(@RequestParam(value = "vendorId", required = false) Long vendorId,
        @RequestParam(value = "time", required = false) String time,
        @RequestParam(value = "startTime", required = false) String startTime,
        @RequestParam(value = "endTime", required = false) String endTime) {
        return orderAnalysisService.getOrdersOverview(vendorId, time,startTime,endTime);
    }

    @GetMapping("/vendors/orders") @ApiOperation(tags = {"CMC",
        "CSC"}, value = "平台订单分析", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult vendorsOrders(@RequestParam(value = "condition", required = false) String condition,
        @RequestParam(value = "time", required = false) String time,
        @RequestParam(value = "startTime", required = false) String startTime,
        @RequestParam(value = "endTime", required = false) String endTime) {
        return orderAnalysisService.vendorOrderAnalyse(condition, time,startTime,endTime);
    }

    @GetMapping("/tenants/orders") @ApiOperation(tags = {"CMC",
        "CSC"}, value = "租户订单分析", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult tenantsOrders(@RequestParam(value = "tenantId", required = false) Long tenantId,
        @RequestParam(value = "condition", required = false) String condition,
        @RequestParam(value = "time", required = false) String time,
        @RequestParam(value = "startTime", required = false) String startTime,
        @RequestParam(value = "endTime", required = false) String endTime) {
        return orderAnalysisService.tenantOrderAnalyse(tenantId, condition, time,startTime,endTime);
    }

    @GetMapping("/orders/status") @ApiOperation(tags = {"CMC",
        "CSC"}, value = "订单状态分析", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult ordersStatus(@RequestParam(value = "time", required = false) String time,
        @RequestParam(value = "startTime", required = false) String startTime,
        @RequestParam(value = "endTime", required = false) String endTime,
        @RequestParam(value = "status", required = false) String status,
        @RequestParam(value = "condition", required = false) String condition) {
        return orderAnalysisService.orderStatusAnalyse(time,startTime,endTime, status, condition);
    }

    @GetMapping("/orders/types") @ApiOperation(tags = {"CMC",
        "CSC"}, value = "订单类型分析", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult ordersTypes(@RequestParam(value = "time", required = false) String time,
        @RequestParam(value = "startTime", required = false) String startTime,
        @RequestParam(value = "endTime", required = false) String endTime,
        @RequestParam(value = "type", required = false) String type,
        @RequestParam(value = "condition", required = false) String condition) {
        return orderAnalysisService.orderTypeAnalyse(time,startTime,endTime, type, condition);
    }

    @GetMapping("/resources/orders") @ApiOperation(tags = {"CMC",
        "CSC"}, value = "资源订单分析", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult resourcesOrders(@RequestParam(value = "time", required = false) String time,
        @RequestParam(value = "startTime", required = false) String startTime,
        @RequestParam(value = "endTime", required = false) String endTime,
        @RequestParam(value = "serviceCode", required = false) String serviceCode,
        @RequestParam(value = "condition", required = false) String condition) {
        return orderAnalysisService.resourceOrderAnalyse(time,startTime,endTime, serviceCode, condition);
    }

    @GetMapping("/bills/overview") @ApiOperation(tags = {"CMC",
        "CSC"}, value = "收入分析概览", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult billsOverview() {
        return billAnalysisService.incomeOverview();
    }

    @GetMapping("/bills") @ApiOperation(tags = {"CMC",
        "CSC"}, value = "收入分析", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult billsAnalysis(@RequestParam(value = "vendorId", required = false) Long vendorId,
        @RequestParam(value = "time", required = false) String time,
        @RequestParam(value = "serviceCode", required = false) String serviceCode,
        @RequestParam(value = "condition", required = false) String condition) {
        return billAnalysisService.billsAnalysis(vendorId, time, serviceCode, condition);
    }

    @GetMapping("/tenants/resources") @ApiOperation(tags = {"CMC",
        "CSC"}, value = "租户资源分析", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult tenantResourceAnalysis(@RequestParam(value = "tenantId", required = false) Long tenantId) {
        return tenantAnalysisService.tenantResourceUsed(tenantId);
    }

    @GetMapping("/tenants") @ApiOperation(tags = {"CMC",
        "CSC"}, value = "租户分析", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult tenantAnalysis(@RequestParam(value = "tenantId", required = false) Long tenantId,
        @RequestParam(value = "time", required = false) String time,
        @RequestParam(value = "startTime", required = false) String startTime,
        @RequestParam(value = "endTime", required = false) String endTime,
        @RequestParam(value = "resourceType", required = false) String resourceType,
        @RequestParam(value = "condition", required = false) String condition) {
        return tenantAnalysisService.tenantAnalysis(condition, time, startTime, endTime, tenantId, resourceType);
    }

    /**
     * ----------------容量分析----------------
     */

    @GetMapping("/resources/cloudCapacity") @ApiOperation(tags = {"CMC",
        "CSC"}, value = "私有云平台物理、虚拟资源总量", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult cloudCapacity(Pager pager, @RequestParam(value = "vendorName", required = false) String vendorName) {
        return resourceAnalysisService.cloudCapacity(pager,vendorName);
    }

    @GetMapping("/resources/capacityNew") @ApiOperation(tags = {"CMC",
        "CSC"}, value = "使用趋势分析", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult capacityNew(@RequestParam(value = "vendorId", required = false) Long vendorId,
        @RequestParam(value = "vdcId", required = false) Long vdcId,
        @RequestParam(value = "clusterId", required = false) Long clusterId,
        @RequestParam(value = "time", required = false) String time,
        @RequestParam(value = "startTime", required = false) String startTime,
        @RequestParam(value = "endTime", required = false) String endTime,
        @RequestParam(value = "resourceType", required = false) String resourceType,
        @RequestParam(value = "condition", required = false) String condition) {
        return resourceAnalysisService.resourceCapacityNew(vendorId, vdcId, clusterId, time,startTime,endTime, resourceType, condition);
    }

    /**
     * ------------账单分析--------------
     */
    @GetMapping("/billsAnalysisNew") @ApiOperation(tags = {"CMC",
        "CSC"}, value = "账单分析", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult billsAnalysisNew(@RequestParam(value = "time") String time,
        @RequestParam(value = Common.CONDITION) String condition,
        @RequestParam(value = "label", required = false) String label) {
        return billAnalysisService.billsAnalysisNew(time, condition, label);
    }

    @GetMapping("/billsAnalysisNew/download") @ApiOperation(tags = {"CMC",
        "CSC"}, value = "账单分析导出", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult billsAnalysisNewDownload(@RequestParam(value = "time") String time,
        @RequestParam(value = Common.CONDITION) String condition,
        @RequestParam(value = "label", required = false) String label, HttpServletRequest request,
        HttpServletResponse response) {
        BocloudResult bocloudResult = billAnalysisService.billsAnalysisNewDownload(time, condition, label);
        if (bocloudResult.isSuccess()) {
            String content = null;
            try {
                @SuppressWarnings("unchecked") Map<String, Object> result =
                    JSONObject.parseObject(JSONObject.toJSONString(bocloudResult.getData()), HashMap.class);
                response.setContentType("application/vnd.ms-excel");
                String fileName = result.get("fileName").toString();
                final String userAgent = request.getHeader("user-agent").toLowerCase();
                if (userAgent.contains("trident") || userAgent.contains("edge")) {
                    fileName = URLEncoder.encode(fileName, "UTF-8");
                } else {
                    fileName = new String(fileName.getBytes("utf-8"), "iso-8859-1");
                }
                response.setCharacterEncoding("utf-8");
                response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
                OutputStream outputStream = response.getOutputStream();
                content = (String)result.get("data");
                if (content != null) {
                    byte[] bytes = Base64.getDecoder().decode(content);
                    outputStream.write(bytes);
                    outputStream.close();
                }
            } catch (Exception e) {
                return new BocloudResult(false, "内部错误");
            }
        }
        return bocloudResult;
    }

    /**
     * ------------账单管理-账单概览--------------
     */
    @GetMapping("/getUserTop5") @ApiOperation(tags = {"CMC",
        "CSC"}, value = "用户消费TOP5", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult getUserTop5() {
        return billAnalysisService.getUserTop5();
    }
}
