package com.bocloud.cos.booter.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.cos.intf.ReportConfigTemplateService;

/**
 * 报表配置模板controller层
 *
 * @author lyy
 * @version 2.0
 * @time 2017年12月22日
 */
@Api(value = "报表配置模板", tags = "报表配置模板")
@RestController
@RequestMapping("/v1/templates")
public class ReportConfigTemplateController {

    @Autowired
    private ReportConfigTemplateService reportConfigTemplateService;

    /**
     * 报表模板列表展示
     *
     * @param pager
     * @param sorter
     * @param context
     * @return
     */
    @ApiOperation(tags={"CMC","CSC"},value = "查询报表模板列表", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult templates(Pager pager, @RequestParam(value = Common.SORTER, required = false) String sorter,
                                   @Value(Common.REQ_CONTEXT) RequestContext context) {
        return reportConfigTemplateService.list(pager.getPage(), pager.getRows(), pager.getParams(), pager.getSorter(),
                context);
    }

    /**
     * 删除报表模板
     *
     * @param id
     * @return
     */
    @ApiOperation(tags={"CMC","CSC"},value = "删除报表模板", response = BocloudResult.class, httpMethod = "DELETE", produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping(value = "/{id}")
    public BocloudResult remove(@PathVariable(value = Common.ID) Long id) {
        return reportConfigTemplateService.remove(id);
    }

    /**
     * 修改报表模板状态
     *
     * @param id
     * @param params
     * @param context
     * @return
     */
    @ApiOperation(tags={"CMC","CSC"},value = "修改报表模板状态", response = BocloudResult.class, httpMethod = "PUT", produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping(value = "/{id}")
    public BocloudResult modify(@PathVariable(value = Common.ID) Long id,
                                @RequestParam(value = Common.PARAMS, required = true) String params,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        JSONObject object = JSONTools.isJSONObj(params);
        if (null != object) {
            return reportConfigTemplateService.update(id, object.getBoolean("status"), context.getTarget());
        } else {
            return new BocloudResult(false, "参数格式不正确");
        }
    }

    /**
     * 新增报表配置模板
     *
     * @param params
     * @param context
     * @return
     */
    @ApiOperation(tags={"CMC","CSC"},value = "新增报表配置模板", response = BocloudResult.class, httpMethod = "POST", produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public BocloudResult create(@RequestParam(value = Common.PARAMS, required = true) String params,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        JSONObject object = JSONTools.isJSONObj(params);
        if (null != object) {
            // 获取基础信息
            JSONObject basics = object.getJSONObject("basics");
            // 获取目录信息
            JSONArray contents = object.getJSONArray("contents");
            // 获取订阅信息
            JSONObject subscription = object.getJSONObject("subscription");
            return reportConfigTemplateService.create(basics, contents, subscription, context);
        } else {
            return new BocloudResult(false, "参数格式不正确");
        }
    }

    /**
     * 查询报表模板详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags={"CMC","CSC"},value = "查询报表模板详情", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping(value = "/{id}")
    public BocloudResult detail(@PathVariable(value = Common.ID) Long id) {
        return reportConfigTemplateService.detail(id);
    }
}
