package com.bocloud.cos.booter.controller;

import com.alibaba.fastjson.JSONObject;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.ReportTemplateHistory;
import com.bocloud.cos.enums.ReportCategory;
import com.bocloud.cos.enums.ReportFileFormat;
import com.bocloud.cos.intf.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.export.ExporterInputItem;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.FastDateFormat;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author hufy
 */
@Api(value = "报表导出", tags = "报表导出")
@RestController
@RequestMapping("/v1/reports")
public class ReportDownloadController {

    @SuppressWarnings("unused")
    private static final Logger logger = Logger.getLogger(ReportDownloadController.class);

    @Autowired
    private PlatformInfoReportService platformInfoReportService;
    @Autowired
    private CloudServerReportService cloudServerReportService;
    @Autowired
    private ServerReportService serverReportService;
    @Autowired
    private StorageReportService storageReportService;
    @Autowired
    private ChargeReportService chargeReportService;
    @Autowired
    private MonitorReportService monitorReportService;
    @Autowired
    private TenantReportService tenantReportService;

    @Autowired
    private ReportTemplateHistoryService reportTemplateHistoryService;

    /**
     * 报表导出功能
     *
     * @param id
     * @param response
     * @param request
     * @param params
     * @return
     * @throws IOException
     */
    @ApiOperation(tags={"CMC","CSC"},value = "报表导出功能", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping(value = "/{id}")
    public BocloudResult reports(@PathVariable(value = Common.ID) Long id, HttpServletResponse response,
                                 HttpServletRequest request, @RequestParam(value = Common.PARAMS, required = false) String params,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) throws IOException {

        if (null == params) {
            return reportTemplateHistoryService.detail(id, context);
        } else {
            JSONObject jsonObject = JSONTools.isJSONObj(params);
            String fileType = jsonObject.getString("fileFormat");
            String name = DateTools.formatTime(new Date());
            // 获取文件名称
            BocloudResult detail = reportTemplateHistoryService.get(id);
            ReportTemplateHistory history = (ReportTemplateHistory) detail.getData();
            if (history != null) {
                name = history.getName();
            } else {
                return new BocloudResult(false, "查询失败");
            }
            if (null != jsonObject) {
                Map<String, Object> paramMap = MapTools.simpleMap("id", id);
                paramMap.put(Common.PARAMS, params);
                ReportFileFormat type = null;
                switch (fileType) {
                    case "application/pdf":
                        response.setContentType(fileType);
                        name = name + ".pdf";
                        type = ReportFileFormat.PDF;
                        break;
                    case "application/msword":
                        response.setContentType(fileType);
                        name = name + ".docx";
                        type = ReportFileFormat.WORD;
                        break;
                    case "application/vnd.ms-excel":
                        response.setContentType(fileType);
                        name = name + ".xlsx";
                        type = ReportFileFormat.EXCEL;
                        break;
                    default:
                }
                InputStream inputStream = null;
                OutputStream outputStream = null;
                try {
                    final String userAgent = request.getHeader("user-agent").toLowerCase();
                    if (userAgent.contains("trident") || userAgent.contains("edge")) {
                        name = URLEncoder.encode(name, "UTF-8");
                    } else {
                        name = new String(name.getBytes("utf-8"), "iso-8859-1");
                    }
                    response.setHeader("Content-disposition", "inline; filename=" + name);
                    responseFile(response, params, type, id);
                } catch (UnsupportedEncodingException e) {
                    logger.error("加密名称失败");
                }

            }
        }
        return null;
    }

    public void responseFile(HttpServletResponse response, String params, ReportFileFormat format, Long id)
            throws IOException {
        JSONObject jsonObj = JSONTools.isJSONObj(params);
        if (jsonObj != null) {
            @SuppressWarnings("unused")
            OutputStream outputStream = response.getOutputStream();
            String reportCategory = jsonObj.getString("reportCategory");
            ReportCategory reportCat = ReportCategory.valueOf(reportCategory);
            switch (reportCat) {
                case Platform:
                    List<ExporterInputItem> platList = platformInfoReportService.createReport(id);
                    platformInfoReportService.exportReport(platList, response.getOutputStream(), format);
                    break;
                case CloudServer:
                    List<ExporterInputItem> cloudList = cloudServerReportService.createReport(id);
                    cloudServerReportService.exportReport(cloudList, response.getOutputStream(), format);
                    break;
                case Server:
                    List<ExporterInputItem> serverList = serverReportService.createReport(id);
                    serverReportService.exportReport(serverList, response.getOutputStream(), format);
                    break;
                case Storage:
                    List<ExporterInputItem> storageList = storageReportService.createReport(id);
                    storageReportService.exportReport(storageList, response.getOutputStream(), format);
                    break;
                case Charge:
                    List<ExporterInputItem> chargeList = chargeReportService.createReport(id);
                    chargeReportService.exportReport(chargeList, response.getOutputStream(), format);
                    break;
                case Monitor:
                    List<ExporterInputItem> monitorList = monitorReportService.createReport(id);
                    monitorReportService.exportReport(monitorList, response.getOutputStream(), format);
                    break;
                case Tenant:
                    List<ExporterInputItem> tenantList = tenantReportService.createReport(id);
                    tenantReportService.exportReport(tenantList, response.getOutputStream(), format);
                    break;
                default:
                    break;
            }

        }
    }

    /**
     * This function is called when we need to download our PDF
     *
     * @throws IOException
     * @throws ClassNotFoundException
     */
    @ApiOperation(tags={"CMC","CSC"},value = "查询报表名称", response = BocloudResult.class, httpMethod = "GET", produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping(value = "/{id}/filename")
    public BocloudResult getFileName(@PathVariable("id") Long id, @Value(Common.REQ_CONTEXT) RequestContext context)
            throws JRException, IOException, ClassNotFoundException {
        BocloudResult detail = reportTemplateHistoryService.get(id);
        ReportTemplateHistory history = (ReportTemplateHistory) detail.getData();
        if (history != null) {
            return new BocloudResult(true, history.getName(), "查询成功");
        } else {
            return new BocloudResult(false, "查询失败");
        }

    }

    /**
     * This function is used to genearate name each time we d ownload PDF file.
     *
     * @return
     */
    public String generateDocName() {

        String fileName = StringUtils.EMPTY;
        String prefixFile = "PDF";
        FastDateFormat format = FastDateFormat.getInstance("_dd_MM_yyyy_hh_mm_ss");

        String currentDateAndTime = format.format(new Date());
        String fileExtension = ".pdf";
        fileName = prefixFile + currentDateAndTime + fileExtension;

        return fileName;

    }
}