package com.bocloud.cos.booter.listener;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.enums.*;
import com.bocloud.cos.intf.*;
import com.bocloud.cos.model.*;
import com.bocloud.cos.params.CloneParams;
import com.bocloud.cos.repository.*;
import com.bocloud.cos.service.internal.SmsInternalService;
import com.bocloud.cos.service.internal.SmsSystemConfigService;
import com.bocloud.cos.service.internal.SmsTenantService;
import com.bocloud.cos.service.resource.operate.AbstractIaasResourceOperate;
import com.bocloud.cos.service.resource.operate.AbstractPaasResourceOperate;
import com.bocloud.cos.service.resource.operate.AbstractResourceOperate;
import com.bocloud.cos.service.resource.operate.ResourceOperateUtils;
import com.bocloud.cos.utils.OrderItemUtils;
import com.bocloud.internel.entity.Tenant;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.*;

/**
 * 申请监听器
 *
 * @author weiwei
 */

/**
 * @author weiwei
 */
@Component("flowEventListener") public class FlowEventListener {

    Logger logger = LoggerFactory.getLogger(FlowEventListener.class);
    private final AmqpTemplate amqpTemplate;
    private final ResourceApplyService resourceApplyService;
    private final CosService cosService;
    private final RosService rosService;
    private final SmsSystemConfigService configService;
    private final SmsInternalService socService;
    private final ServiceOrderRepository orderRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final ServiceSkuRepository serviceSkuRepository;
    private final OrderItemRepository orderItemRepository;
    private final OrderSnService orderSnService;
    private final SmsTenantService tenantService;
    private final ServiceCategoryRepository serviceCategoryRepository;
    private final ResourceQuotaRepository resourceQuotaRepository;
    private final ResourceQuotaService resourceQuotaService;
    private final LockFactory lockFactory;

    public FlowEventListener(AmqpTemplate amqpTemplate, ResourceApplyService resourceApplyService,
        CosService cosService, RosService rosService, SmsSystemConfigService configService,
        SmsInternalService socService, ServiceOrderRepository orderRepository,
        CloudServiceRepository cloudServiceRepository, ServiceSkuRepository serviceSkuRepository,
        OrderItemRepository orderItemRepository, OrderSnService orderSnService, SmsTenantService tenantService,
        ServiceCategoryRepository serviceCategoryRepository, ResourceQuotaRepository resourceQuotaRepository,
        ResourceQuotaService resourceQuotaService, ServiceBillRepository billRepository,
        BillItemRepository itemRepository, LockFactory lockFactory) {
        this.amqpTemplate = amqpTemplate;
        this.resourceApplyService = resourceApplyService;
        this.cosService = cosService;
        this.rosService = rosService;
        this.configService = configService;
        this.socService = socService;
        this.orderRepository = orderRepository;
        this.cloudServiceRepository = cloudServiceRepository;
        this.serviceSkuRepository = serviceSkuRepository;
        this.orderItemRepository = orderItemRepository;
        this.orderSnService = orderSnService;
        this.tenantService = tenantService;
        this.serviceCategoryRepository = serviceCategoryRepository;
        this.resourceQuotaRepository = resourceQuotaRepository;
        this.resourceQuotaService = resourceQuotaService;
        this.lockFactory = lockFactory;
    }

    @RabbitListener(queues = "bocloud.flow.queue", admin = "rabbitAdmin", autoStartup = "true")
    public void onMessage(Message message) {
        try {
            String content = new String(message.getBody());
            if (!StringUtils.isNotEmpty(content)) {
                return;
            }
            logger.info("收到审核结果消息：{}", content);
            JSONObject object = JSONObject.parseObject(content, JSONObject.class);
            if (null == object || object.isEmpty()) {
                return;
            }
            String category = object.getString("ApplyCategory");
            // 如果消息是服务类则处理，不是则处理，消息返回mq
            if (ApplyCategory.ApplicationOperate.name().equalsIgnoreCase(category) || ApplyCategory.AlterationOperate
                .name().equalsIgnoreCase(category) || ApplyCategory.ExtensionOperate.name().equalsIgnoreCase(category)
                || ApplyCategory.UnsubscribeOperate.name().equalsIgnoreCase(category)
                || ApplyCategory.ResourceComputerClone.name().equalsIgnoreCase(category)) {
                logger.info("收到服务相关审核结果消息:" + content);
                String status = object.getString("ApplyStatus");
                operate(category, object, status);
            } else {
                logger.info("收到其他结果消息，发到其他机器");
                logger.info("This message category [{}] should not process in this service, reject it!", category);
                Object headerObject = message.getMessageProperties().getHeaders().get("retrytimes");
                Long retry = null == headerObject ? 0L : (Long)headerObject;
                if (retry < 20) {
                    message.getMessageProperties().setHeader("retrytimes", retry + 1);
                    message.getMessageProperties().setDelay(1000);
                    amqpTemplate.convertAndSend("bocloud.flow.queue", message);
                } else {
                    logger.warn("This message has retry for 20 times drop it!");
                }
            }
        } catch (Exception e) {
            logger.error("operate message error:", e);
        }
    }

    /**
     * 处理任务类审批消息
     *
     * @param category
     * @param object
     * @param status
     */
    private void operate(String category, JSONObject object, String status) {
        if (BocloudStatus.Service.APPROVED.name().equals(status)) {// 审批通过
            switch (ApplyCategory.valueOf(category)) {
                case ApplicationOperate:
                    ApplicationParam param =
                        JSONObject.parseObject(object.getString("ApplyParams"), ApplicationParam.class);
                    ServiceOrder applyOrder = JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    RequestContext applyContext =
                        JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    //处理审批后的订单逻辑
                    param.setContext(applyContext);
                    applyOrder.setApplyParam(JSONObject.toJSONString(param));
                    applyOrderHandle(applyOrder);
                    break;
                case AlterationOperate:
                    //变更审批后的参数
                    AlterationParam alteration =
                        JSONObject.parseObject(object.getString("ApplyParams"), AlterationParam.class);
                    ServiceOrder alterationOrder =
                        JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    RequestContext context =
                        JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    //处理审批后的订单逻辑
                    AlterationParam oldParams =
                        JSONObject.parseObject(alterationOrder.getApplyParam(), AlterationParam.class);
                    alteration.setContext(context);
                    alteration.setExpiredTime(oldParams.getExpiredTime());
                    alterationOrderHandle(alterationOrder, alteration);
                    break;
                case ExtensionOperate:
                    ExtensionParam extensionParam =
                        JSONObject.parseObject(object.getString("ApplyParams"), ExtensionParam.class);
                    ServiceOrder extensionOrder = JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    RequestContext extensionContext =
                        JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    //处理审批后的订单逻辑
                    extensionParam.setContext(extensionContext);
                    extensionOrderHandle(extensionOrder, extensionParam);
                    break;
                case UnsubscribeOperate:
                    CancelationParam cancelation =
                        JSONObject.parseObject(object.getString("ApplyParams"), CancelationParam.class);
                    RequestContext cancelationContext =
                        JSONObject.parseObject(object.getString("ApplyContext"), RequestContext.class);
                    Result cancelationResult = resourceApplyService.cancelation(cancelation, cancelationContext);
                    if (cancelationResult.isFailed()) {
                        NotifyMessage message =
                            new NotifyMessage(cancelationContext, "服务实例【" + cancelation.getInstanceName() + "】退订失败通知",
                                cancelationResult.getMessage(), true, false);
                        amqpTemplate.convertAndSend("event.notice.queue", message);
                    }
                    break;
                case ResourceComputerClone:
                    CloneParams config = JSONObject.parseObject(object.getString("ApplyParams"), CloneParams.class);
                    RequestContext cloneContext =
                        JSONObject.parseObject(object.getString("ApplyContext"), RequestContext.class);
                    Result cloneResult = resourceApplyService.clone(config, cloneContext);
                    if (cloneResult.isFailed()) {
                        NotifyMessage message =
                            new NotifyMessage(cloneContext, "服务实例【" + config.getInstanceName() + "】克隆失败通知",
                                cloneResult.getMessage(), true, false);
                        amqpTemplate.convertAndSend("event.notice.queue", message);
                    }
                    break;
                default:
                    break;
            }
        } else if (BocloudStatus.Service.UNRATIFIED.name().equals(status)) {// 审批不通过
            switch (ApplyCategory.valueOf(category)) {
                case ApplicationOperate:
                    ApplicationParam param =
                        JSONObject.parseObject(object.getString("ApplyParams"), ApplicationParam.class);
                    ServiceOrder order = JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    order.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                    orderRepository.update(order);
                    RequestContext context =
                        JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    List<OrderItem> items = orderItemRepository.listByOrderId(order.getId());
                    if (order.getServiceCode().startsWith("kvs.") || order.getServiceCode().startsWith("rds.") || order
                        .getServiceCode().startsWith("cms.")) {
                        //软件安装特殊处理 回调处理配额那边软件安装直回调一个消息处理
                        items.removeIf(
                            orderItem -> !(orderItem.getServiceCode().startsWith("kvs.") || orderItem.getServiceCode()
                                .startsWith("rds.") || orderItem.getServiceCode().startsWith("cms.")));
                    }
                    List<QuotaBean> quotas = param.getQuotas();
                    for (OrderItem item : items) {
                        OrderMessage applyMessage = new OrderMessage(item.getSn(), OrderMessage.Operation.Application,
                            OrderItemStatus.OpenedFailure, null, "资源申请失败：审批未通过", null, null, null,
                            param.getConfigs().getString("category"), context);
                        applyMessage.setQuotas(quotas);
                        amqpTemplate.convertAndSend("resource.result.queue", applyMessage);
                    }
                    break;
                case AlterationOperate:
                    AlterationParam alteration =
                        JSONObject.parseObject(object.getString("ApplyParams"), AlterationParam.class);
                    ServiceOrder alterOrder = JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    alterOrder.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                    orderRepository.update(alterOrder);
                    RequestContext alterationContext =
                        JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    AlterationParam oldParams =
                        JSONObject.parseObject(alterOrder.getApplyParam(), AlterationParam.class);
                    alteration.setExpiredTime(oldParams.getExpiredTime());
                    OrderMessage alterMessage =
                        new OrderMessage(alteration.getInstance(), OrderMessage.Operation.Upgrade,
                            OrderItemStatus.ChangeFailure, null, alteration.getInstanceName() + "变更失败：审批未通过", null,
                            null, null, alteration.getCategory(), alterationContext);
                    alterMessage.setNewSn(oldParams.getNewInstance());
                    alterMessage.setQuotas(oldParams.getQuotas());
                    amqpTemplate.convertAndSend("resource.result.queue", alterMessage);
                    List<OrderItem> alterItems = orderItemRepository.listByOrderId(alterOrder.getId());
                    for (OrderItem alterItem : alterItems) {
                        if (!alterItem.getSn().equals(oldParams.getNewInstance())) {
                            OrderMessage orderMessage =
                                new OrderMessage(alteration.getInstance(), OrderMessage.Operation.Upgrade,
                                    OrderItemStatus.ChangeFailure, null, alteration.getInstanceName() + "变更失败：审批未通过",
                                    null, null, null, alteration.getCategory(), alterationContext);
                            orderMessage.setNewSn(alterItem.getSn());
                            amqpTemplate.convertAndSend("resource.result.queue", orderMessage);
                        }
                    }
                    break;
                case ExtensionOperate:
                    ExtensionParam extension =
                        JSONObject.parseObject(object.getString("ApplyParams"), ExtensionParam.class);
                    ServiceOrder extensionOrder = JSONObject.parseObject(object.getString("order"), ServiceOrder.class);
                    RequestContext extensionContext =
                        JSONObject.parseObject(object.getString("requestContext"), RequestContext.class);
                    List<OrderItem> extensionItems = orderItemRepository.listByOrderId(extensionOrder.getId());
                    //批量更新订单项
                    extensionItems.forEach(orderItem -> {
                        orderItem.setStatus(OrderItemStatus.ExtensionFailure);
                        orderItem.setErrorMsg("延期失败：审批未通过");
                        orderItemRepository.update(orderItem);
                    });
                    extensionOrder.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                    orderRepository.update(extensionOrder);
                    NotifyMessage message =
                        new NotifyMessage(extensionContext, "服务实例【" + extension.getInstanceName() + "】延期失败通知", "审批未通过",
                            true, false);
                    amqpTemplate.convertAndSend("event.notice.queue", message);
                    break;
                default:
                    break;
            }
        }
    }

    private void extensionOrderHandle(ServiceOrder extensionOrder, ExtensionParam extensionParam) {
        try {
            ExtensionParam param = JSONObject.parseObject(extensionOrder.getApplyParam(), ExtensionParam.class);
            //延期月份
            Integer extension = extensionParam.getExtension();
            DurationModel.Mode mode = DurationModel.Mode.Month;
            param.setExtension(extensionParam.getExtension());
            List<OrderItem> orderItems = orderItemRepository.listByOrderId(extensionOrder.getId());
            for (OrderItem orderItem : orderItems) {
                DurationModel duration = JSONObject.parseObject(orderItem.getDuration(), DurationModel.class);
                DurationModel durationModel = new DurationModel();
                mode = duration.getMode();
                durationModel.setMode(duration.getMode());
                durationModel.setAmount(extension);
                orderItem.setDuration(JSONObject.toJSONString(durationModel));
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(orderItem.getEffectiveTime());
                if (duration.getMode().equals(DurationModel.Mode.Month)) {
                    calendar.add(Calendar.MONTH, extension);
                } else {
                    calendar.add(Calendar.YEAR, extension);
                }
                orderItem.setExpiredTime(calendar.getTime());
                orderItem.setStatus(OrderItemStatus.Extensioning);
                orderItemRepository.update(orderItem);
            }
            if (mode.equals(DurationModel.Mode.Year)) {
                param.setExtension(extensionParam.getExtension() * 12);
            }
            extensionOrder.setApplyParam(JSONObject.toJSONString(param));
            extensionOrder.setOrderStatus(ChargeCommon.OrderStatus.unpay);
            orderRepository.update(extensionOrder);
        } catch (Exception e) {
            logger.error("extension order handle error", e);
        }
    }

    private void alterationOrderHandle(ServiceOrder alterationOrder, AlterationParam params) {
        try {
            AbstractResourceOperate alterationOperate = ResourceOperateUtils
                .getAlterationOperate(params.getService(), params.getInstance(), params.getOperation(),
                    params.getUpgrade(), params.getExpiredTime(), cloudServiceRepository, serviceSkuRepository,
                    orderItemRepository, lockFactory, orderSnService, serviceCategoryRepository);
            //找之前待审批订单项 更新订单项
            List<OrderItem> auditItems = orderItemRepository.listAuditItems(alterationOrder.getId());
            auditItems.forEach(item -> {
                item.setStatus(OrderItemStatus.Canceld);
                item.setOrderId(null);
                orderItemRepository.update(item);
            });
            //生成新的订单项
            List<OrderItem> newAlterationItems = alterationOperate.buildAlterationItem();
            Tenant tenant = tenantService.detail(alterationOrder.getTenantId());
            Assert.notNull(tenant, "租户信息不存在");
            // 订单项设置折扣
            for (OrderItem newAlterationItem : newAlterationItems) {
                List<ElementModel> elementModelList =
                    JSONArray.parseArray(newAlterationItem.getElements(), ElementModel.class);
                elementModelList.forEach(elementModel -> elementModel.setRatio(tenant.getDiscount()));
                newAlterationItem.setElements(JSONObject.toJSONString(elementModelList));
                newAlterationItem.setFinalPrice(newAlterationItem.getBasicPrice().multiply(tenant.getDiscount()));
                newAlterationItem.setOrderId(alterationOrder.getId());
            }
            OrderItem oldItem = orderItemRepository.queryBySn(params.getInstance());
            //配额处理
            Map<String, Integer> alterationQuota = OrderItemUtils.getItemQuota(newAlterationItems);
            List<QuotaBean> quotaBeans = new ArrayList<>();
            Map<String, Integer> oldAlterationQuota = OrderItemUtils.getItemQuota(auditItems);
            Map<String, Integer> oldQuota;
            //软件安装特殊处理
            if (alterationOperate instanceof AbstractPaasResourceOperate) {
                ElementModel elementModel = params.getUpgrade().get(0);
                List<OrderItem> oldItems =
                    orderItemRepository.listByOrderId(oldItem.getOrderId(), elementModel.getServiceCode());
                oldQuota = OrderItemUtils.getItemQuota(oldItems);
            } else {
                oldQuota = OrderItemUtils.getItemQuota(Lists.newArrayList(oldItem));
            }
            for (Map.Entry<String, Integer> entry : alterationQuota.entrySet()) {
                String code = entry.getKey();
                //计算总的操做配额的量
                Integer oldNumber = Optional.ofNullable(oldQuota.get(code)).orElse(0);
                quotaBeans.add(new QuotaBean(code, alterationQuota.get(code) - oldNumber));
                Integer oldAlterationNumber = Optional.ofNullable(oldAlterationQuota.get(code)).orElse(0);
                alterationQuota.put(code, alterationQuota.get(code) - oldAlterationNumber);
            }
            Result result = handleQuota(alterationQuota, params.getContext().getTenant());
            //操作产生的配额
            params.setQuotas(quotaBeans);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            switch (params.getOperation()) {
                case Upgrade:
                    if (alterationOperate instanceof AbstractPaasResourceOperate) {
                        for (OrderItem alterationItem : newAlterationItems) {
                            List<ElementModel> elementModels =
                                JSONArray.parseArray(alterationItem.getElements(), ElementModel.class);
                            if ("vmware.storage.disk".equals(alterationItem.getServiceCode())) {
                                ElementModel diskElement = elementModels.get(0);
                                List<SpecModel> diskSpecModel = diskElement.getSpecs();
                                Integer disk =
                                    diskSpecModel.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0))
                                        .reduce(Integer::sum).orElse(0);
                                params.setDataDisk(disk);
                            } else {
                                ElementModel upgradeElement = elementModels.get(0);
                                List<SpecModel> specs = upgradeElement.getSpecs();
                                Integer cpu = specs.stream().map(spec -> Optional.ofNullable(spec.getCpu()).orElse(0))
                                    .reduce(Integer::sum).orElse(0);
                                BigDecimal memory = specs.stream()
                                    .map(spec -> Optional.ofNullable(spec.getMemory()).orElse(BigDecimal.ZERO))
                                    .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
                                Integer disk = specs.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0))
                                    .reduce(Integer::sum).orElse(0);
                                params.setUpgradeSkuId(upgradeElement.getSkuId());
                                params.setCpu(cpu);
                                params.setMemory(memory);
                                params.setSystemDisk(disk);
                            }
                            alterationItem.setElements(JSONObject.toJSONString(elementModels));
                            alterationItem.setFinalPrice(alterationItem.getBasicPrice().multiply(tenant.getDiscount()));
                            alterationItem.setOrderId(alterationOrder.getId());
                        }
                    } else {
                        // 获取升级规格 设置升级后的cpu内存大小到变更参数里面
                        OrderItem newAlterationItem = newAlterationItems.get(0);
                        List<ElementModel> elementModels =
                            JSONArray.parseArray(newAlterationItem.getElements(), ElementModel.class);
                        ElementModel element = elementModels.get(0);
                        List<SpecModel> specs = element.getSpecs();
                        Integer cpu = specs.stream().map(spec -> Optional.ofNullable(spec.getCpu()).orElse(0))
                            .reduce(Integer::sum).orElse(0);
                        BigDecimal memory =
                            specs.stream().map(spec -> Optional.ofNullable(spec.getMemory()).orElse(BigDecimal.ZERO))
                                .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
                        params.setUpgradeSkuId(element.getSkuId());
                        params.setCpu(cpu);
                        params.setMemory(memory);
                        Integer systemDisk = specs.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0))
                            .reduce(Integer::sum).orElse(0);
                        if (!systemDisk.equals(0)) {
                            params.setSystemDisk(systemDisk);
                        }
                    }
                    break;
                case EditDisk:
                    OrderItem editItem = newAlterationItems.get(0);
                    List<ElementModel> editDiskElements =
                        JSONArray.parseArray(editItem.getElements(), ElementModel.class);
                    editDiskElements.forEach(elementModel -> elementModel.setRatio(tenant.getDiscount()));
                    editItem.setElements(JSONObject.toJSONString(editDiskElements));
                    editItem.setFinalPrice(editItem.getBasicPrice().multiply(tenant.getDiscount()));
                    editItem.setOrderId(alterationOrder.getId());
                    break;
                case AddDisk:
                    int diskTotal = 0;
                    for (OrderItem alterationItem : newAlterationItems) {
                        List<ElementModel> diskElements =
                            JSONArray.parseArray(alterationItem.getElements(), ElementModel.class);
                        ElementModel diskElement = diskElements.get(0);
                        List<SpecModel> diskSpecModel = diskElement.getSpecs();
                        Integer disk = diskSpecModel.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0))
                            .reduce(Integer::sum).orElse(0);
                        diskTotal += disk;
                        // 订单项设置折扣
                        diskElements.forEach(elementModel -> elementModel.setRatio(tenant.getDiscount()));
                        alterationItem.setElements(JSONObject.toJSONString(diskElements));
                        alterationItem.setFinalPrice(alterationItem.getBasicPrice().multiply(tenant.getDiscount()));
                        alterationItem.setOrderId(alterationOrder.getId());
                    }
                    params.setDataDisk(diskTotal);
                default:
                    break;
            }
            params.setCategory(alterationOperate.getCategory().name());
            params.setOldStatus(oldItem.getStatus());
            // 保存新订单
            orderItemRepository.batchSave(newAlterationItems, newAlterationItems.size());
            params.setNewInstance(newAlterationItems.get(0).getSn());
            params.setOrderSn(alterationOrder.getSn());
            alterationOrder.setApplyParam(JSONObject.toJSONString(params));
            // 预付费生成未支付账单
            if (ChargeCommon.ChargeMode.PreCharge == alterationOrder.getChargeMode()) {
                alterationOrder.setOrderStatus(ChargeCommon.OrderStatus.unpay);
                orderRepository.update(alterationOrder);
            } else {
                alterationOrder.setOrderStatus(ChargeCommon.OrderStatus.paid);
                orderRepository.update(alterationOrder);
                if (alterationOperate instanceof AbstractPaasResourceOperate) {
                    amqpTemplate.convertAndSend("paas.resource.modification.queue", params);
                } else {
                    amqpTemplate.convertAndSend("iaas.resource.modification.queue", params);
                }
            }
        } catch (Exception e) {
            logger.error("alteration order handle error", e);
        }
    }

    /**
     * 申请订单审批处理(生成新订单项，配额更新)
     *
     * @param order
     */
    private void applyOrderHandle(ServiceOrder order) {
        try {
            ApplicationParam param = JSONObject.parseObject(order.getApplyParam(), ApplicationParam.class);
            AbstractResourceOperate applyOperate = ResourceOperateUtils
                .getApplyOperate(order.getServiceCode(), param.getEmption(), param.getElements(),
                    cloudServiceRepository, serviceSkuRepository, orderItemRepository, serviceCategoryRepository,
                    lockFactory, orderSnService);
            CloudService service = cloudServiceRepository.queryByCode(order.getServiceCode());
            //找待审批订单（有流程的话就有待审批订单，需要更新订单项）
            List<OrderItem> auditItems = orderItemRepository.listAuditItems(order.getId());
            if (!auditItems.isEmpty()) {
                Map<String, Integer> quotaMap = OrderItemUtils.getItemQuota(auditItems);
                //生成新订单项
                List<OrderItem> items = applyOperate.buildApplyItem();
                //回滚审批前订单配额
                Map<String, Integer> newQuota = OrderItemUtils.getItemQuota(items);
                for (Map.Entry<String, Integer> entry : newQuota.entrySet()) {
                    String code = entry.getKey();
                    Integer number = Optional.ofNullable(quotaMap.get(code)).orElse(0);
                    newQuota.put(code, newQuota.get(code) - number);
                }
                handleQuota(newQuota, order.getTenantId());
                Assert.isTrue(!items.isEmpty(), "生成订单内容错误");
                Tenant tenant = tenantService.detail(order.getTenantId());
                Assert.notNull(tenant, "租户信息不存在");
                if (!ListTool.isEmpty(items)) {
                    for (OrderItem item : items) {
                        item.setOrderId(order.getId());
                        List<ElementModel> elementModels = JSONArray.parseArray(item.getElements(), ElementModel.class);
                        if (elementModels.isEmpty()) {
                            ElementModel elementModel = new ElementModel();
                            elementModel.setRatio(tenant.getDiscount());
                            elementModels = Lists.newArrayList(elementModel);
                        } else {
                            elementModels.forEach(elementModel -> elementModel.setRatio(tenant.getDiscount()));
                        }
                        item.setElements(JSONObject.toJSONString(elementModels));
                        item.setFinalPrice(item.getBasicPrice().multiply(tenant.getDiscount()));
                        orderItemRepository.save(item);
                    }
                    auditItems.forEach(item -> {
                        item.setStatus(OrderItemStatus.Canceld);
                        item.setOrderId(null);
                        orderItemRepository.update(item);
                    });

                }
                order.setItems(items);
            }
            ApplicationParam applicationParam = JSONObject.parseObject(order.getApplyParam(), ApplicationParam.class);
            JSONObject configs = applicationParam.getConfigs();
            if (null == configs) {
                configs = new JSONObject();
            }
            Long skuId = null;
            ServiceSku sku = null;
            for (ElementModel element : applicationParam.getElements()) {
                if (service.getCode().equals(element.getServiceCode())) {
                    skuId = element.getSkuId();
                    sku = serviceSkuRepository.query(skuId);
                    break;
                }
            }
            ServiceCategory category = null;
            configs.put("skuId", skuId);
            if (null != sku) {
                configs.put("skuName", sku.getName());
                category = serviceCategoryRepository.query(sku.getCategoryId());
            }
            if (null != category) {
                configs.put("categoryId", category.getId());
                configs.put("categoryCode", category.getCode());
                configs.put("categoryProps", category.getProps());
            }
            configs.put("category", applyOperate.getCategory().name());
            if (StringUtils.isNotEmpty(service.getTemplateName())) {
                configs.put("templateName", service.getTemplateName());
            }
            applicationParam.setService(service.getCode());
            applicationParam.setConfigs(configs);
            if (ChargeCommon.ChargeMode.PostCharge == order.getChargeMode()) {
                order.setOrderStatus(ChargeCommon.OrderStatus.paid);
            } else {
                order.setOrderStatus(ChargeCommon.OrderStatus.unpay);
            }
            applicationParam.setOrder(order);
            //年计费下发到底层服务特殊处理
            EmptionModel emption = applicationParam.getEmption();
            DurationModel duration = emption.getDuration();
            DurationModel.Mode mode = duration.getMode();
            //下发到底层服务的时候按年计费时间做特殊处理
            if (mode == DurationModel.Mode.Year) {
                EmptionModel paramEmption = applicationParam.getEmption();
                DurationModel dura = paramEmption.getDuration();
                dura.setAmount(dura.getAmount() * 12);
                dura.setMode(DurationModel.Mode.Month);
                paramEmption.setDuration(dura);
                applicationParam.setEmption(paramEmption);
            }
            order.setApplyParam(JSONObject.toJSONString(applicationParam));
            //后付费
            if (ChargeCommon.ChargeMode.PostCharge == order.getChargeMode()) {
                order.setOrderStatus(ChargeCommon.OrderStatus.paid);
                orderRepository.update(order);
                if ("vmware.nsx.vpc".equalsIgnoreCase(service.getCode())) {
                    logger.info("后付费NSX-T Vpc申请参数结构：" + JSONObject.toJSONString(applicationParam));
                    amqpTemplate.convertAndSend("network.creation.queue", applicationParam);
                } else {
                    List<ApplicationParam> messageParams = Lists.newArrayList(applicationParam);
                    logger.info("后付费资源申请参数结构：" + JSONObject.toJSONString(messageParams));
                    if (applyOperate instanceof AbstractPaasResourceOperate) {
                        amqpTemplate.convertAndSend("paas.resource.creation.queue", messageParams);
                    } else if (applyOperate instanceof AbstractIaasResourceOperate) {
                        amqpTemplate.convertAndSend("iaas.resource.creation.queue", messageParams);
                    } else {
                        amqpTemplate.convertAndSend("iaas.resource.creation.queue", messageParams);
                    }
                }
            } else {
                order.setOrderStatus(ChargeCommon.OrderStatus.unpay);
                orderRepository.update(order);
            }
        } catch (Exception e) {
            logger.error("apply order handle error", e);
        }
    }

    /**
     * 调用boc
     * http://10.10.5.23:30022/boclouddeployerapi/cmpCluster/v2.4/createCluster
     *
     * @param callback
     * @param result
     * @return
     */
    private Result callBoc(String callback, JSONObject params, String result, String message, RequestContext context) {
        String token = socService.getToken(context.getTarget(), context.getCatalog().name());
        logger.info("获取token：{}", token);
        HttpClient httpClient = new HttpClient(20 * 1000 * 1000, PostDataFormat.RAW);
        try {
            params.put("result", result);
            params.put("callback", callback);
            params.put("message", message);
            Map<String, Object> header = new HashMap<>();
            header.put("accept", "application/json");
            header.put("content-type", "application/json");
            if (StringUtils.isNotEmpty(token)) {
                header.put("token", token);
            }
            String url = configService.getByCode("k8sAddress") + "/boclouddeployerapi/cmpCluster/v2.4/createVM";
            logger.info("调用boc地址：{}", url);
            logger.info("调用boc参数：{}", params.toJSONString());
            return httpClient.post(header, MapTools.simpleMap("vmInfo", params), url);
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return Result.FAILED("回调容器平台失败");
        } finally {
            httpClient.close();
        }
    }

    private Result callBoc(String callback, JSONObject params, String bocUrl, String result, String message,
        RequestContext context) {
        HttpClient httpClient = new HttpClient(20 * 1000 * 1000, PostDataFormat.RAW);
        String token = socService.getToken(context.getTarget(), context.getCatalog().name());
        logger.info("获取token：{}", token);
        try {
            params.put("callback", callback);
            params.put("result", result);
            params.put("message", message);
            Map<String, Object> header = new HashMap<>();
            header.put("accept", "application/json");
            header.put("content-type", "application/json");
            if (StringUtils.isNotEmpty(token)) {
                header.put("token", token);
            }
            String url = configService.getByCode("k8sAddress") + bocUrl;
            logger.info("调用boc地址：{}", url);
            logger.info("调用boc参数：{}", params.toJSONString());
            return httpClient.post(header, params, url);
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return Result.FAILED("回调容器平台失败");
        } finally {
            httpClient.close();
        }
    }

    private Result handleQuota(Map<String, Integer> quotaMap, Long tenantId) {
        Tenant tenant = tenantService.detail(tenantId);
        if (!tenant.getQuotaAble()) {
            return new Result(true, "租户未开启配额,不需要计算配额");
        }
        if (null == quotaMap || quotaMap.isEmpty()) {
            return new Result(true, "无配额信息需更新");
        }
        List<ResourceQuota> list = resourceQuotaRepository.listByTenantId(tenantId);
        Assert.notEmpty(list, "未查询到配额信息");
        Map<String, ResourceQuota> map = new HashMap<>(8);
        for (ResourceQuota quota : list) {
            map.put(quota.getCode(), quota);
        }
        List<ResourceQuota> quotas = Lists.newArrayList();
        for (Map.Entry<String, Integer> entry : quotaMap.entrySet()) {
            String code = entry.getKey();
            if (!"vmware.storage.disk".equals(code)) {
                String quotaName;
                if (QuotaCategory.CPU.getCode().equals(code)) {
                    quotaName = QuotaCategory.CPU.getName();
                } else if (QuotaCategory.MEM.getCode().equals(code)) {
                    quotaName = QuotaCategory.MEM.getName();
                } else if (QuotaCategory.DISK.getCode().equals(code)) {
                    quotaName = QuotaCategory.DISK.getName();
                } else {
                    CloudService cloudService = cloudServiceRepository.queryByCode(code);
                    quotaName = cloudService.getName();
                }
                if (!map.containsKey(entry.getKey())) {
                    return new Result(false, "未查询到" + quotaName + "配额信息");
                }
                ResourceQuota quota = map.get(entry.getKey());
                if (entry.getValue() + quota.getUsed() > quota.getQuota()) {
                    return new Result(false, "剩余" + quotaName + "配额不足");
                }
                quota.setQuotaActionValue(entry.getValue().longValue());
                quotas.add(quota);
            }
        }
        Result result = resourceQuotaService.quotaAction(quotas, ResourceQuota.Action.Order, tenantId);
        if (result.isFailed()) {
            return result;
        }
        return new Result(true, "更新配额信息成功");
    }

}
