package com.bocloud.cos.repository;

import com.bocloud.common.model.RequestContext;
import com.bocloud.cos.common.utils.ArrayFormatter;
import com.bocloud.cos.enums.ReportFrequenceType;
import com.bocloud.cos.model.ChargeInformationReport;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * @author lyy
 */
@Repository("chargeReportRepository")
public class ChargeReportRepository extends BasicGenericDao<ChargeInformationReport, Long> {

    public ChargeReportRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<ChargeInformationReport> queryChargeReport(String[] resourceIds,
                                                           ReportFrequenceType reportFrequenceType, Map<String, Object> param, RequestContext context)
            throws Exception {
        Long tenantId = context.getTenant();
        param.put("ids", ArrayFormatter.toLongList(resourceIds));
        if (0L != tenantId) {
            param.put("projectIds", ArrayFormatter.toLongList(resourceIds));
        }
        String timeClause = "";
        switch (reportFrequenceType) {
            case ByDay:
                timeClause = " and r.gmt_create >=:startTime and r.gmt_create <= :endTime ";
                break;
            case ByWeek:
                timeClause = " and DATE_FORMAT(r.gmt_create,'%Y-%m-%d') in (:dayList) ";
                break;
            case ByMonth:
                timeClause = " and r.gmt_create >=:startDate and r.gmt_create< :endDate ";
                break;
            default:
                break;
        }
        String sql = "select r.sn,roi.resource_category,r.category,r.gmt_create, "
                + " r.gmt_paid,r.status,p.name as project_name,u.name as user_name,t.name as tenant_name from resource_order r LEFT JOIN "
                + " resource_order_item roi ON r.id = roi.order_id "
                + " LEFT JOIN project p ON p.id = r.project_id LEFT JOIN `user` u ON r.user_id = u.id LEFT JOIN tenant t ON t.id = r.tenant_id "
                + " where r.is_deleted=0 and r.status = 'SUCCESS' " + timeClause + "and r.project_id in(:projectIds) ";
        return this.list(ChargeInformationReport.class, sql, param);
    }
}
