package com.bocloud.cos.repository;

import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.PortalPanel;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @author lyy
 */
@Service("portalPanelRepository")
public class PortalPanelRepository extends BasicGenericDao<PortalPanel, Long> {

    public PortalPanelRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<PortalPanel> listPortalPanel(String serverName) throws Exception {
        String sql = "select * from portal_panel where server_name = :serverName and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("serverName", serverName);
        return this.list(PortalPanel.class, sql, params);
    }

    public List<PortalPanel> list(String serverName, Long tenantId, Long projectId) throws Exception {
        String sql = "select * from portal_panel where server_name = :serverName and is_deleted = 0 ";
        Map<String, Object> params = MapTools.simpleMap("serverName", serverName);
        if (null != tenantId && 0L != tenantId) {
            sql += " and tenant_id=:tenantId ";
            params.put("tenantId", tenantId);
        }
        if (null != projectId && 0L != projectId) {
            sql += " and project_id=:projectId ";
            params.put("projectId", projectId);
        }
        return this.list(PortalPanel.class, sql, params);
    }


    public PortalPanel savePortalPanel(PortalPanel portalPanel) throws Exception {
        if (this.baseSave(portalPanel)) {
            return portalPanel;
        }
        return null;
    }

    public Boolean remove(Long id) throws Exception {
        String sql = "update portal_panel set is_deleted = 1 where id = :id and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        return this.execute(sql, params) > 0;
    }

    public PortalPanel query(Long id) throws Exception {
        String sql = "select * from portal_panel where is_deleted = 0 and id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<PortalPanel> portalPanels = this.list(PortalPanel.class, sql, paramMap);
        if (ListTool.isEmpty(portalPanels)) {
            return null;
        }
        return portalPanels.get(0);
    }

    public Boolean removeByServerName(String serverName, Long userId) throws Exception {
        String sql = "update portal_panel set is_deleted = 1,mender_id=:userId where server_name = :server_name and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("serverName", serverName);
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public Boolean batchSave(List<PortalPanel> list) throws Exception {
        return this.batchSave(list, 1000);
    }

    public List<PortalPanel> queryBackUpByServerName(String serverName) throws Exception {
        String sql = "select * from portal_panel_backup where server_name = :serverName";
        Map<String, Object> params = MapTools.simpleMap("serverName", serverName);
        return this.list(PortalPanel.class, sql, params);
    }

    public List<PortalPanel> queryByServerName(String serverName) throws Exception {
        String sql = "select * from portal_panel where server_name = :serverName";
        Map<String, Object> params = MapTools.simpleMap("serverName", serverName);
        return this.list(PortalPanel.class, sql, params);
    }

    public List<PortalPanel> queryByServerName(String serverName, Long tenantId) throws Exception {
        String sql = "select * from portal_panel where server_name = :serverName  and tenant_id=:tenantId";
        Map<String, Object> params = MapTools.simpleMap("serverName", serverName);
        params.put("tenantId", tenantId);
        return this.list(PortalPanel.class, sql, params);
    }

    public List<PortalPanel> queryDefaultSetting() throws Exception {
        String sql = "SELECT * FROM portal_panel WHERE is_deleted = 0 AND server_name = 'CSC' AND tenant_id = 0 ";
        return this.list(PortalPanel.class, sql, null);
    }

    public Boolean deteleByTenantId(Long tenantId, Long userId) throws Exception {
        String sql = "update portal_panel set is_deleted = 1,mender_id=:userId where tenant_id = :tenantId and is_deleted = 0 ";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }
}
