package com.bocloud.cos.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.ReportConfigTemplate;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * @author lyy
 */
@Repository("reportConfigTemplateRepository")
public class ReportConfigTemplateRepository extends BasicGenericDao<ReportConfigTemplate, Long> {

    public ReportConfigTemplateRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<ReportConfigTemplate> list(int page, int rows, List<Param> params, Map<String, String> sorter,
                                           RequestContext context, Boolean isAdmin) throws Exception {
        String sql = "SELECT a.*, b.name AS creater FROM rp_report_config_template a LEFT JOIN `user` b ON a.create_id = b.id WHERE a.is_deleted = 0 ";
        if (context.getTenant() == null || context.getTenant().equals(0L)) {
            sql += " and a.tenant_id =0 ";
        } else {
            if (isAdmin) {
                sql += " and a.tenant_id = :tenantId ";
            } else {
                sql += " and a.create_id = :userId ";
            }
        }
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> param = QueryBuilder.getParam(params);
        param.put("tenantId", context.getTenant());
        param.put("userId", context.getTarget());
        return this.list(ReportConfigTemplate.class, sql, param);
    }

    public ReportConfigTemplate saveReportConfigTemplate(ReportConfigTemplate reportConfigTemplate) throws Exception {
        if (this.baseSave(reportConfigTemplate)) {
            return reportConfigTemplate;
        }
        return null;
    }

    public Boolean remove(Long id) throws Exception {
        String sql = "update rp_report_config_template set is_deleted = 1 where id = :id and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        return this.execute(sql, params) > 0;
    }

    public int count(List<Param> params, RequestContext context, Boolean isAdmin) throws Exception {
        String sql = "select count(1) from rp_report_config_template where is_deleted = 0 ";
        if (context.getTenant() == null || context.getTenant().equals(0L)) {
            sql += " and (tenant_id =0 or tenant_id is null) ";
        } else {
            if (isAdmin) {
                sql += " and tenant_id = :tenantId ";
            } else {
                sql += " and create_id = :userId ";
            }
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "");
        Map<String, Object> param = QueryBuilder.getParam(params);
        param.put("tenantId", context.getTenant());
        param.put("userId", context.getTarget());
        return this.countQuery(sql, param).intValue();
    }

    public int countForSchedule() throws Exception {
        String sql = "select count(1) from rp_report_config_template ct join (select id from user where user.is_deleted=0 and user.status='NORMAL') u on ct.create_id = u.id where is_deleted = 0 and status=1 ";
        return this.countQuery(sql, null).intValue();
    }

    public List<ReportConfigTemplate> listForSchedule(int page, int rows) throws Exception {
        String sql = "SELECT a.*, b.name AS creater FROM rp_report_config_template a JOIN (select id,name from user where user.is_deleted=0 and user.status='NORMAL') b ON a.create_id = b.id WHERE a.is_deleted = 0 and a.status=1 ";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, null, null), "a");
        return this.list(ReportConfigTemplate.class, sql);
    }

    public ReportConfigTemplate query(Long id) throws Exception {
        String sql = "select * from rp_report_config_template where is_deleted = 0 and id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<ReportConfigTemplate> reportConfigTemplates = this.list(ReportConfigTemplate.class, sql, paramMap);
        if (ListTool.isEmpty(reportConfigTemplates)) {
            return null;
        }
        return reportConfigTemplates.get(0);
    }

    public ReportConfigTemplate detail(Long id) throws Exception {
        String sql = "SELECT a.*, b.name AS creater,c.`status` AS subscriptionStatus FROM rp_report_config_template a LEFT JOIN `user` b ON a.create_id = b.id LEFT JOIN rp_publish_info c ON a.id = config_template_id WHERE a.id =:id AND a.is_deleted = 0 ";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<ReportConfigTemplate> reportConfigTemplates = this.list(ReportConfigTemplate.class, sql, paramMap);
        if (ListTool.isEmpty(reportConfigTemplates)) {
            return null;
        }
        return reportConfigTemplates.get(0);
    }

    public ReportConfigTemplate queryByName(String name, Long tenantId) throws Exception {
        String sql = "select * from rp_report_config_template where is_deleted=0 and name=:name and tenant_id=:tenantId";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        paramMap.put("tenantId", tenantId);
        List<ReportConfigTemplate> objects = this.list(ReportConfigTemplate.class, sql, paramMap);
        if (!ListTool.isEmpty(objects)) {
            return objects.get(0);
        }
        return null;
    }

}
