package com.bocloud.cos.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.enums.ReportCategory;
import com.bocloud.cos.model.ReportResource;
import com.bocloud.database.core.BasicDao;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.bocloud.internel.entity.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * @author lyy
 */
@Repository("reportResourceDao")
public class ReportResourceRepository extends BasicGenericDao<ReportResource, Long> {

    public static final String PLATFORM = "Platform";
    public static final String STORAGE = "Storage";
    public static final String CHARGE = "Charge";
    public static final String TENANT = "Tenant";
    public static final String MONITOR = "Monitor";
    public static final String CLOUDSERVER = "CloudServer";
    @Autowired
    private BasicDao basicDao;

    public ReportResourceRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public int count(List<Param> params, String reportCategory, RequestContext context, Boolean isAdmin, Boolean isCmp,
                     List<Long> projectIds) throws Exception {
        String sql = null;
        Long userId = context.getTarget();
        Long tenantId = context.getTenant();
        ReportCategory category = ReportCategory.valueOf(reportCategory);
        Map<String, Object> param = QueryBuilder.getParam(params);
        switch (category) {
            case Platform:
                sql = "select count(1) from cloud_vendor where is_deleted = 0";
                break;
            case CloudServer:
                if (isCmp) {
                    sql = "select count(1) from cloud_server a where a.is_deleted = 0 and is_template = 0 ";
                } else {
                    String csName = null;
                    for (Param paramData : params) {
                        Map<String, Object> map = paramData.getParam();
                        for (String key : map.keySet()) {
                            if ("name".equals(key)) {
                                csName = String.valueOf(map.get(key));
                                break;
                            }
                        }
                    }
                    sql = "select count(1) from (select a.*,v.name vendor_name,b.expired_time,t.name tenant_name from cloud_server a left join tenant t on a.tenant_id = t.id left join cloud_vendor v on a.vendor_id = v.id,object_relation b where a.is_deleted = 0 AND a.is_template = 0 and a.id = b.res_id and b.category = 'Computer' and b.catalog = 'USER'  and a.project_id in (:projectIds)  AND a.is_recycle = '0') ct where 1=1 ";
                    if (null != csName) {
                        sql += "and ct.name like :csName ";
                        param.put("csName", "%" + csName + "%");
                    }
                }
                break;
            case Server:
                sql = "select count(1) from server where is_deleted = 0";
                break;
            case Storage:
                sql = "select count(1) from cluster where type='CEPH' AND is_deleted = 0";
                break;
            case Charge:
                sql = "SELECT count(1) from tenant where is_deleted = 0 ";
                if (!isCmp) {
                    sql = "SELECT COUNT(1) FROM project p WHERE is_deleted = 0 AND (p.tenant_id = :tenantId or p.id in(SELECT project_id FROM user_project WHERE tenant_id = :tenantId AND user_id = :userId)) ";
                }
                break;
            case Monitor:
                sql = "select count(1) from resource r where r.category='Computer' and r.is_deleted = 0 and r.catalog in ('PHYSICAL','LOGICAL') and r.private_ips != '[]'";
                break;
            case Tenant:
                sql = "SELECT count(1) FROM tenant WHERE is_deleted = 0";
                break;
            default:
                break;
        }
        if (!CLOUDSERVER.equals(reportCategory) && !CHARGE.equals(reportCategory)) {
            if (null == context.getTenant() || context.getTenant().equals(0L)) {
                if (PLATFORM.equals(reportCategory) || STORAGE.equals(reportCategory)
                        || TENANT.equals(reportCategory)) {
                    sql += "";
                } else {
                    sql += " and (tenant_id =0 or tenant_id is null) ";
                }
            } else {
                if (MONITOR.equals(reportCategory)) {
                    sql = "SELECT count(1) FROM(SELECT id,NAME,remark,gmt_create,target,category,catalog,private_ips,tenant_id FROM "
                            + " resource WHERE category = 'Computer' AND is_deleted = 0) r LEFT JOIN "
                            + " (SELECT res_id,target AS user_id FROM object_relation WHERE category = 'Computer') o "
                            + " ON r.target = o.res_id WHERE r.category = 'Computer' AND r.catalog IN ('PHYSICAL', 'LOGICAL') "
                            + " AND r.private_ips != '[]' ";
                }
                if (isAdmin) {
                    // 租户管理员
                    sql += " and tenant_id = :tenantId ";
                } else {
                    sql += " and user_id = :userId ";
                }
            }
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "");
        param.put("tenantId", tenantId);
        param.put("userId", userId);
        param.put("projectIds", projectIds);
        return this.countQuery(sql, param).intValue();
    }

    public List<ReportResource> list(Integer page, Integer rows, List<Param> params, Map<String, String> sorterMap,
                                     String reportCategory, RequestContext context, Boolean isAdmin, Boolean isCmp, List<Long> projectIds)
            throws Exception {
        String sql = null;
        Long userId = context.getTarget();
        Long tenantId = context.getTenant();
        Map<String, Object> param = QueryBuilder.getParam(params);
        ReportCategory reportCat = ReportCategory.valueOf(reportCategory);
        switch (reportCat) {
            case Platform:
                sql = "select id,name,remark from cloud_vendor where is_deleted=0";
                break;
            case CloudServer:
                if (isCmp) {
                    sql = "select a.id,a.name,a.remark from cloud_server a where a.is_deleted = 0 and is_template = 0 ";
                } else {
                    String csName = null;
                    Iterator<Param> iterator = params.iterator();
                    while (iterator.hasNext()) {
                        Param paramData = iterator.next();
                        Map<String, Object> map = paramData.getParam();
                        for (String key : map.keySet()) {
                            if ("name".equals(key)) {
                                csName = String.valueOf(map.get(key));
                                iterator.remove();
                                break;
                            }
                        }
                    }
                    sql = "select a.* from cloud_server a  left join region r on r.region_id = a.region_id and r.vendor_id = a.vendor_id  left join zone z on z.zone_id = a.zone and z.vendor_id = a.vendor_id  left join tenant t on a.tenant_id = t.id left join cloud_vendor v on a.vendor_id = v.id,object_relation b where a.is_deleted = 0 AND a.is_template = 0 and a.id = b.res_id and b.category = 'Computer' and b.catalog = 'USER'  and a.project_id in (:projectIds)  AND a.is_recycle = '0' ";
                    if (null != csName) {
                        sql += "AND a.`name` like :csName ";
                        param.put("csName", "%" + csName + "%");
                    }
                }
                break;
            case Server:
                sql = "select id,name,remark from server where is_deleted=0";
                break;
            case Storage:
                sql = "select id,name,remark from cluster where type='CEPH' AND is_deleted=0";
                break;
            case Charge:
                sql = "SELECT id,name,remark FROM tenant where is_deleted = 0 ";
                if (!isCmp) {
                    sql = "SELECT id,name,remark FROM project p WHERE is_deleted = 0 AND (p.tenant_id = :tenantId or p.id in(SELECT project_id FROM user_project WHERE tenant_id = :tenantId AND user_id = :userId)) ";
                }
                break;
            case Monitor:
                sql = "select id,name,remark,gmt_create,private_ips from resource r where r.category='Computer' and r.is_deleted = 0 and r.catalog in ('PHYSICAL','LOGICAL') and r.private_ips != '[]'";
                break;
            case Tenant:
                sql = "SELECT id,name,remark FROM tenant WHERE is_deleted = 0";
                break;
            default:
                break;
        }
        if (!CLOUDSERVER.equalsIgnoreCase(reportCategory) && !CHARGE.equals(reportCategory)) {
            if (null == context.getTenant() || context.getTenant().equals(0L)) {
                if (PLATFORM.equals(reportCategory) || STORAGE.equals(reportCategory)
                        || TENANT.equals(reportCategory)) {
                    sql += "";
                } else {
                    sql += " and (tenant_id =0 or tenant_id is null) ";
                }
            } else {
                if (MONITOR.equals(reportCategory)) {
                    sql = "SELECT id,name,remark,gmt_create,private_ips FROM(SELECT id,NAME,remark,gmt_create,target,category,catalog,private_ips,tenant_id FROM "
                            + " resource WHERE category = 'Computer' AND is_deleted = 0) r LEFT JOIN "
                            + " (SELECT res_id,target AS user_id FROM object_relation WHERE category = 'Computer') o "
                            + " ON r.target = o.res_id WHERE r.category = 'Computer' AND r.catalog IN ('PHYSICAL', 'LOGICAL') "
                            + " AND r.private_ips != '[]' ";
                }
                if (!CLOUDSERVER.equals(reportCategory)) {
                    if (isAdmin) {
                        // 租户管理员
                        sql += " and tenant_id = :tenantId ";
                    } else {
                        sql += " and user_id = :userId ";
                    }
                }
            }
        }
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorterMap), "");
        param.put("tenantId", tenantId);
        param.put("userId", userId);
        param.put("projectIds", projectIds);
        return this.list(ReportResource.class, sql, param);
    }

    public List<User> queryUsersByUserIds(List<Long> userIds) throws Exception {
        String sql = "select u.id,u.tenant_id,u.is_admin from user u where u.id in (:userIds)";
        Map<String, Object> paramMap = MapTools.simpleMap("userIds", userIds);
        List<Object> objects = this.basicDao.queryForList(sql, paramMap, User.class);
        List<User> users = new ArrayList<>();
        for (Object user : objects) {
            users.add((User) user);
        }
        return users;
    }

}
