package com.bocloud.cos.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.common.utils.DateFormatter;
import com.bocloud.cos.entity.ReportTemplateHistory;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author lyy
 */
@Repository("reportTemplateHistoryRepository")
public class ReportTemplateHistoryRepository extends BasicGenericDao<ReportTemplateHistory, Long> {

    public ReportTemplateHistoryRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<ReportTemplateHistory> list(int page, int rows, List<Param> params, Map<String, String> sorter)
            throws Exception {
        String sql = "select * from rp_report_template_history a where a.is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> param = QueryBuilder.getParam(params);
        return this.list(ReportTemplateHistory.class, sql, param);
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from rp_report_template_history where is_deleted = 0";
        sql = QueryBuilder.buildRaw(sql, params, null, "");
        Map<String, Object> param = QueryBuilder.getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public Boolean remove(Long id) throws Exception {
        String sql = "update rp_report_template_history set is_deleted = 1 where id = :id and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        return this.execute(sql, params) > 0;
    }

    public ReportTemplateHistory query(Long id) throws Exception {
        String sql = "select * from rp_report_template_history where is_deleted = 0 and id = :id";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<ReportTemplateHistory> templateToResolves = this.list(ReportTemplateHistory.class, sql, paramMap);
        if (ListTool.isEmpty(templateToResolves)) {
            return null;
        }
        return templateToResolves.get(0);
    }

    public Boolean removeByConfigTemplateId(Long configTemplateId) throws Exception {
        String sql = "update rp_report_template_history set is_deleted = 1 where config_template_id = :configTemplateId and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("configTemplateId", configTemplateId);
        return this.execute(sql, params) > 0;
    }

    public Boolean queryIfExist(Long configTemplateId, Date triggerDate) throws Exception {
        String sql = "select * from rp_report_template_history where config_template_id = :configId and DATE_FORMAT(trigger_time,'%Y-%m-%d') =:triggerTime";
        Map<String, Object> paramMap = MapTools.simpleMap("configId", configTemplateId);
        paramMap.put("triggerTime", DateFormatter.formatToString(triggerDate, "yyyy-MM-dd"));
        List<ReportTemplateHistory> result = this.list(ReportTemplateHistory.class, sql, paramMap);
        return result.size() > 0;
    }

    @Override
    public void save(ReportTemplateHistory reportTemplateHistory) {
        this.baseSave(reportTemplateHistory);
    }

}
