package com.bocloud.cos.repository;

import com.bocloud.cos.common.utils.ArrayFormatter;
import com.bocloud.cos.enums.ReportFrequenceType;
import com.bocloud.cos.model.ServerInformationReport;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * @author hufy
 */
@Repository("serverReportRepository")
public class ServerReportRepository extends BasicGenericDao<ServerInformationReport, Long> {

    public ServerReportRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }


    public List<ServerInformationReport> queryServerInformationReport(String[] resourceIds,
                                                                      ReportFrequenceType reportFrequenceType, Map<String, Object> param) throws Exception {
        param.put("ids", ArrayFormatter.toLongList(resourceIds));
        String sql = "SELECT s.`name`,s.manage_ip,s.`status`,r.roomName,r.rackName,s.category, "
                + " IFNULL(cs.cloudServerCount, 0) AS cloudServerCount,scg.cpu_amount,scg.memory_capacity,scg.disk_capacity "
                + " FROM `server` s LEFT JOIN (SELECT ra.id,ra.`name` AS rackName,ro.`name` AS roomName "
                + " FROM rack ra LEFT JOIN room ro ON ra.room_id = ro.id WHERE ra.is_deleted = 0 "
                + " ) r ON s.rack_id = r.id LEFT JOIN (SELECT s.id,COUNT(cs.host_id) AS cloudServerCount "
                + " FROM `server` s LEFT JOIN cloud_server cs ON s.id = cs.host_id WHERE cs.is_deleted = 0 "
                + " GROUP BY s.id) cs ON cs.id = s.id LEFT JOIN (SELECT s.id,sc.cpu_amount,sc.memory_capacity, "
                + " sc.disk_capacity FROM `server` s LEFT JOIN server_config sc ON s.id = sc.server_id WHERE s.is_deleted=0 "
                + " ) scg ON s.id = scg.id where s.id in (:ids) ";
        return this.list(ServerInformationReport.class, sql, param);
    }
}
