package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.common.ReportConstants;
import com.bocloud.cos.common.utils.DateFormatter;
import com.bocloud.cos.entity.ContentTemplate;
import com.bocloud.cos.entity.Paragraph;
import com.bocloud.cos.entity.ReportConfigTemplate;
import com.bocloud.cos.entity.ReportTemplateHistory;
import com.bocloud.cos.enums.ReportCategory;
import com.bocloud.cos.enums.ReportFileFormat;
import com.bocloud.cos.enums.ReportFrequenceType;
import com.bocloud.cos.intf.ReportCreateService;
import com.bocloud.cos.repository.*;
import com.bocloud.cos.service.jasper.handler.JasperHandler;
import com.bocloud.cos.service.jasper.utils.JasperReportUtils;
import net.sf.jasperreports.engine.*;
import net.sf.jasperreports.engine.data.JsonDataSource;
import net.sf.jasperreports.engine.query.JsonQueryExecuterFactory;
import net.sf.jasperreports.engine.util.JRLoader;
import net.sf.jasperreports.export.ExporterInputItem;
import net.sf.jasperreports.export.SimpleExporterInputItem;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.joda.time.DateTime;
import org.joda.time.DateTimeConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.apache.commons.lang3.StringUtils;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.util.*;

/**
 * @author hufy
 */
public abstract class AbstractReportService implements ReportCreateService {

    private static Logger logger = LoggerFactory.getLogger(AbstractReportService.class.getName());

    @Autowired
    protected ReportConfigTemplateRepository reportConfigTemplateRepository;

    @Autowired
    protected ParagraphTemplateRelaRepository paragraphTemplateRelaRepository;

    @Autowired
    protected ParagraphRepository paragraphRepository;

    @Autowired
    protected ContentTemplateRepository contentTemplateRepository;

    @Autowired
    protected ReportTemplateHistoryRepository reportTemplateHistoryRepository;

    @Autowired
    protected ReportResourceRepository reportResourceRepository;

    public void extractTimeParam(Date triggerTime, ReportFrequenceType reportFrequenceType,
                                 ReportConfigTemplate configTemplate, Map<String, Object> param, Boolean isRealTime) {
        String reportCat = configTemplate.getReportCategory();
        ReportCategory reportCategory = ReportCategory.valueOf(reportCat);
        // 主机，存储 没有时间
        switch (reportCategory) {
            case Server:
                return;
            case Storage:
                return;
            default:
                break;
        }

        switch (reportFrequenceType) {
            case ByDay:
                String[] startTimeArr = configTemplate.getStartTime().split(":");
                String[] endTimeArr = configTemplate.getEndTime().split(":");
                Integer startHour = Integer.parseInt(startTimeArr[0]);
                Integer startMinute = Integer.parseInt(startTimeArr[1]);
                Integer endHour = Integer.parseInt(endTimeArr[0]);
                Integer endMinute = Integer.parseInt(endTimeArr[1]);
                Date startTime = null;
                Date endTime = null;
                DateTime theDay = null;
                if (isRealTime) {
                    // 是否实时
                    theDay = DateTime.now();
                } else {
                    // 获取昨天
                    theDay = new DateTime(triggerTime.getTime()).plusDays(-1);
                }
                startTime = theDay.withHourOfDay(startHour).withMinuteOfHour(startMinute).toDate();
                endTime = theDay.withHourOfDay(endHour).withMinuteOfHour(endMinute).toDate();
                param.put("startTime", new java.sql.Timestamp(startTime.getTime()));
                param.put("endTime", new java.sql.Timestamp(endTime.getTime()));
                break;
            case ByWeek:
                List<String> dayList = new ArrayList<>();
                String weeks = configTemplate.getWeeks();
                DateTime theWeek = null;
                if (isRealTime) {
                    // 获取本周
                    theWeek = DateTime.now();
                } else {
                    // 获取上周的这个时间点
                    theWeek = new DateTime(triggerTime.getTime()).plusDays(-7);
                }
                List<String> split = JSONArray.parseArray(weeks, String.class);
                for (String week : split) {
                    int i = Integer.parseInt(week);
                    if (i == 0) {
                        // jodatime 的星期天是7
                        i = DateTimeConstants.SUNDAY;
                    }
                    DateTime dateTime = theWeek.withDayOfWeek(i);
                    dayList.add(DateFormatter.formatToString(dateTime.toDate(), "yyyy-MM-dd"));
                }
                param.put("dayList", dayList);
                break;
            case ByMonth:
                DateTime theMonth = null;
                if (isRealTime) {
                    theMonth = DateTime.now();
                } else {
                    // 获取上月
                    theMonth = new DateTime(triggerTime.getTime()).plusMonths(-1);
                }
                // 获取上个月的最后一天
                int maxDay = theMonth.dayOfMonth().withMaximumValue().dayOfMonth().get();
                int startDay = Integer.parseInt(configTemplate.getStartDate());
                int endDay = Integer.parseInt(configTemplate.getEndDate());
                if (startDay > maxDay) {
                    startDay = maxDay;
                }
                if (endDay > maxDay) {
                    endDay = maxDay;
                }
                java.sql.Timestamp startDate = new java.sql.Timestamp(
                        theMonth.withDayOfMonth(startDay).withHourOfDay(0).withMinuteOfHour(0).toDate().getTime());
                java.sql.Timestamp endDate = new java.sql.Timestamp(
                        theMonth.withDayOfMonth(endDay).withHourOfDay(0).withMinuteOfHour(0).toDate().getTime()
                                + 24 * 60 * 60 * 1000L);
                param.put("startDate", startDate);
                param.put("endDate", endDate);
                break;
            default:
                break;
        }
    }

    @SuppressWarnings("unchecked")
    @Override
    public List<ExporterInputItem> createReport(Long historyId) {
        ReportFrequenceType reportFrequenceType = null;
        ReportConfigTemplate configTemplate = null;
        ReportTemplateHistory history = null;

        try {
            history = reportTemplateHistoryRepository.query(historyId);
            configTemplate = reportConfigTemplateRepository.query(history.getConfigTemplateId());
            String cycleType = configTemplate.getCycleType();
            reportFrequenceType = ReportFrequenceType.valueOf(cycleType);
        } catch (Exception e) {
            logger.error(ExceptionUtils.getFullStackTrace(e));
            return Collections.emptyList();
        }
        // 如果构建时间
        Date buildTime = history.getTriggerTime();
        Map<String, Object> param = new HashMap<>(16);
        // 处理时间参数，监控报表不需要时间格式处理
        if (!"Monitor".equals(configTemplate.getReportCategory())) {
            extractTimeParam(buildTime, reportFrequenceType, configTemplate, param, false);
        }
        try {
            List<Paragraph> paragraphs = paragraphRepository.getByConfigTemplate(history.getConfigTemplateId());
            List<ExporterInputItem> printList = new ArrayList<>();
            // 设置标题
            String title = getTitleName();
            for (Paragraph paragraph : paragraphs) {
                // 处理表格
                String resourceIdStr = paragraph.getResourceIds();
                String[] resourceIds = resourceIdStr.split(",");
                // 先查询出表格数据
                // 此方法抽象出来了,放到子类去实现
                RequestUserTransformer requestUserTransformer = new RequestUserTransformer(configTemplate).invokeData();
                RequestContext context = requestUserTransformer.getContext();
                Boolean isAdmin = requestUserTransformer.getAdmin();
                @SuppressWarnings("rawtypes")
                List reports = getMainTableReport(reportFrequenceType, param, resourceIds, context, isAdmin,
                        configTemplate, paragraph, history);
                Map<String, Object> paramsMap = new HashMap<String, Object>(16);
                paramsMap.put(ReportConstants.TITLE, title);
                paramsMap.put(ReportConstants.SUB_TITLE, paragraph.getName());
                setReportDataSource(paramsMap, reports);
                // 配置数据源
                // output stream
                InputStream reportStream = getMainTableJasperStream();

                JasperReport jasperReport = (JasperReport) JRLoader.loadObject(reportStream);
                // 生成print
                JasperPrint print = JasperFillManager.fillReport(jasperReport, paramsMap);
                // 添加print
                printList.add(new SimpleExporterInputItem(print));
                // // 预览显示

                // 查询选择的图表模板数据
                List<ContentTemplate> contentTemplates = contentTemplateRepository
                        .queryContentByParagraphId(paragraph.getId());
                if (!ListTool.isEmpty(contentTemplates)) {
                    for (ContentTemplate contentTemplate : contentTemplates) {
                        String handlerName = contentTemplate.getHandlerName();
                        if (StringUtils.isNotEmpty(handlerName)) {
                            Class<?> handlerClass = Class
                                    .forName("com.bocloud.cos.service.jasper.handler.impl." + handlerName);
                            Object instance = handlerClass.getConstructor(ReportCreateService.class).newInstance(this);
                            JasperHandler jasperHandler = (JasperHandler) instance;
                            param.put(ReportConstants.CHART_NAME, contentTemplate.getName());
                            JasperPrint jasperPrint = jasperHandler.getJasperPrint(history, param, contentTemplate,
                                    reportFrequenceType, resourceIds, paragraph, configTemplate);
                            if (jasperPrint != null) {
                                printList.add(new SimpleExporterInputItem((jasperPrint)));
                            }
                        }
                    }
                }
            }
            return printList;
        } catch (Exception e) {
            logger.error(ExceptionUtils.getFullStackTrace(e));
        }
        return Collections.EMPTY_LIST;
    }

    @Override
    public void exportReport(List<ExporterInputItem> printList, OutputStream outputStream,
                             ReportFileFormat fileFormat) {
        if (printList == null || printList.size() == 0) {
            return;
        }
        switch (fileFormat) {
            case PDF:
                JasperReportUtils.exportReportListToPdf(printList, outputStream);
                break;
            case WORD:
                JasperReportUtils.exportReportListToRtf(printList, outputStream);
                break;
            case EXCEL:
                JasperReportUtils.exportReportListToExcel(printList, outputStream);
                break;
            default:
                break;
        }
    }

    /**
     * 获取japser流
     *
     * @return
     * @throws Exception
     */
    @Override
    public abstract InputStream getMainTableJasperStream() throws Exception;

    /**
     * 获取表格信息
     *
     * @param reportFrequenceType 报表周期类型 日报 周报 月报
     * @param param               查询条件的param
     * @param resourceIds         需要查询的资源id
     * @param isAdmin
     * @param configTemplate
     * @param paragraph
     * @param history
     * @return
     */
    @SuppressWarnings("rawtypes")
    @Override
    public abstract List getMainTableReport(ReportFrequenceType reportFrequenceType, Map<String, Object> param,
                                            String[] resourceIds, RequestContext context, Boolean isAdmin, ReportConfigTemplate configTemplate,
                                            Paragraph paragraph, ReportTemplateHistory history);

    private void setReportDataSource(Map<String, Object> params, List<Object> reportsList) {
        String jsonStr = null;
        if (!ListTool.isEmpty(reportsList)) {
            jsonStr = JSONObject.toJSONString(MapTools.simpleMap("result", reportsList),
                    SerializerFeature.WriteDateUseDateFormat);
        } else {
            jsonStr = "{}";
        }
        InputStream inputStream = null;
        try {
            inputStream = new ByteArrayInputStream(jsonStr.getBytes("utf-8"));
        } catch (UnsupportedEncodingException e) {
            logger.error("不支持的编码", e);
        }
        JsonDataSource jsonDataSource = null;
        try {
            jsonDataSource = new JsonDataSource(inputStream);
        } catch (JRException e) {
            logger.error("配置数据源失败", e);
        }
        params.put("REPORT_DATA_SOURCE", jsonDataSource);
        // 配置其他参数
        params.put(JsonQueryExecuterFactory.JSON_LOCALE, Locale.SIMPLIFIED_CHINESE);
        params.put(JRParameter.REPORT_LOCALE, Locale.CHINA);
    }

    private class RequestUserTransformer {
        private ReportConfigTemplate configTemplate;
        private RequestContext context;
        private Boolean isAdmin;

        public RequestUserTransformer(ReportConfigTemplate configTemplate) {
            this.configTemplate = configTemplate;
        }

        public RequestContext getContext() {
            return context;
        }

        public Boolean getAdmin() {
            return isAdmin;
        }

        public RequestUserTransformer invokeData() throws Exception {
            context = new RequestContext();
            context.setTenant(configTemplate.getTenantId());
            context.setTarget(configTemplate.getCreateId());
            isAdmin = Catalog.Tenant == context.getCatalog();
            return this;
        }
    }
}
