package com.bocloud.cos.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.enums.CloudProviderType;
import com.bocloud.cos.model.MonitorCategoryModel;
import com.bocloud.cos.model.MonitorModel;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.cos.common.utils.DateUtils;
import com.bocloud.cos.common.utils.MathUtils;
import com.bocloud.cos.entity.Paragraph;
import com.bocloud.cos.entity.ReportConfigTemplate;
import com.bocloud.cos.entity.ReportHistoryResult;
import com.bocloud.cos.entity.ReportTemplateHistory;
import com.bocloud.cos.enums.ReportFrequenceType;
import com.bocloud.cos.intf.MonitorReportService;
import com.bocloud.cos.model.MonitorInfoReport;
import com.bocloud.cos.model.ReportPair;
import com.bocloud.cos.repository.MonitorReportRepository;
import com.bocloud.cos.repository.ReportHistoryResultRepository;
import com.bocloud.cos.service.internal.CmsPrometheusService;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Author hufy
 */
@Service("monitorReportService")
public class MonitorReportServiceImpl extends AbstractReportService implements MonitorReportService {
	private static Logger logger = LoggerFactory.getLogger(MonitorReportServiceImpl.class);
	@Autowired
	private MonitorReportRepository monitorReportRepository;
	@Autowired
	private ReportHistoryResultRepository reportHistoryResultRepository;
	// @Autowired
	// private CmpService cmpService;
	@Autowired
	private CmsPrometheusService cmsPrometheusService;

	@Override
	public InputStream getMainTableJasperStream() throws Exception {
		return this.getClass().getClassLoader().getResourceAsStream("template/monitor/main.jasper");
	}

	@Override
	public String getTitleName() {
		return "监控报表";
	}

	@Override
	public List<MonitorInfoReport> getMainTableReport(ReportFrequenceType reportFrequenceType,
			Map<String, Object> param, String[] resourceIds, RequestContext context, Boolean isAdmin,
			ReportConfigTemplate configTemplate, Paragraph paragraph, ReportTemplateHistory history) {
		List<MonitorInfoReport> list = Collections.emptyList();
		try {
			/**
			 * 查询缓存的结果
			 */
			ReportHistoryResult cacheResult = reportHistoryResultRepository.query(history.getId(), paragraph.getId(),
					0L);
			if (cacheResult == null) {
				/**
				 * 缓存为空,计算数据，存入缓存
				 */
				list = monitorReportRepository.queryMonitorInfoReport(resourceIds, reportFrequenceType, param);

				// 增加查询监控数据
				appendMonitorData(list, reportFrequenceType, configTemplate);

				cacheResult = new ReportHistoryResult();
				String content = JSONObject.toJSONString(list);
				cacheResult.setContent(content);
				cacheResult.setHistoryId(history.getId());
				cacheResult.setContentTemplateId(0L);
				cacheResult.setParagraphId(paragraph.getId());
				reportHistoryResultRepository.save(cacheResult);
			} else {
				list = JSONArray.parseArray(cacheResult.getContent(), MonitorInfoReport.class);
			}
		} catch (Exception e) {
			logger.error(ExceptionUtils.getFullStackTrace(e));
		}
		return list;
	}

	/**
	 * 增加查询监控数据
	 *
	 * @param list
	 * @param reportFrequenceType
	 * @param configTemplate
	 */
	private void appendMonitorData(List<MonitorInfoReport> list, ReportFrequenceType reportFrequenceType,
			ReportConfigTemplate configTemplate) {
		// 处理时间格式
		long time = configTemplate.getGmtCreate().getTime();
		// String start = null;
		String end = null;
		String interval = null;
		SimpleDateFormat format = new SimpleDateFormat("dd");
		switch (reportFrequenceType) {
		case ByDay:
			interval = "1d";
			// DateTime startTimeByDay = new
			// DateTime(time).withHourOfDay(0).withMinuteOfHour(0).withSecondOfMinute(0);
			// start = String.valueOf(startTimeByDay.toDate().getTime());
			DateTime endTimeByDay = new DateTime(time).withDayOfMonth(new DateTime(time).getDayOfMonth() + 1)
					.withHourOfDay(0).withMinuteOfHour(0).withSecondOfMinute(0);
			end = String.valueOf(endTimeByDay.toDate().getTime() / 1000);
			break;
		case ByWeek:
			interval = "1w";
			int dayOfWeek = new DateTime(time).getDayOfWeek();
			int dayOfMonth = new DateTime(time).getDayOfMonth();
			// int startNumber = 0;
			int endNumer = 0;
			if (dayOfWeek - 1 != 0) {
				// startNumber = dayOfWeek - 1;
				endNumer = 7 - dayOfWeek;
			}
			Calendar calendarDate = Calendar.getInstance();
			calendarDate.setTime(configTemplate.getGmtCreate());
			calendarDate.set(Calendar.DAY_OF_MONTH, calendarDate.getActualMaximum(Calendar.DAY_OF_MONTH));
			int last = Long.valueOf(format.format(calendarDate.getTime())).intValue();
			DateTime endTimeByWeek = null;
			if (last < dayOfMonth + endNumer) {
				endTimeByWeek = new DateTime(time).withMonthOfYear(new DateTime(time).getMonthOfYear() + 1)
						.withDayOfMonth(dayOfMonth + endNumer - last + 1).withHourOfDay(0).withMinuteOfHour(0)
						.withSecondOfMinute(0);
			} else {
				endTimeByWeek = new DateTime(time).withDayOfMonth(dayOfMonth + endNumer + 1).withHourOfDay(0)
						.withMinuteOfHour(0).withSecondOfMinute(0);
			}
			// DateTime startTimeByWeek = new DateTime(time).withDayOfMonth(dayOfMonth -
			// startNumber).withHourOfDay(0)
			// .withMinuteOfHour(0).withSecondOfMinute(0);
			// start = String.valueOf(startTimeByWeek.toDate().getTime());

			end = String.valueOf(endTimeByWeek.toDate().getTime() / 1000);
			break;
		case ByMonth:
			// DateTime startTimeByMonth = new
			// DateTime(time).withDayOfMonth(1).withHourOfDay(0).withMinuteOfHour(0)
			// .withSecondOfMinute(0);
			// start = String.valueOf(startTimeByMonth.toDate().getTime());
			// 获取创建报表规则月最后一天
			Calendar calendar = Calendar.getInstance();
			calendar.setTime(configTemplate.getGmtCreate());
			calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMaximum(Calendar.DAY_OF_MONTH));
			int lastDay = Long.valueOf(format.format(calendar.getTime())).intValue();
			DateTime endTimeByMonth = new DateTime(time).withMonthOfYear(new DateTime(time).getMonthOfYear() + 1)
					.withDayOfMonth(1).withHourOfDay(0).withMinuteOfHour(0).withSecondOfMinute(0);
			end = String.valueOf(endTimeByMonth.toDate().getTime() / 1000);
			interval = lastDay + "d";
			break;
		default:
			break;
		}
		for (MonitorInfoReport monitorInfoReport : list) {
			String exprCpuAvg = null;
			String exprCpuMax = null;
			String exprMemAvg = null;
			String exprMemMax = null;
			String uuid = monitorInfoReport.getUuid();
			if (CloudProviderType.VMWARE.name().equals(monitorInfoReport.getVendorType())) {
				exprCpuAvg = "round(avg_over_time(vsphere_vm_cpu_usage_average{uuid = \"" + uuid + "\"}[" + interval
						+ "]), 0.01)";
				exprCpuMax = "round(max_over_time(vsphere_vm_cpu_usage_average{uuid = \"" + uuid + "\"}[" + interval
						+ "]), 0.01)";
				exprMemAvg = "round(avg_over_time(vsphere_vm_mem_usage_average{uuid = \"" + uuid + "\"}[" + interval
						+ "]), 0.01)";
				exprMemMax = "round(max_over_time(vsphere_vm_mem_usage_average{uuid = \"" + uuid + "\"}[" + interval
						+ "]), 0.01)";
			}
			String CpuAvgValue = cmsPrometheusService.query(exprCpuAvg, end);
			String CpuMaxValue = cmsPrometheusService.query(exprCpuMax, end);
			String MemAvgValue = cmsPrometheusService.query(exprMemAvg, end);
			String MemMaxValue = cmsPrometheusService.query(exprMemMax, end);
			monitorInfoReport.setCpuUsageAvg(CpuAvgValue);
			monitorInfoReport.setCpuUsageMax(CpuMaxValue);
			monitorInfoReport.setRamUsageAvg(MemAvgValue);
			monitorInfoReport.setRamUsageMax(MemMaxValue);
		}

	}

	// 通过json将数据转换为MonitorModel
	@SuppressWarnings("unused")
	private MonitorModel getMonitorModel(Object data) {
		String text;
		MonitorModel model = null;

		if (data != null) {
			text = JSON.toJSONString(data);
			try {
				model = JSON.parseObject(text, MonitorModel.class);
			} catch (JSONException e) {
				logger.error("监控数据解析失败", e);
				model = null;
			}
		}

		return model;
	}

	// 获取平均值
	@SuppressWarnings("unused")
	private String getAverageByMonitorModel(String type, MonitorModel model, String startTime, String endTime) {
		if (model == null) {
			return "";
		}

		List<String> keys = model.getKeys();
		List<MonitorCategoryModel> values = model.getValues();

		if (keys == null || keys.size() == 0 || values == null || values.size() == 0) {
			return "";
		}

		String timeStr;
		int fromIndex = -1, endIndex = -1;
		boolean isDate;

		int size = keys.size();
		for (int i = 0; i < size; i++) {
			timeStr = keys.get(i);
			isDate = DateUtils.isDate(timeStr, DateUtils.STANDARD_DATETIME_FORMAT);
			if (!isDate) {
				String msg = "监控数据错误,无法将[" + timeStr + "]不是时间格式的数据";
				logger.error(msg);
				continue;
			}
			if (timeStr.compareTo(startTime) < 0 || timeStr.compareTo(endTime) > 0) {
				continue;
			}
			if (fromIndex == -1) {
				fromIndex = i;
			}
			endIndex = i;
		}

		if (fromIndex == -1) {
			return "";
		}

		String value;
		double sum = 0;
		int count = 0;

		// CPU,MEM按天查询只有一条数据
		List<String> data = values.get(0).getData();
		if (keys.size() != data.size()) {
			String msg = "监控数据错误,keys的大小和data的大小不一致";
			logger.error(msg);
			throw new RuntimeException(msg);
		}

		for (int i = fromIndex; i <= endIndex; i++) {
			value = data.get(i);
			if (MathUtils.isNumber(value)) {
				count++;
				sum += Double.parseDouble(value);
			}
		}

		if (count == 0) {
			return "";
		}

		double average = MathUtils.round(sum / count, 2);
		return String.valueOf(average);
	}

	@Override
	public BocloudResult<List<ReportPair<List<MonitorInfoReport>>>> queryDetail(Long configTemplateId,
			ReportTemplateHistory history, Boolean isRealTime) {
		List<ReportPair<List<MonitorInfoReport>>> resultList = new ArrayList<>();
		ReportConfigTemplate configTemplate = null;
		ReportFrequenceType reportFrequenceType = null;
		Map<String, Object> param = new HashMap<>(16);
		try {
			if (isRealTime != null && isRealTime) {
				// 查看实时数据
				configTemplate = reportConfigTemplateRepository.query(configTemplateId);
				String cycleType = configTemplate.getCycleType();
				reportFrequenceType = ReportFrequenceType.valueOf(cycleType);
				// 处理时间参数
				extractTimeParam(null, reportFrequenceType, configTemplate, param, true);
			} else if (history != null) {
				// 查看历史数据
				configTemplateId = history.getConfigTemplateId();
				configTemplate = reportConfigTemplateRepository.query(configTemplateId);
				String cycleType = configTemplate.getCycleType();
				reportFrequenceType = ReportFrequenceType.valueOf(cycleType);
				// 处理时间参数
				if (!"Monitor".equals(configTemplate.getReportCategory())) {
					extractTimeParam(history.getTriggerTime(), reportFrequenceType, configTemplate, param, false);
				}
			}
			try {
				List<Paragraph> paragraphs = paragraphRepository.getByConfigTemplate(configTemplateId);
				for (Paragraph paragraph : paragraphs) {
					String resourceIdStr = paragraph.getResourceIds();
					String[] resourceIds = resourceIdStr.split(",");
					// 先查询出表格数据
					List<MonitorInfoReport> reports = getMainTableReport(reportFrequenceType, param, resourceIds, null,
							null, configTemplate, paragraph, history);
					resultList.add(new ReportPair<>(paragraph.getName(), reports));
				}
			} catch (Exception e) {
				logger.error(ExceptionUtils.getFullStackTrace(e));
				return new BocloudResult<>(false, "查询失败:"+e.getMessage());
			}
		} catch (Exception e) {
			logger.error(ExceptionUtils.getFullStackTrace(e));
			return new BocloudResult<>(false, "查询失败:"+e.getMessage());
		}
		return new BocloudResult<>(true, resultList, "查询成功");
	}

	public ReportHistoryResultRepository getReportHistoryResultRepository() {
		return reportHistoryResultRepository;
	}

}
