package com.bocloud.cos.service;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.cos.entity.PortalPanel;
import com.bocloud.cos.intf.PortalPanelService;
import com.bocloud.cos.repository.PortalPanelRepository;
import com.bocloud.cos.service.internal.SmsUserService;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.bocloud.internel.entity.User;
import com.bocloud.cos.model.PortalPanelBean;
import com.bocloud.cos.utils.ReportConstants;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author lyy
 */
@Service("portalPanelService")
public class PortalPanelServiceImpl implements PortalPanelService {
    private static final Logger logger = LoggerFactory.getLogger(PortalPanelServiceImpl.class);
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private PortalPanelRepository portalPanelRepository;
    @Autowired
    private SmsUserService smsUserService;

    @Override
    public BocloudResult list(String serverName, RequestContext context) {
        try {
            List<PortalPanel> portalPanels;
            if (ReportConstants.CMC.equals(serverName)) {
                portalPanels = this.portalPanelRepository.listPortalPanel(serverName);
            } else {
                BocloudResult<User> userResult = smsUserService.detail(context.getTarget());
                if (userResult.isFailed() || null == userResult.getData()) {
                    return new BocloudResult(false, "当前用户存在问题");
                }
                User loginUser = userResult.getData();
                portalPanels = this.portalPanelRepository.list(serverName, loginUser.getTenantId(), null);
            }
            return new BocloudResult(true, portalPanels, "查询成功");
        } catch (Exception e) {
            logger.error("Query Portal Panel error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(PortalPanel portalPanel, Long userId, Long tenantId) {
        try {
            portalPanel.setGmtCreate(new Date());
            portalPanel.setCreatorId(userId);
            portalPanel.setDeleted(false);
            portalPanel.setTenantId(tenantId);
            PortalPanel pane = this.portalPanelRepository.savePortalPanel(portalPanel);
            if (null == pane) {
                logger.warn("Create Portal Panel fail!");
                return new BocloudResult(false, "新增失败");
            }
            return new BocloudResult(true, "新增成功");
        } catch (Exception e) {
            logger.error("Create Portal Panel error:", e);
            return new BocloudResult(false, "新增失败");
        }
    }

    @Override
    public BocloudResult update(PortalPanelBean bean, Long userId) {
        String path = PortalPanel.class.getSimpleName() + "_" + bean.getId();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            PortalPanel panel = portalPanelRepository.query(bean.getId());
            if (null == panel) {
                logger.error("Portal Panel not exist");
                return new BocloudResult(false, "首页配置不存在");
            }
            BeanUtils.copyProperties(bean, panel);
            panel.setGmtModify(new Date());
            panel.setMenderId(userId);
            portalPanelRepository.update(panel);
            return new BocloudResult(true, "更新成功");
        } catch (Exception e) {
            logger.error("update portal Panel fail:", e);
            return new BocloudResult(false, "更新失败");
        }
    }

    @Override
    public BocloudResult remove(Long id) {
        String path = PortalPanel.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            PortalPanel panel = portalPanelRepository.query(id);
            if (null == panel) {
                logger.error("Portal Panel not exist");
                return new BocloudResult(false, "首页配置不存在");
            }
            this.portalPanelRepository.remove(id);
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Remove Portal Panel fail:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult reset(String serverName, Long userId) {
        String path = PortalPanel.class.getSimpleName() + "_" + serverName;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            List<PortalPanel> portalPanels = portalPanelRepository.queryBackUpByServerName(serverName);
            if (portalPanels.isEmpty()) {
                return new BocloudResult(false, "无出厂数据，无法恢复");
            }
            for (PortalPanel portalPanel : portalPanels) {
                portalPanel.setId(null);
                portalPanel.setCreatorId(userId);
                portalPanel.setGmtCreate(new Date());
            }
            portalPanelRepository.removeByServerName(serverName, userId);
            portalPanelRepository.batchSave(portalPanels);
            return new BocloudResult(true, "恢复成功");
        } catch (Exception e) {
            logger.error("Remove Portal Panel fail:", e);
            return new BocloudResult(false, "恢复失败");
        }
    }

    @Override
    public BocloudResult save(List<PortalPanel> portalPanels, String serverName, Long userId, Long tenantId,
                              Long projectId) {
        try {
            Map<Long, PortalPanel> old = Maps.newHashMap();
            List<PortalPanel> panels;
            if (ReportConstants.CMC.equals(serverName)) {
                panels = portalPanelRepository.queryByServerName(serverName);
            } else {
                panels = portalPanelRepository.queryByServerName(serverName, tenantId);
            }
            for (PortalPanel portalPanel : panels) {
                old.put(portalPanel.getId(), portalPanel);
            }
            for (PortalPanel portalPanel : portalPanels) {
                if (portalPanel.getId() == null) {
                    portalPanel.setCreatorId(userId);
                    portalPanel.setGmtCreate(new Date());
                    portalPanel.setDeleted(false);
                    portalPanel.setServerName(serverName);
                    portalPanel.setTenantId(tenantId);
                    portalPanel.setProjectId(projectId);
                    // 没有id 走新增
                    portalPanelRepository.save(portalPanel);
                    continue;
                }
                if (old.containsKey(portalPanel.getId())) {
                    PortalPanel oldPanel = old.get(portalPanel.getId());
                    oldPanel.setGmtModify(new Date());
                    oldPanel.setMenderId(userId);
                    oldPanel.setConfig(portalPanel.getConfig());
                    oldPanel.setLocation(portalPanel.getLocation());
                    portalPanelRepository.update(oldPanel);
                    old.remove(portalPanel.getId());
                }
            }
            for (PortalPanel portalPanel : old.values()) {
                portalPanelRepository.remove(portalPanel.getId());
            }
        } catch (Exception e) {
            return new BocloudResult(false, "保存失败");
        }

        return new BocloudResult(true, "保存成功");
    }

    @Override
    public BocloudResult setting(Long tenantId, Long userId) {
        // 查询默认展示数据
        try {
            List<PortalPanel> defaultSettingData = portalPanelRepository.queryDefaultSetting();
            for (PortalPanel portalPanel : defaultSettingData) {
                portalPanel.setId(null);
                portalPanel.setTenantId(tenantId);
                portalPanel.setCreatorId(userId);
                portalPanel.setGmtCreate(new Date());
            }
            Boolean batchSaveData = portalPanelRepository.batchSave(defaultSettingData);
            if (batchSaveData) {
                return new BocloudResult(true, "保存成功");
            } else {
                return new BocloudResult(false, "保存失败");
            }
        } catch (Exception e) {
            logger.error("Batch Save Portal Panel fail:", e);
            return new BocloudResult(false, "保存失败");
        }
    }

    @Override
    public BocloudResult deleteByTenantId(Long tenantId, Long userId) {
        String path = PortalPanel.class.getSimpleName() + "_removeByTenantId_" + tenantId;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            this.portalPanelRepository.deteleByTenantId(tenantId, userId);
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Remove Portal Panel fail:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

}
