package com.bocloud.cos.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.ListTool;
import com.bocloud.cos.enums.CloudProviderType;
import com.bocloud.cos.intf.ReportService;
import com.bocloud.cos.repository.ReportRepository;
import com.bocloud.cos.service.internal.CmsPrometheusService;
import com.bocloud.cos.service.utils.EmailSender;
import com.bocloud.cos.service.utils.ExcelChartUtil;
import com.bocloud.internel.entity.CloudServer;
import com.bocloud.internel.entity.User;
import com.csvreader.CsvWriter;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.nio.charset.Charset;
import java.util.*;

/**
 * @author lyy
 * @version 1.0
 * @date 2021/3/15 17:14
 */
@Service("reportService")
public class ReportServiceImpl implements ReportService {
    private static Logger logger = LoggerFactory.getLogger(ReportServiceImpl.class);
    private static SXSSFWorkbook workbook;
    @Autowired
    private ReportRepository reportRepository;
    @Autowired
    private CmsPrometheusService cmsPrometheusService;
    @Value(("${report.cache.file:/reports/}"))
    private String filePath;
    @Autowired
    private EmailSender emailSender;

    @Override
    public BocloudResult vmList(Pager pager, RequestContext context) {
        List<CloudServer> list;
        int page = pager.getPage();
        int rows = pager.getRows();
        List<Param> params = pager.getParams();
        Map<String, String> sorter = pager.getSorter();
        boolean simple = pager.getSimple();
        int total;
        GridBean gridBean;
        try {
            params = null == params ? new ArrayList<>() : params;
            sorter = null == sorter ? new HashMap<>() : sorter;
            sorter.put("gmtCreate", Common.ONE);
            total = reportRepository.vmList(1, Integer.MAX_VALUE, params, sorter).size();
            list = reportRepository.vmList(page, rows, params, sorter);
            gridBean = GridHelper.getBean(page, rows, total, list);
            return new BocloudResult(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult vmsDownload(Pager pager, RequestContext context) {
        CsvWriter csvWriter = null;
        try {
            List<Param> params = pager.getParams();
            Map<String, String> sorter = pager.getSorter();
            List<CloudServer> cloudServers = reportRepository.vmList(1, Integer.MAX_VALUE, null == params ? new ArrayList<>()
                    : params, null == sorter ? new HashMap<>() : sorter);
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            csvWriter = new CsvWriter(byteArrayOutputStream, ',', Charset.forName("GBK"));
            //标题
            String[] headers = {"虚拟机名称", "虚拟机类型", "IP地址", "配置信息", "租户", "项目", "申请人", "使用人", "所属平台"};
            csvWriter.writeRecord(headers);
            //内容
            for (CloudServer cloudServer : cloudServers) {
                List<String> content = Lists.newArrayList();
                content.add(cloudServer.getName());
                content.add(cloudServer.getVendorType());
                if (CloudProviderType.OPENSTACK.name().equalsIgnoreCase(cloudServer.getVendorType()) || "MANAGEONE".equalsIgnoreCase(cloudServer.getVendorType())) {
                    JSONArray privateIps = JSONArray.parseArray(cloudServer.getPrivateIps());
                    if (null != privateIps) {
                        String ipString = "";
                        List<JSONObject> objects = privateIps.toJavaList(JSONObject.class);
                        if (!ListTool.isEmpty(objects)) {
                            for (JSONObject object : objects) {
                                List<Object> addresses = object.getJSONArray("addresses");
                                for (Object address : addresses) {
                                    if (StringUtils.isEmpty(JSONObject.parseObject(JSONObject.toJSONString(address))
                                            .getString("address"))) {
                                        continue;
                                    }
                                    ipString += JSONObject.parseObject(JSONObject.toJSONString(address)).getString(
                                            "address")
                                            + ",";
                                }
                            }
                        }
                        if (ipString.length() > 0) {
                            ipString = ipString.substring(0, ipString.length() - 1);
                        }
                        content.add(ipString);
                    } else {
                        content.add("");
                    }
                } else if (CloudProviderType.TENCENT.name().equalsIgnoreCase(cloudServer.getVendorType())) {
                    String privateIps = cloudServer.getPrivateIps();
                    String privateIpString = "";
                    List<String> privateIpLists = JSONObject.parseArray(privateIps, String.class);
                    if (!ListTool.isEmpty(privateIpLists)) {
                        for (String object : privateIpLists) {
                            privateIpString += object + ",";
                        }
                        if (privateIpString.length() > 0) {
                            privateIpString = privateIpString.substring(0, privateIpString.length() - 1);
                        }
                    }
                    content.add(privateIpString);
                } else if (CloudProviderType.HUAWEI.name().equalsIgnoreCase(cloudServer.getVendorType())) {
                    String privateIps = cloudServer.getPrivateIps();
                    String ips = "";
                    List<JSONObject> parseArray = JSON.parseArray(privateIps, JSONObject.class);
                    if (null != parseArray && parseArray.size() > 0) {
                        for (JSONObject jsonObject : parseArray) {
                            String address = jsonObject.get("address").toString();
                            ips += address + ",";
                        }
                    }
                    content.add(ips);
                } else if (CloudProviderType.TIANYI.name().equalsIgnoreCase(cloudServer.getVendorType())) {
                    String privateIps = cloudServer.getPrivateIps();
                    String ips = "";
                    List<String> parseArray = JSON.parseArray(privateIps, String.class);
                    if (!ListTool.isEmpty(parseArray)) {
                        for (String ip : parseArray) {
                            ips += ip + ",";
                        }
                    }
                    if (ips.length() > 0) {
                        ips = ips.substring(0, ips.length() - 1);
                    }
                    content.add(ips);
                } else if (CloudProviderType.QCLOUD.name().equalsIgnoreCase(cloudServer.getVendorType())) {
                    String privateIps = cloudServer.getPrivateIps();
                    String privateIpString = "";
                    List<JSONObject> jSONObjectLists = JSONObject.parseArray(privateIps, JSONObject.class);
                    if (!ListTool.isEmpty(jSONObjectLists)) {
                        JSONObject jsonObject = jSONObjectLists.get(0);
                        if (null != jsonObject) {
                            privateIpString = jsonObject.getString("address");
                        }
                    }
                    content.add(privateIpString);
                } else if (CloudProviderType.VMWARE.name().equalsIgnoreCase(cloudServer.getVendorType())) {
                    String privateIps = cloudServer.getPrivateIps();
                    String ipString = "";
                    if (!StringUtils.isEmpty(privateIps)) {
                        List<JSONObject> objects = JSONObject.parseArray(privateIps, JSONObject.class);
                        for (JSONObject object : objects) {
                            String address = object.getString("address");
                            if (StringUtils.isEmpty(address)) {
                                continue;
                            }
                            ipString += address + ",";
                        }
                    }
                    if (ipString.length() > 0) {
                        ipString = ipString.substring(0, ipString.length() - 1);
                    }
                    content.add(ipString);
                } else if (CloudProviderType.AWS.name().equalsIgnoreCase(cloudServer.getVendorType())) {
                    String privateIps = cloudServer.getPrivateIps();
                    String privateIpString = "";
                    List<String> privateIpLists = JSONObject.parseArray(privateIps, String.class);
                    if (!ListTool.isEmpty(privateIpLists)) {
                        for (String object : privateIpLists) {
                            privateIpString += object + ",";
                        }
                        if (privateIpString.length() > 0) {
                            privateIpString = privateIpString.substring(0, privateIpString.length() - 1);
                        }
                    }
                    content.add(privateIpString);
                }

                StringBuilder specification = new StringBuilder();
                content.add(specification.append(cloudServer.getCpu()).append("C/").append(cloudServer.getMemory()).append("GB/")
                        .append(cloudServer.getDisk()).append("GB").toString());
                content.add(cloudServer.getTenantName());
                content.add(cloudServer.getProjectName());
                content.add(cloudServer.getTargetName());
                content.add(cloudServer.getOwnerName());
                content.add(cloudServer.getVendorName());
                csvWriter.writeRecord(content.toArray(new String[content.size()]));
            }
            csvWriter.close();
            byte[] ba = byteArrayOutputStream.toByteArray();
            Map<String, Object> resultMap = Maps.newHashMap();
            resultMap.put("fileName", "虚拟机清单报表" + System.currentTimeMillis() + ".csv");
            resultMap.put("data", Base64.getEncoder().encodeToString(ba));
            return new BocloudResult(true, resultMap, "下载成功");
        } catch (Exception e) {
            logger.error("虚拟机清单报表下载失败", e);
            return new BocloudResult(false, "虚拟机清单报表下载失败");
        } finally {
            if (null != csvWriter) {
                csvWriter.close();
            }
        }
    }

    @Override
    public BocloudResult sendVmPerformance() {
        try {
            //获取所有使用者的虚拟机
            List<CloudServer> cloudServers = reportRepository.list();
            //查询所有用户信息
            Map<Long, User> userMap = reportRepository.userList();
            //根据使用人分组
            Map<Long, List<CloudServer>> vmMap = Maps.newHashMap();
//            for (CloudServer cloudServer : cloudServers) {
//                String participators = cloudServer.getParticipators();
//                if (null != participators && !"".equals(participators)) {
//                    String[] userIds = participators.split(",");
//                    for (String userId : userIds) {
//                        if (vmMap.containsKey(Long.valueOf(userId))) {
//                            List<CloudServer> vms = vmMap.get(Long.valueOf(userId));
//                            vms.add(cloudServer);
//                            vmMap.put(Long.valueOf(userId), vms);
//                        } else {
//                            List<CloudServer> vms = Lists.newArrayList();
//                            vms.add(cloudServer);
//                            vmMap.put(Long.valueOf(userId), vms);
//                        }
//                    }
//                }
//            }
            List<File> files = Lists.newArrayList();
            for (Long id : vmMap.keySet()) {
                List<CloudServer> vms = vmMap.get(id);
                for (CloudServer cloudServer : vms) {
                    //获取监控数据
                    List<JSONObject> monitor = getMonitor(cloudServer.getName(), cloudServer.getHost(), cloudServer.getVendorType(), cloudServer.getInstanceId());
                    if (null == monitor || 0 == monitor.size()) {
                        logger.info("虚拟机[" + cloudServer.getName() + "]监控数据为空，已跳过！");
                        continue;
                    }
                    JSONArray privateIps = JSONArray.parseArray(cloudServer.getPrivateIps());
                    StringBuilder address = new StringBuilder();
                    String ipData = null;
                    if ("VMWARE".equalsIgnoreCase(cloudServer.getVendorType())) {
                        List<JSONObject> ips = privateIps.toJavaList(JSONObject.class);
                        for (JSONObject ip : ips) {
                            String ipString = ip.getString("address");
                            String networkName = ip.getString("networkName");
                            if (null != ipString) {
                                address.append(ipString).append("(").append(null != networkName ? networkName : "").append(")").append(",");
                            }
                        }
                        ipData = StringUtils.substringBeforeLast(address.toString(), ",");
                    } else {
                        List<String> ips = privateIps.toJavaList(String.class);
                        for (String ip : ips) {
                            address.append("(内网)").append(null != ip ? ip : "");
                            break;
                        }
                        ipData = address.toString();
                    }
                    //生成报表文件
                    SXSSFWorkbook excel = ExcelChartUtil.createLineChart(monitor, cloudServer.getName(), ipData);
                    File file = new File(filePath + cloudServer.getName() + System.currentTimeMillis() + ".xlsx");
                    if (!file.exists()) {
                        file.getParentFile().mkdirs();
                        file.createNewFile();
                    }
                    FileOutputStream fout = new FileOutputStream(file);
                    excel.write(fout);
                    fout.close();
                    excel.close();
                    files.add(file);
                }
                String[] emails = {userMap.get(id).getEmail()};
                //发生邮件
                emailSender.send("虚拟机性能周报", emails, "这是您的报表，请查收", files);
                //清除缓存文件
                delAllFile(filePath);
            }
            return null;
        } catch (Exception e) {
            logger.error("监控报表定时发送失败", e);
            delAllFile(filePath);
            return null;
        }
    }

    //删除指定文件夹下所有文件
    public static boolean delAllFile(String path) {
        File file = new File(path);
        if (!file.exists()) {
            return false;
        }
        if (!file.isDirectory()) {
            return false;
        }
        String[] tempList = file.list();
        File temp;
        for (int i = 0; i < tempList.length; i++) {
            if (path.endsWith(File.separator)) {
                temp = new File(path + tempList[i]);
            } else {
                temp = new File(path + File.separator + tempList[i]);
            }
            if (temp.isFile()) {
                temp.delete();
            }
        }
        return true;
    }

    //获取监控数据
    private List<JSONObject> getMonitor(String name, String hostName, String category, String instanceId) {
        List<JSONObject> result = Lists.newArrayList();
        //获取当前时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Long endTime = calendar.getTimeInMillis() / 1000;
        //获取上一周的时间节点
        Calendar todayStart = Calendar.getInstance();
        //把当前时间赋给日历
        todayStart.setTime(new Date());
        //设置为七天前
        todayStart.add(Calendar.DAY_OF_MONTH, -7);
        todayStart.set(Calendar.HOUR_OF_DAY, 0);
        todayStart.set(Calendar.MINUTE, 0);
        todayStart.set(Calendar.SECOND, 0);
        todayStart.set(Calendar.MILLISECOND, 0);
        Long startTime = todayStart.getTimeInMillis() / 1000;
        if ("VMWARE".equalsIgnoreCase(category)) {
            //cpu使用率
            JSONObject cpu = new JSONObject();
            cpu.put("step", 20);
            cpu.put("start", String.valueOf(startTime));
            cpu.put("end", String.valueOf(endTime));
            cpu.put("expr", "[\"round(vsphere_vm_cpu_usage_average{cpu=\\\"instance-total\\\",vmname=\\\"" + name + "\\\",esxhostname=\\\"" + hostName + "\\\"},0.01)\"]");
            cpu.put("name", "[\"cpu利用率\"]");
            cpu.put("unit", "%");
            BocloudResult cpuResult = cmsPrometheusService.chart(cpu);
            if (cpuResult.isFailed()) {
                logger.error(cpuResult.getMessage());
            } else {
                JSONObject cpuData = JSONObject.parseObject(String.valueOf(cpuResult.getData()));
                JSONArray cpuRows = cpuData.getJSONArray("rows");
                if (null != cpuRows && 0 != cpuRows.size()) {
                    result.add(cpuData);
                } else {
                    logger.info("虚拟机[" + name + "]cpu利用率获取监控数据为空，已跳过！");
                }
            }
            //内存使用率
            JSONObject mem = new JSONObject();
            mem.put("step", 20);
            mem.put("start", String.valueOf(startTime));
            mem.put("end", String.valueOf(endTime));
            mem.put("expr", "[\"round(vsphere_vm_mem_usage_average{vmname=\\\"" + name + "\\\",esxhostname=\\\"" + hostName + "\\\"},0.01)\"]");
            mem.put("name", "[\"内存利用率\"]");
            mem.put("unit", "%");
            BocloudResult memResult = cmsPrometheusService.chart(mem);
            if (memResult.isFailed()) {
                logger.error(memResult.getMessage());
            } else {
                JSONObject memData = JSONObject.parseObject(String.valueOf(memResult.getData()));
                JSONArray memRows = memData.getJSONArray("rows");
                if (null != memRows && 0 != memRows.size()) {
                    result.add(memData);
                } else {
                    logger.info("虚拟机[" + name + "]内存利用率获取监控数据为空，已跳过！");
                }
            }
            //磁盘读取/写入速率
            JSONObject disk = new JSONObject();
            disk.put("step", 20);
            disk.put("start", String.valueOf(startTime));
            disk.put("end", String.valueOf(endTime));
            disk.put("expr", "[\"round(vsphere_vm_disk_read_average{disk=\\\"instance-total\\\",vmname=\\\"" + name + "\\\",esxhostname=\\\"" + hostName + "\\\"},0.01)\",\"round(vsphere_vm_disk_write_average{disk=\\\"instance-total\\\",vmname=\\\"" + name + "\\\",esxhostname=\\\"" + hostName + "\\\"},0.01)\"]");
            disk.put("name", "[\"磁盘读取/写入速率\"]");
            disk.put("unit", "KB/s");
            BocloudResult diskResult = cmsPrometheusService.chart(disk);
            if (diskResult.isFailed()) {
                logger.error(diskResult.getMessage());
            } else {
                JSONObject diskData = JSONObject.parseObject(String.valueOf(memResult.getData()));
                JSONArray diskRows = diskData.getJSONArray("rows");
                if (null != diskRows && 0 != diskRows.size()) {
                    result.add(diskData);
                } else {
                    logger.info("虚拟机[" + name + "]磁盘使用速率获取监控数据为空，已跳过！");
                }
            }
            //网络使用速率
            JSONObject network = new JSONObject();
            network.put("step", 20);
            network.put("start", String.valueOf(startTime));
            network.put("end", String.valueOf(endTime));
            network.put("expr", "[\"round(vsphere_vm_disk_read_average{disk=\\\"instance-total\\\",vmname=\\\"" + name + "\\\",esxhostname=\\\"" + hostName + "\\\"},0.01)\",\"round(vsphere_vm_disk_write_average{disk=\\\"instance-total\\\",vmname=\\\"" + name + "\\\",esxhostname=\\\"" + hostName + "\\\"},0.01)\"]");
            network.put("name", "[\"网络使用速率\"]");
            network.put("unit", "KB/s");
            BocloudResult networkResult = cmsPrometheusService.chart(network);
            if (networkResult.isFailed()) {
                logger.error(networkResult.getMessage());
            } else {
                JSONObject networkData = JSONObject.parseObject(String.valueOf(networkResult.getData()));
                JSONArray networkRows = networkData.getJSONArray("rows");
                if (null != networkRows && 0 != networkRows.size()) {
                    result.add(networkData);
                } else {
                    logger.info("虚拟机[" + name + "]网络使用速率获取监控数据为空，已跳过！");
                }
            }
        } else if ("ALIYUN".equalsIgnoreCase(category)) {
            //cpu利用率
            JSONObject cpu = new JSONObject();
            cpu.put("step", 20);
            cpu.put("start", String.valueOf(startTime));
            cpu.put("end", String.valueOf(endTime));
            cpu.put("expr", "[\"round(aliyun_CPUUtilization{instance_id=\\\"" + instanceId + "\\\"}, 0.01)\"]");
            cpu.put("name", "[\"cpu利用率\"]");
            cpu.put("unit", "%");
            BocloudResult cpuResult = cmsPrometheusService.chart(cpu);
            if (cpuResult.isFailed()) {
                logger.error(cpuResult.getMessage());
            } else {
                JSONObject cpuData = JSONObject.parseObject(String.valueOf(cpuResult.getData()));
                if (null != cpuData.getJSONArray("keys") && 0 != cpuData.getJSONArray("keys").size()) {
                    result.add(cpuData);
                } else {
                    logger.info("虚拟机[" + name + "]cpu利用率获取监控数据为空，已跳过！");
                }
            }
            //内存使用率
            JSONObject mem = new JSONObject();
            mem.put("step", 20);
            mem.put("start", String.valueOf(startTime));
            mem.put("end", String.valueOf(endTime));
            mem.put("expr", "[\"round(aliyun_memory_usedutilization{instance_id=\\\"" + instanceId + "\\\"}, 0.01)\"]");
            mem.put("name", "[\"内存使用率\"]");
            mem.put("unit", "%");
            BocloudResult memResult = cmsPrometheusService.chart(mem);
            if (memResult.isFailed()) {
                logger.error(memResult.getMessage());
            } else {
                JSONObject memData = JSONObject.parseObject(String.valueOf(memResult.getData()));
                if (null != memData.getJSONArray("keys") && 0 != memData.getJSONArray("keys").size()) {
                    result.add(memData);
                } else {
                    logger.info("虚拟机[" + name + "]内存使用率获取监控数据为空，已跳过！");
                }
            }
            //磁盘使用率
            JSONObject disk = new JSONObject();
            disk.put("step", 20);
            disk.put("start", String.valueOf(startTime));
            disk.put("end", String.valueOf(endTime));
            disk.put("expr", "[\"round(aliyun_diskusage_utilization{instance_id=\\\"" + instanceId + "\\\"},0.01)\"]");
            disk.put("name", "[\"磁盘使用率\"]");
            disk.put("unit", "KB/s");
            BocloudResult diskResult = cmsPrometheusService.chart(disk);
            if (diskResult.isFailed()) {
                logger.error(diskResult.getMessage());
            } else {
                JSONObject diskData = JSONObject.parseObject(String.valueOf(diskResult.getData()));
                if (null != diskData.getJSONArray("keys") && 0 != diskData.getJSONArray("keys").size()) {
                    result.add(diskData);
                } else {
                    logger.info("虚拟机[" + name + "]磁盘使用速率获取监控数据为空，已跳过！");
                }
            }
            //内网流入宽带
            JSONObject intranetInput = new JSONObject();
            intranetInput.put("step", 20);
            intranetInput.put("start", String.valueOf(startTime));
            intranetInput.put("end", String.valueOf(endTime));
            intranetInput.put("expr", "[\"aliyun_IntranetInRate{instance_id=\\\"" + instanceId + "\\\"}\"]");
            intranetInput.put("name", "[\"内网流入宽带\"]");
            intranetInput.put("unit", "bit/s");
            BocloudResult intranetInputResult = cmsPrometheusService.chart(intranetInput);
            if (intranetInputResult.isFailed()) {
                logger.error(intranetInputResult.getMessage());
            } else {
                JSONObject intranetInputData = JSONObject.parseObject(String.valueOf(intranetInputResult.getData()));
                if (null != intranetInputData.getJSONArray("keys") && 0 != intranetInputData.getJSONArray("keys").size()) {
                    result.add(intranetInputData);
                } else {
                    logger.info("虚拟机[" + name + "]内网流入宽带获取监控数据为空，已跳过！");
                }
            }
            //内网流出宽带
            JSONObject intranetOutput = new JSONObject();
            intranetOutput.put("step", 20);
            intranetOutput.put("start", String.valueOf(startTime));
            intranetOutput.put("end", String.valueOf(endTime));
            intranetOutput.put("expr", "[\"aliyun_IntranetOutRate{instance_id=\\\"" + instanceId + "\\\"}\"]");
            intranetOutput.put("name", "[\"内网流出宽带\"]");
            intranetOutput.put("unit", "bit/s");
            BocloudResult intranetOutputResult = cmsPrometheusService.chart(intranetOutput);
            if (intranetOutputResult.isFailed()) {
                logger.error(intranetOutputResult.getMessage());
            } else {
                JSONObject intranetOutputData = JSONObject.parseObject(String.valueOf(intranetOutputResult.getData()));
                if (null != intranetOutputData.getJSONArray("keys") && 0 != intranetOutputData.getJSONArray("keys").size()) {
                    result.add(intranetOutputData);
                } else {
                    logger.info("虚拟机[" + name + "]内网流出宽带获取监控数据为空，已跳过！");
                }
            }
        } else if ("HUAWEI".equalsIgnoreCase(category)) {
            return null;
        } else {
            return null;
        }
        return result;
    }
}
