package com.bocloud.cos.service;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.cos.entity.ReportConfigTemplate;
import com.bocloud.cos.entity.ReportTemplateHistory;
import com.bocloud.cos.enums.ReportCategory;
import com.bocloud.cos.intf.*;
import com.bocloud.cos.model.*;
import com.bocloud.cos.repository.ReportConfigTemplateRepository;
import com.bocloud.cos.repository.ReportHistoryResultRepository;
import com.bocloud.cos.repository.ReportTemplateHistoryRepository;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Maps;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author lyy
 */
@Service("reportTemplateHistoryService")
public class ReportTemplateHistoryServiceImpl implements ReportTemplateHistoryService {
    private static final Logger logger = LoggerFactory.getLogger(ReportConfigTemplateServiceImpl.class);
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private ReportTemplateHistoryRepository reportTemplateHistoryRepository;
    @Autowired
    private PlatformInfoReportService platformInfoReportService;
    @Autowired
    private ReportConfigTemplateRepository reportConfigTemplateRepository;
    @Autowired
    private ReportHistoryResultRepository reportHistoryResultRepository;
    @Autowired
    private TenantReportService tenantReportService;
    @Autowired
    private ChargeReportService chargeReportService;
    @Autowired
    private CloudServerReportService cloudServerReportService;
    @Autowired
    private ServerReportService serverReportService;
    @Autowired
    private StorageReportService storageReportService;
    @Autowired
    private MonitorReportService monitorReportService;

    @Override
    public BocloudResult list(int page, int rows, List<Param> params, Map<String, String> sorter) {
        GridBean gridBean;
        try {
            if (null == params) {
                params = new ArrayList<>();
            }
            if (null == sorter) {
                sorter = Maps.newHashMap();
            }
            if (sorter.isEmpty()) {
                sorter = new HashMap<>(16);
                sorter.put("triggerTime", Common.ONE);
            }
            int count = this.reportTemplateHistoryRepository.count(params);
            List<ReportTemplateHistory> list = this.reportTemplateHistoryRepository.list(page, rows, params, sorter);
            gridBean = GridHelper.getBean(page, rows, count, list);
            return new BocloudResult(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult remove(Long id) {
        String path = ReportConfigTemplate.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ReportTemplateHistory resolve = reportTemplateHistoryRepository.query(id);
            if (null == resolve) {
                logger.error("Template To Resolve not exist");
                return new BocloudResult(false, "历史报表模板不存在");
            }
            // 删除历史报表模板
            reportTemplateHistoryRepository.remove(id);
            // 历史报表结果数据也删除
            reportHistoryResultRepository.delete(id);
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Remove Template To Resolve fail:", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult get(Long id) {
        ReportTemplateHistory result = null;
        try {
            result = reportTemplateHistoryRepository.query(id);
        } catch (Exception e) {
            logger.error(ExceptionUtils.getFullStackTrace(e));
        }
        if (result != null) {
            return new BocloudResult(true, result, "查询成功");
        } else {
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult detail(Long id, RequestContext context) {
        try {
            ReportTemplateHistory result = reportTemplateHistoryRepository.query(id);
            if (result == null) {
                return new BocloudResult(false, "找不到对应的记录");
            }
            Long configTemplateId = result.getConfigTemplateId();
            ReportConfigTemplate configTemplate = reportConfigTemplateRepository.query(configTemplateId);
            String reportCategoryStr = configTemplate.getReportCategory();

            ReportCategory reportCategory = ReportCategory.valueOf(reportCategoryStr);
            switch (reportCategory) {
                case Platform:
                    BocloudResult<List<ReportPair<List<PlatformInfoReport>>>> BocloudResult = platformInfoReportService
                            .queryDetail(null, result, false);
                    return new BocloudResult(BocloudResult.isSuccess(), BocloudResult.getData(), BocloudResult.getMessage());
                case Tenant:
                    BocloudResult<List<ReportPair<List<TenantInformationReport>>>> tenantResult = tenantReportService
                            .queryDetail(null, result, false);
                    return new BocloudResult(tenantResult.isSuccess(), tenantResult.getData(), tenantResult.getMessage());
                case Charge:
                    BocloudResult<List<ReportPair<List<ChargeInformationReport>>>> chargeResult = chargeReportService
                            .queryDetail(null, result, false, context);
                    return new BocloudResult(chargeResult.isSuccess(), chargeResult.getData(), chargeResult.getMessage());
                case CloudServer:
                    BocloudResult<List<ReportPair<List<CloudServerInfoReport>>>> cloudServerResult = cloudServerReportService
                            .queryDetail(null, result, false);
                    return new BocloudResult(cloudServerResult.isSuccess(), cloudServerResult.getData(),
                            cloudServerResult.getMessage());
                case Server:
                    BocloudResult<List<ReportPair<List<ServerInformationReport>>>> serverResult = serverReportService
                            .queryDetail(null, result, false);
                    return new BocloudResult(serverResult.isSuccess(), serverResult.getData(), serverResult.getMessage());
                case Storage:
                    BocloudResult<List<ReportPair<List<StorageInformationReport>>>> storageResult = storageReportService
                            .queryDetail(null, result, false);
                    return new BocloudResult(storageResult.isSuccess(), storageResult.getData(),
                            storageResult.getMessage());

                case Monitor:
                    BocloudResult<List<ReportPair<List<MonitorInfoReport>>>> monitorResult = monitorReportService
                            .queryDetail(null, result, false);
                    return new BocloudResult(monitorResult.isSuccess(), monitorResult.getData(),
                            monitorResult.getMessage());
                default:
                    break;
            }

        } catch (Exception e) {
            logger.error(ExceptionUtils.getFullStackTrace(e));
        }
        return new BocloudResult(false, "查询失败");
    }

}
