package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.AESEncrptor;
import com.bocloud.common.encrypt.Encryptor;
import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import com.bocloud.cos.engine.FlowEngine;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.enums.*;
import com.bocloud.cos.enums.ChargeCommon.ChargeMode;
import com.bocloud.cos.intf.*;
import com.bocloud.cos.model.*;
import com.bocloud.cos.model.DurationModel.Mode;
import com.bocloud.cos.params.CloneParams;
import com.bocloud.cos.repository.*;
import com.bocloud.cos.service.internal.ImsInternalService;
import com.bocloud.cos.service.internal.SmsTenantService;
import com.bocloud.cos.service.resource.operate.*;
import com.bocloud.cos.service.utils.AlterationModel;
import com.bocloud.cos.service.utils.ApplicationModel;
import com.bocloud.cos.service.utils.IdGenerator;
import com.bocloud.cos.utils.OrderItemUtils;
import com.bocloud.internel.entity.*;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.*;
import java.util.Map.Entry;
import java.util.concurrent.TimeUnit;

/**
 * 服务操作业务实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年4月15日
 */
@Service("resourceApplyService") public class ResourceApplyServiceImpl implements ResourceApplyService {

    private static final Logger logger = LoggerFactory.getLogger(ResourceApplyServiceImpl.class);
    private final ServiceOrderRepository serviceOrderRepository;
    private final AmqpTemplate amqpTemplate;
    private final OrderItemRepository orderItemRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final ServiceCategoryRepository serviceCategoryRepository;
    private final ServiceSkuRepository serviceSkuRepository;
    private final ServiceSlaRepository serviceSlaRepository;
    private final SlaInstanceRepository slaInstanceRepository;
    private final ResourceQuotaRepository resourceQuotaRepository;
    private final SmsTenantService tenantService;
    private final ResourceQuotaService resourceQuotaService;
    private final ServiceBillRepository billRepository;
    private final BillItemRepository itemRepository;
    private final ImsInternalService imsInternalService;
    private final OrderSnService orderSnService;
    private final CosService cosService;
    private final RosService rosService;
    private final LockFactory lockFactory;
    private final TenantAccountRepository accountRepository;
    private final FlowEngine engine;
    private final ServiceFlowConfigRepository serviceFlowConfigRepository;

    public ResourceApplyServiceImpl(OrderItemRepository orderItemRepository,
        ServiceOrderRepository serviceOrderRepository, ResourceQuotaService resourceQuotaService,
        ImsInternalService imsInternalService, AmqpTemplate amqpTemplate, CloudServiceRepository cloudServiceRepository,
        ServiceCategoryRepository serviceCategoryRepository, ServiceSkuRepository serviceSkuRepository,
        CosService cosService, ServiceSlaRepository serviceSlaRepository, BillItemRepository itemRepository,
        ServiceBillRepository billRepository, RosService rosService, SlaInstanceRepository slaInstanceRepository,
        OrderSnService orderSnService, LockFactory lockFactory, ResourceQuotaRepository resourceQuotaRepository,
        SmsTenantService tenantService, TenantAccountRepository accountRepository, FlowEngine engine,
        ServiceFlowConfigRepository serviceFlowConfigRepository) {
        this.orderItemRepository = orderItemRepository;
        this.serviceOrderRepository = serviceOrderRepository;
        this.resourceQuotaService = resourceQuotaService;
        this.imsInternalService = imsInternalService;
        this.amqpTemplate = amqpTemplate;
        this.cloudServiceRepository = cloudServiceRepository;
        this.serviceCategoryRepository = serviceCategoryRepository;
        this.serviceSkuRepository = serviceSkuRepository;
        this.cosService = cosService;
        this.serviceSlaRepository = serviceSlaRepository;
        this.itemRepository = itemRepository;
        this.billRepository = billRepository;
        this.rosService = rosService;
        this.slaInstanceRepository = slaInstanceRepository;
        this.orderSnService = orderSnService;
        this.lockFactory = lockFactory;
        this.resourceQuotaRepository = resourceQuotaRepository;
        this.tenantService = tenantService;
        this.accountRepository = accountRepository;
        this.engine = engine;
        this.serviceFlowConfigRepository = serviceFlowConfigRepository;
    }

    @Override public BocloudResult application(ApplicationParam applicationParam, RequestContext context) {
        logger.info("接收到订购消息：{}", JSONObject.toJSON(applicationParam));
        // 申请校验规格 校验服务是否存在等
        BocloudResult<ApplicationModel> result = applyCheck(applicationParam, context, false);
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        ApplicationModel model = result.getData();
        CloudService service = model.getService();
        Long slaId = model.getSlaId();
        ServiceSla sla = model.getSla();
        // 获取资源操作对象
        AbstractResourceOperate operate = model.getOperate();
        Long skuId = model.getSkuId();
        ServiceCategory category = model.getCategory();
        ServiceSku sku = model.getSku();
        try {
            EmptionModel emption = applicationParam.getEmption();
            DurationModel duration = emption.getDuration();
            Mode mode = duration.getMode();
            ChargeMode chargeMode = ChargeCommon.ChargeMode.PostCharge;
            if (mode != Mode.Hour) {
                chargeMode = ChargeCommon.ChargeMode.PreCharge;
            }
            // 下单
            ServiceOrder order = new ServiceOrder();
            order.setServiceCode(service.getCode());
            // 生成订单号，增加分布式锁
            String path = IdGenerator.class.getSimpleName() + "_ordersn";
            try (BocloudACLock lock = lockFactory.getACLock(path)) {
                Assert.isTrue(lock.acquire(1, TimeUnit.MINUTES), "生成订单号请求超时");
                order.setSn(IdGenerator.getInstance().nextId());
            } catch (Exception e) {
                logger.error("Get error message：{}", e.getMessage());
                return new BocloudResult<>(false, "资源申请失败 : " + e.getMessage());
            }
            order.setChargeMode(chargeMode);
            order.setSlaAgreed(null != slaId && 0L != slaId);
            if (ChargeMode.PostCharge.equals(chargeMode)) {
                order.setOrderStatus(ChargeCommon.OrderStatus.paid);
            } else {
                order.setOrderStatus(ChargeCommon.OrderStatus.unpay);
            }
            order.setType(ChargeCommon.Type.apply);
            order.setGmtCreate(new Date());
            order.setAutoRenew(false);
            LocationModel location = applicationParam.getLocation();
            // 设置订单归属
            order.setVendorId(location.getVendorId());
            order.setBusinessId(location.getBusinessId());
            order.setProjectId(location.getProjectId());
            order.setUserId(location.getOwnerId());
            order.setTenantId(context.getTenant());
            order.setResourceStatus(ChargeCommon.ResourceStatus.creating);
            serviceOrderRepository.save(order);
            // 生成新的订单项
            List<OrderItem> items = model.getOrderItems();
            order.setItems(items);
            Tenant tenant = tenantService.detail(order.getTenantId());
            if (!ListTool.isEmpty(items)) {
                for (OrderItem item : items) {
                    item.setOrderId(order.getId());
                    List<ElementModel> elementModels = JSONArray.parseArray(item.getElements(), ElementModel.class);
                    if (elementModels.isEmpty()) {
                        ElementModel elementModel = new ElementModel();
                        elementModel.setRatio(tenant.getDiscount());
                        elementModels = Lists.newArrayList(elementModel);
                    } else {
                        elementModels.forEach(elementModel -> elementModel.setRatio(tenant.getDiscount()));
                    }
                    item.setElements(JSONObject.toJSONString(elementModels));
                    item.setFinalPrice(item.getBasicPrice().multiply(tenant.getDiscount()));
                    orderItemRepository.save(item);
                }
            }
            if (null != sla) {
                SlaInstance slaInstance =
                    new SlaInstance(sla.getName(), sla.getContent(), new Date(), context.getTarget(),
                        sla.getServiceId(), slaId, order.getId());
                slaInstanceRepository.save(slaInstance);
            }
            JSONObject configs = applicationParam.getConfigs();
            if (null == configs) {
                configs = new JSONObject();
            }
            configs.put("skuId", skuId);
            if (null != sku) {
                configs.put("skuName", sku.getName());
            }
            if (null != category) {
                configs.put("categoryId", category.getId());
                configs.put("categoryCode", category.getCode());
                configs.put("categoryProps", category.getProps());
            }
            configs.put("category", operate.getCategory().name());
            if (StringUtils.isNotEmpty(service.getTemplateName())) {
                configs.put("templateName", service.getTemplateName());
            }
            applicationParam.setConfigs(configs);
            applicationParam.setOrder(order);
            applicationParam.setContext(context);
            Long flowId =
                serviceFlowConfigRepository.getServiceFlowId(service.getId(), ApplyCategory.ApplicationOperate);
            Boolean noFlow = Optional.ofNullable(configs.getBoolean("noFlow")).orElse(false);
            if (null != flowId && !noFlow) {
                items.forEach(item -> {
                    item.setStatus(OrderItemStatus.Audit);
                    orderItemRepository.update(item);
                });
                order.setOrderStatus(ChargeCommon.OrderStatus.audit);
                serviceOrderRepository.update(order);
                Map<String, Object> content =
                    getParams(ApplyCategory.ApplicationOperate.name(), JSONObject.toJSONString(applicationParam),
                        "申请审核服务【" + service.getName() + "】使用", context);
                content.put("order", JSONObject.toJSONString(order));
                content.put("requestContext", JSONObject.toJSONString(context));
                Execution execution =
                    engine.startInstanceById(flowId, context, applicationParam.getLocation().getProjectId(), content);
                if (execution == null || execution.getFlowInstance() == null) {
                    order.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                    serviceOrderRepository.update(order);
                    return new BocloudResult(false, "服务申请使用审核流程启动失败");
                }
                return new BocloudResult(true, "服务申请使用审核流程启动成功");
            } else {
                //下发到底层服务的时候按年计费时间做特殊处理
                if (mode == Mode.Year) {
                    EmptionModel paramEmption = applicationParam.getEmption();
                    DurationModel dura = paramEmption.getDuration();
                    dura.setAmount(dura.getAmount() * 12);
                    dura.setMode(Mode.Month);
                    paramEmption.setDuration(dura);
                    applicationParam.setEmption(paramEmption);
                }
                order.setApplyParam(JSONObject.toJSONString(applicationParam));
                serviceOrderRepository.update(order);
                // 后付费
                if (ChargeCommon.ChargeMode.PostCharge == order.getChargeMode()) {
                    order.setOrderStatus(ChargeCommon.OrderStatus.paid);
                    serviceOrderRepository.update(order);
                    if (service.getCode().startsWith("operations.")) {
                        cosService.create(applicationParam, applicationParam.getContext());
                    } else if (service.getCode().startsWith("fabric.")) {
                        rosService.create(applicationParam, applicationParam.getContext());
                    } else if ("vmware.nsx.vpc".equalsIgnoreCase(service.getCode())) {
                        logger.info("后付费NSX-T Vpc申请参数结构：" + JSONObject.toJSONString(applicationParam));
                        amqpTemplate.convertAndSend("network.creation.queue", applicationParam);
                    } else {
                        List<ApplicationParam> messageParams = Lists.newArrayList(applicationParam);
                        logger.info("后付费资源申请参数结构：" + JSONObject.toJSONString(messageParams));
                        if (operate instanceof AbstractPaasResourceOperate) {
                            amqpTemplate.convertAndSend("paas.resource.creation.queue", messageParams);
                        } else if (operate instanceof AbstractIaasResourceOperate) {
                            amqpTemplate.convertAndSend("iaas.resource.creation.queue", messageParams);
                        } else {
                            amqpTemplate.convertAndSend("iaas.resource.creation.queue", messageParams);
                        }
                    }
                    return new BocloudResult<>(true, "申请已提交,请等待资源创建。");
                } else {
                    order.setOrderStatus(ChargeCommon.OrderStatus.unpay);
                    serviceOrderRepository.update(order);
                    return new BocloudResult<>(true, order.getSn(), "申请已提交,支付订单后资源即可创建");
                }
            }
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult<>(false, "资源申请失败 : " + e.getMessage());
        }
    }

    private Map<String, Object> getParams(String category, String params, String theme, RequestContext context) {
        Map<String, Object> map = new HashMap<>(16);
        map.put("ApplyCategory", category);
        map.put("ApplyParams", params);
        map.put("ApplyTheme", theme);
        map.put("ApplyContext", context);
        return map;
    }

    @Override
    public BocloudResult<ApplicationModel> applyCheck(ApplicationParam param, RequestContext context, boolean isCheck) {
        try {
            CloudService service = cloudServiceRepository.queryByCode(param.getService());
            Assert.notNull(service, "该服务不存在");
            Assert.isTrue(service.getStatus().equals("PUBLISHED"), "服务未发布");
            Tenant tenant = tenantService.detail(context.getTenant());
            Assert.notNull(tenant, "申请人所在租户不存在");
            // 商品sku
            List<ElementModel> elements = param.getElements();
            // 服务协议信息
            Long slaId = param.getSla();
            ServiceSla sla = null;
            if (null == slaId) {
                List<ServiceSla> slas =
                    serviceSlaRepository.listByService(service.getId(), ServiceSla.Status.enable.name());
                if (!ListTool.isEmpty(slas)) {
                    sla = slas.get(0);
                    slaId = sla.getId();
                }
            } else {
                sla = serviceSlaRepository.query(slaId);
                Assert.notNull(sla, "该服务的SLA协议不存在，无法订购");
            }
            Long skuId = null;
            for (ElementModel element : elements) {
                if (service.getCode().equals(element.getServiceCode())) {
                    skuId = element.getSkuId();
                    break;
                }
            }
            ServiceCategory category = null;
            ServiceSku sku = null;
            if (null != skuId && 0L != skuId) {
                sku = serviceSkuRepository.query(skuId);
                Assert.notNull(sku, "该服务产品不存在");
                category = serviceCategoryRepository.query(sku.getCategoryId());
                Assert.notNull(category, "该服务类型不存在");
                // sku不为空，则说明需要走计费，需要校验租户授信额度
                TenantAccount account = accountRepository.getAccount(context.getTenant());
                Assert.notNull(account, "未查询到租户授信额度");
            }
            AbstractResourceOperate applyOperate = ResourceOperateUtils
                .getApplyOperate(service.getCode(), param.getEmption(), param.getElements(), cloudServiceRepository,
                    serviceSkuRepository, orderItemRepository, serviceCategoryRepository, lockFactory, orderSnService);
            Assert.notNull(applyOperate, "暂不支持该类服务订购操作");
            List<OrderItem> orderItems = applyOperate.buildApplyItem();
            Map<String, Integer> quotaMap = OrderItemUtils.getItemQuota(orderItems);
            Result result = handleQuota(quotaMap, context.getTenant(), isCheck);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            ApplicationModel model =
                new ApplicationModel(service, slaId, sla, skuId, sku, category, orderItems, applyOperate);
            return new BocloudResult<>(true, model, "资源申请校验成功");
        } catch (Exception e) {
            logger.error("resource apply check error:", e);
            return new BocloudResult<>(false, "资源申请失败" + e.getMessage());
        }
    }

    @Override public Result alteration(AlterationParam alterationParam, RequestContext context) {
        try {
            BocloudResult<AlterationModel> alterCheckResult = alterCheck(alterationParam, context);
            if (alterCheckResult.isFailed()) {
                return new Result(false, alterCheckResult.getMessage());
            }
            AlterationModel model = alterCheckResult.getData();
            ServiceOrder order = model.getOrder();
            CloudService service = model.getService();
            OrderItem oldItem = model.getItem();
            String code = service.getCode();
            // 获取资源变更操作对象 生成变更订单项
            List<OrderItem> alterationItems = model.getAlterationItems();
            AbstractResourceOperate alterationOperate = model.getAlterationOperate();
            ServiceOrder newOrder = new ServiceOrder();
            newOrder.setServiceCode(service.getCode());
            // 生成订单号，增加分布式锁
            String path = IdGenerator.class.getSimpleName() + "_ordersn";
            try (BocloudACLock lock = lockFactory.getACLock(path)) {
                Assert.isTrue(lock.acquire(1, TimeUnit.MINUTES), "生成订单号请求超时");
                newOrder.setSn(IdGenerator.getInstance().nextId());
            } catch (Exception e) {
                logger.error("Get error message：{}", e.getMessage());
                return new BocloudResult<>(false, "资源变更失败 : " + e.getMessage());
            }
            newOrder.setChargeMode(order.getChargeMode());
            newOrder.setSlaAgreed(order.getSlaAgreed());
            if (ChargeMode.PostCharge.equals(order.getChargeMode())) {
                newOrder.setOrderStatus(ChargeCommon.OrderStatus.paid);
            } else {
                newOrder.setOrderStatus(ChargeCommon.OrderStatus.unpay);
            }
            newOrder.setType(ChargeCommon.Type.alterat);
            newOrder.setGmtCreate(new Date());
            newOrder.setAutoRenew(order.getAutoRenew());
            newOrder.setVendorId(order.getVendorId());
            newOrder.setBusinessId(order.getBusinessId());
            newOrder.setProjectId(order.getProjectId());
            newOrder.setUserId(order.getUserId());
            newOrder.setTenantId(order.getTenantId());
            newOrder.setResourceStatus(ChargeCommon.ResourceStatus.changing);
            serviceOrderRepository.save(newOrder);
            Tenant tenant = tenantService.detail(order.getTenantId());
            switch (alterationParam.getOperation()) {
                case Upgrade:
                    if (alterationOperate instanceof AbstractPaasResourceOperate) {
                        for (OrderItem alterationItem : alterationItems) {
                            List<ElementModel> elementModels =
                                JSONArray.parseArray(alterationItem.getElements(), ElementModel.class);
                            if ("vmware.storage.disk".equals(alterationItem.getServiceCode())) {
                                ElementModel diskElement = elementModels.get(0);
                                List<SpecModel> diskSpecModel = diskElement.getSpecs();
                                Integer disk =
                                    diskSpecModel.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0))
                                        .reduce(Integer::sum).orElse(0);
                                alterationParam.setDataDisk(disk);
                            } else {
                                ElementModel upgradeElement = elementModels.get(0);
                                List<SpecModel> specs = upgradeElement.getSpecs();
                                Integer cpu = specs.stream().map(spec -> Optional.ofNullable(spec.getCpu()).orElse(0))
                                    .reduce(Integer::sum).orElse(0);
                                BigDecimal memory = specs.stream()
                                    .map(spec -> Optional.ofNullable(spec.getMemory()).orElse(BigDecimal.ZERO))
                                    .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
                                Integer disk = specs.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0))
                                    .reduce(Integer::sum).orElse(0);
                                alterationParam.setUpgradeSkuId(upgradeElement.getSkuId());
                                alterationParam.setCpu(cpu);
                                alterationParam.setMemory(memory);
                                alterationParam.setSystemDisk(disk);
                            }
                            alterationItem.setElements(JSONObject.toJSONString(elementModels));
                            alterationItem.setFinalPrice(alterationItem.getBasicPrice().multiply(tenant.getDiscount()));
                            alterationItem.setOrderId(newOrder.getId());
                            orderItemRepository.save(alterationItem);
                        }
                    } else {
                        // 获取升级规格 设置升级后的cpu内存大小到变更参数里面
                        OrderItem upgradeItem = alterationItems.get(0);
                        List<ElementModel> elementModels =
                            JSONArray.parseArray(upgradeItem.getElements(), ElementModel.class);
                        ElementModel element = elementModels.get(0);
                        List<SpecModel> specs = element.getSpecs();
                        Integer cpu = specs.stream().map(spec -> Optional.ofNullable(spec.getCpu()).orElse(0))
                            .reduce(Integer::sum).orElse(0);
                        BigDecimal memory =
                            specs.stream().map(spec -> Optional.ofNullable(spec.getMemory()).orElse(BigDecimal.ZERO))
                                .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
                        alterationParam.setUpgradeSkuId(element.getSkuId());
                        alterationParam.setCpu(cpu);
                        alterationParam.setMemory(memory);
                        Integer systemDisk = specs.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0))
                            .reduce(Integer::sum).orElse(0);
                        if (!systemDisk.equals(0)) {
                            alterationParam.setSystemDisk(systemDisk);
                        }
                        // 订单项设置折扣
                        elementModels.forEach(elementModel -> elementModel.setRatio(tenant.getDiscount()));
                        upgradeItem.setElements(JSONObject.toJSONString(elementModels));
                        upgradeItem.setFinalPrice(upgradeItem.getBasicPrice().multiply(tenant.getDiscount()));
                        upgradeItem.setOrderId(newOrder.getId());
                        orderItemRepository.save(upgradeItem);
                    }
                    break;
                case EditDisk:
                    OrderItem editItem = alterationItems.get(0);
                    List<ElementModel> editDiskElements =
                        JSONArray.parseArray(editItem.getElements(), ElementModel.class);
                    editDiskElements.forEach(elementModel -> elementModel.setRatio(tenant.getDiscount()));
                    editItem.setElements(JSONObject.toJSONString(editDiskElements));
                    editItem.setFinalPrice(editItem.getBasicPrice().multiply(tenant.getDiscount()));
                    editItem.setOrderId(newOrder.getId());
                    orderItemRepository.save(editItem);
                    break;
                case AddDisk:
                    int diskTotal = 0;
                    for (OrderItem alterationItem : alterationItems) {
                        List<ElementModel> diskElements =
                            JSONArray.parseArray(alterationItem.getElements(), ElementModel.class);
                        ElementModel diskElement = diskElements.get(0);
                        List<SpecModel> diskSpecModel = diskElement.getSpecs();
                        Integer disk = diskSpecModel.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0))
                            .reduce(Integer::sum).orElse(0);
                        diskTotal += disk;
                        // 订单项设置折扣
                        diskElements.forEach(elementModel -> elementModel.setRatio(tenant.getDiscount()));
                        alterationItem.setElements(JSONObject.toJSONString(diskElements));
                        alterationItem.setFinalPrice(alterationItem.getBasicPrice().multiply(tenant.getDiscount()));
                        alterationItem.setOrderId(newOrder.getId());
                        // 保存新订单
                        orderItemRepository.save(alterationItem);
                    }
                    alterationParam.setDataDisk(diskTotal);
                    break;
                default:
                    break;
            }
            // 变更操作 配额差值
            List<QuotaBean> quotaBeans = new ArrayList<>();
            Map<String, Integer> alterationQuota = OrderItemUtils.getItemQuota(alterationItems);
            Map<String, Integer> oldQuota;
            if (alterationOperate instanceof AbstractPaasResourceOperate) {
                ElementModel elementModel = alterationParam.getUpgrade().get(0);
                List<OrderItem> oldItems =
                    orderItemRepository.listByOrderId(oldItem.getOrderId(), elementModel.getServiceCode());
                oldQuota = OrderItemUtils.getItemQuota(oldItems);
            } else {
                oldQuota = OrderItemUtils.getItemQuota(Lists.newArrayList(oldItem));
            }
            for (Map.Entry<String, Integer> entry : alterationQuota.entrySet()) {
                String quotaCode = entry.getKey();
                Integer number = Optional.ofNullable(oldQuota.get(quotaCode)).orElse(0);
                quotaBeans.add(new QuotaBean(quotaCode, alterationQuota.get(quotaCode) - number));
                alterationQuota.put(quotaCode, alterationQuota.get(quotaCode) - number);
            }
            alterationParam.setQuotas(quotaBeans);
            Result result = handleQuota(alterationQuota, context.getTenant(), false);
            if (result.isFailed()) {
                return new Result(false, result.getMessage());
            }
            alterationParam.setContext(context);
            alterationParam.setCategory(alterationOperate.getCategory().name());
            alterationParam.setService(code);
            alterationParam.setOldStatus(oldItem.getStatus());
            alterationParam.setNewInstance(alterationItems.get(0).getSn());
            alterationParam.setOrderSn(newOrder.getSn());
            newOrder.setApplyParam(JSONObject.toJSONString(alterationParam));
            Long flowId =
                serviceFlowConfigRepository.getServiceFlowId(service.getId(), ApplyCategory.AlterationOperate);
            if (null != flowId) {
                for (OrderItem alterationItem : alterationItems) {
                    alterationItem.setStatus(OrderItemStatus.Audit);
                    orderItemRepository.update(alterationItem);
                }
                newOrder.setOrderStatus(ChargeCommon.OrderStatus.audit);
                serviceOrderRepository.update(newOrder);
                Map<String, Object> content =
                    getParams(ApplyCategory.AlterationOperate.name(), JSONObject.toJSONString(alterationParam),
                        "申请审核服务实例【" + alterationParam.getInstanceName() + "】变更", context);
                content.put("order", JSONObject.toJSONString(newOrder));
                content.put("requestContext", JSONObject.toJSONString(context));
                Execution execution = engine.startInstanceById(flowId, context, newOrder.getProjectId(), content);
                if (execution == null || execution.getFlowInstance() == null) {
                    newOrder.setOrderStatus(ChargeCommon.OrderStatus.canceld);
                    serviceOrderRepository.update(newOrder);
                    return new Result(false, "服务实例变更审核流程启动失败");
                }
                return new Result(true, "服务实例变更审核流程启动成功");
            } else {
                serviceOrderRepository.update(newOrder);
                // 预付费生成未支付账单
                if (ChargeCommon.ChargeMode.PreCharge == newOrder.getChargeMode()) {
                    return new BocloudResult<>(true, newOrder.getSn(), "账单已生成，请支付");
                }
                if (alterationOperate instanceof AbstractPaasResourceOperate) {
                    amqpTemplate.convertAndSend("paas.resource.modification.queue", alterationParam);
                } else {
                    amqpTemplate.convertAndSend("iaas.resource.modification.queue", alterationParam);
                }
                return new BocloudResult<>(true, "申请已提交");
            }
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "申请提交失败：" + e.getMessage());
        }
    }

    @Override public BocloudResult<AlterationModel> alterCheck(AlterationParam params, RequestContext context) {
        try {
            Assert.notNull(params.getOperation(), "请设置服务变更类型");
            // 服务资源实例id
            OrderItem oldItem = null;
            switch (params.getOperation()) {
                case Upgrade:
                case AddDisk:
                    oldItem = orderItemRepository.queryBySn(params.getInstance());
                    break;
                case EditDisk:
                    List<ElementModel> upgrade = params.getUpgrade();
                    ElementModel elementModel = upgrade.get(0);
                    oldItem = orderItemRepository.queryBySn(elementModel.getUuid());
                    break;
                default:
                    break;
            }
            Assert.notNull(oldItem, "原订单明细未找到！");
            CloudService service = cloudServiceRepository.queryByCode(params.getService());
            Assert.notNull(service, "订单明细关联服务未找到！");
            Assert.isTrue(service.getStatus().equals("PUBLISHED"), "服务未发布");
            logger.info("旧订单elements：{}", JSONObject.toJSON(oldItem.getElements()));
            ServiceOrder oldOrder = serviceOrderRepository.query(oldItem.getOrderId());
            Assert.notNull(oldOrder, "原订单未找到！");
            if (ChargeMode.PreCharge == oldOrder.getChargeMode()) {
                Long tenantId = null;
                Long userId = null;
                if (Catalog.User == context.getCatalog()) {
                    userId = context.getTarget();
                } else if (Catalog.Tenant == context.getCatalog()) {
                    tenantId = context.getTenant();
                }
                int count = billRepository
                    .count(ChargeCommon.BillStatus.unpay.name(), ChargeMode.PreCharge.name(), tenantId, userId,
                        oldOrder.getId());
                if (count > 0) {
                    return new BocloudResult<>(false, "当前订单存在未支付的账单，请先支付");
                }
            }
            Tenant tenant = tenantService.detail(context.getTenant());
            Assert.notNull(tenant, "申请租户不存在");
            ServiceCategory category = null;
            if (null != oldItem.getSkuId() && 0L != oldItem.getSkuId()) {
                ServiceSku sku = serviceSkuRepository.query(oldItem.getSkuId());
                Assert.notNull(sku, "订单明细规格未找到！");
                category = serviceCategoryRepository.query(sku.getCategoryId());
                Assert.notNull(category, "订单明细关联服务类型未找到！");
            }
            AbstractResourceOperate alterationOperate = ResourceOperateUtils
                .getAlterationOperate(params.getService(), params.getInstance(), params.getOperation(),
                    params.getUpgrade(), params.getExpiredTime(), cloudServiceRepository, serviceSkuRepository,
                    orderItemRepository, lockFactory, orderSnService, serviceCategoryRepository);
            Assert.notNull(alterationOperate, service.getName() + "服务不支持变更操作");
            List<OrderItem> alterationItems = alterationOperate.buildAlterationItem();
            Map<String, Integer> alterationQuota = OrderItemUtils.getItemQuota(alterationItems);
            Map<String, Integer> oldQuota;
            if ((alterationOperate instanceof AbstractPaasResourceOperate)) {
                ElementModel elementModel = params.getUpgrade().get(0);
                List<OrderItem> oldItems =
                    orderItemRepository.listByOrderId(oldOrder.getId(), elementModel.getServiceCode());
                oldQuota = OrderItemUtils.getItemQuota(oldItems);
            } else {
                oldQuota = OrderItemUtils.getItemQuota(Lists.newArrayList(oldItem));
            }
            for (Map.Entry<String, Integer> entry : alterationQuota.entrySet()) {
                String code = entry.getKey();
                Integer number = Optional.ofNullable(oldQuota.get(code)).orElse(0);
                alterationQuota.put(code, alterationQuota.get(code) - number);
            }
            Result result = handleQuota(alterationQuota, context.getTenant(), true);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            AlterationModel model =
                new AlterationModel(oldOrder, oldItem, service, category, alterationOperate, alterationItems, tenant);
            return new BocloudResult<>(true, model, "资源变更校验成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "变更申请失败：" + e.getMessage());
        }
    }

    @Override public BocloudResult extension(ExtensionParam extensionParam, RequestContext context) {
        try {
            // 服务资源实例id
            String instance = extensionParam.getInstance();
            OrderItem item = orderItemRepository.queryBySn(instance);
            Assert.notNull(item, "订单明细未找到");
            ServiceOrder order = serviceOrderRepository.query(item.getOrderId());
            DurationModel duration = JSONObject.parseObject(item.getDuration(), DurationModel.class);
            //延期月份
            Integer extension = extensionParam.getExtension();
            Assert.notNull(order, "订单未找到");
            if (ChargeMode.PreCharge == order.getChargeMode()) {
                Long tenantId = null;
                Long userId = null;
                if (Catalog.User == context.getCatalog()) {
                    userId = context.getTarget();
                } else if (Catalog.Tenant == context.getCatalog()) {
                    tenantId = context.getTenant();
                }
                int count = billRepository
                    .count(ChargeCommon.BillStatus.unpay.name(), ChargeMode.PreCharge.name(), tenantId, userId,
                        order.getId());
                if (count > 0) {
                    return new BocloudResult<>(false, "当前订单存在未支付的账单，请先支付");
                }
            }
            Assert.isTrue(!ChargeMode.PostCharge.equals(order.getChargeMode()), "后付费订单不能进行延期操作");
            CloudService service = cloudServiceRepository.queryByCode(item.getServiceCode());
            Assert.notNull(service, "订单明细关联的服务未找到");
            extensionParam.setService(service.getCode());
            if (null != item.getSkuId() && 0L != item.getSkuId()) {
                ServiceSku sku = serviceSkuRepository.query(item.getSkuId());
                Assert.notNull(sku, "订单明细规格未找到");
            }
            AbstractResourceOperate extensionOperate = ResourceOperateUtils
                .getExtensionOperate(extensionParam.getService(), extensionParam.getInstance(),
                    extensionParam.getExtension(), orderItemRepository, serviceOrderRepository, lockFactory,
                    orderSnService);
            Assert.notNull(extensionOperate, "不支持延期操作");
            extensionParam.setContext(context);
            extensionParam.setCategory(extensionOperate.getCategory().name());
            extensionParam.setOldStatus(item.getStatus());
            ServiceOrder extensionOrder = new ServiceOrder();
            extensionOrder.setServiceCode(extensionParam.getService());
            // 生成订单号，增加分布式锁
            String path = IdGenerator.class.getSimpleName() + "_ordersn";
            try (BocloudACLock lock = lockFactory.getACLock(path)) {
                Assert.isTrue(lock.acquire(1, TimeUnit.MINUTES), "生成订单号请求超时");
                extensionOrder.setSn(IdGenerator.getInstance().nextId());
            } catch (Exception e) {
                logger.error("Get error message：{}", e.getMessage());
                return new BocloudResult<>(false, "资源延期失败 : " + e.getMessage());
            }
            extensionOrder.setChargeMode(order.getChargeMode());
            extensionOrder.setSlaAgreed(order.getSlaAgreed());
            extensionOrder.setOrderStatus(ChargeCommon.OrderStatus.unpay);
            extensionOrder.setType(ChargeCommon.Type.extension);
            extensionOrder.setGmtCreate(new Date());
            extensionOrder.setAutoRenew(order.getAutoRenew());
            extensionOrder.setVendorId(order.getVendorId());
            extensionOrder.setBusinessId(order.getBusinessId());
            extensionOrder.setProjectId(order.getProjectId());
            extensionOrder.setUserId(order.getUserId());
            extensionOrder.setTenantId(order.getTenantId());
            extensionOrder.setResourceStatus(ChargeCommon.ResourceStatus.creating);
            extensionOrder.setApplyParam(JSONObject.toJSONString(extensionParam));
            serviceOrderRepository.save(extensionOrder);
            Tenant tenant = tenantService.detail(context.getTenant());
            if (extensionOperate instanceof AbstractPaasResourceOperate) {
                //找到所有资源订单项 做延期计费
                List<OrderItem> orderItems = orderItemRepository.listByOrderId(item.getOrderId());
                for (OrderItem orderItem : orderItems) {
                    ServiceSku serviceSku = serviceSkuRepository.query(orderItem.getSkuId());
                    CloudService cloudService = cloudServiceRepository.queryByCode(orderItem.getServiceCode());
                    List<ElementModel> elementModels =
                        JSONArray.parseArray(orderItem.getElements(), ElementModel.class);
                    ElementModel element = elementModels.get(0);
                    BigDecimal unitPrice =
                        OrderPriceUtils.getUnitPrice(cloudService, serviceSku, element.getSpecs(), duration);
                    Integer count = Optional.ofNullable(element.getCount()).orElse(1);
                    unitPrice = unitPrice.multiply(BigDecimal.valueOf(count));
                    OrderItem extensionItem = new OrderItem();
                    extensionItem.setCatalog(OrderItem.Catalog.complex);
                    extensionItem.setSkuId(orderItem.getSkuId());
                    extensionItem.setSn(orderItem.getSn());
                    extensionItem.setServiceCode(serviceSku.getServiceCode());
                    extensionItem.setStatus(OrderItemStatus.Extensioning);
                    DurationModel durationModel = new DurationModel();
                    durationModel.setMode(duration.getMode());
                    durationModel.setAmount(extension);
                    extensionItem.setDuration(JSONObject.toJSONString(durationModel));
                    extensionItem.setDeleted(false);
                    extensionItem.setResources("[]");
                    extensionItem.setEffectiveTime(orderItem.getExpiredTime());
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(orderItem.getExpiredTime());
                    if (duration.getMode().equals(DurationModel.Mode.Month)) {
                        calendar.add(Calendar.MONTH, extension);
                    } else {
                        calendar.add(Calendar.YEAR, extension);
                    }
                    extensionItem.setExpiredTime(calendar.getTime());
                    extensionItem.setBasicPrice(unitPrice);
                    extensionItem.setElements(orderItem.getElements());
                    extensionItem.setFinalPrice(unitPrice.multiply(tenant.getDiscount()));
                    extensionItem.setOrderId(extensionOrder.getId());
                    orderItemRepository.save(extensionItem);
                }
            } else if (!extensionOperate.getCategory().equals(ResourceCategory.Computer)) {
                OrderItem targetItem = orderItemRepository.queryLastSuccessBySn(instance);
                ServiceSku targetSku = serviceSkuRepository.query(targetItem.getSkuId());
                List<ElementModel> elementModels = JSONArray.parseArray(targetItem.getElements(), ElementModel.class);
                ElementModel element = elementModels.get(0);
                BigDecimal targetPrice = OrderPriceUtils.getUnitPrice(service, targetSku, element.getSpecs(), duration);
                OrderItem extensionItem = new OrderItem();
                extensionItem.setCatalog(OrderItem.Catalog.complex);
                extensionItem.setSkuId(targetItem.getSkuId());
                extensionItem.setSn(targetItem.getSn());
                extensionItem.setServiceCode(targetItem.getServiceCode());
                extensionItem.setStatus(OrderItemStatus.Extensioning);
                extensionItem.setElements(targetItem.getElements());
                DurationModel durationModel = new DurationModel();
                durationModel.setMode(duration.getMode());
                extensionItem.setEffectiveTime(targetItem.getExpiredTime());
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(targetItem.getExpiredTime());
                if (duration.getMode().equals(DurationModel.Mode.Month)) {
                    calendar.add(Calendar.MONTH, extension);
                } else {
                    calendar.add(Calendar.YEAR, extension);
                }
                extensionItem.setExpiredTime(calendar.getTime());
                durationModel.setAmount(extension);
                extensionItem.setDuration(JSONObject.toJSONString(durationModel));
                extensionItem.setDeleted(false);
                extensionItem.setResources("[]");
                extensionItem.setBasicPrice(targetPrice);
                extensionItem.setFinalPrice(targetPrice.multiply(tenant.getDiscount()));
                extensionItem.setOrderId(extensionOrder.getId());
                orderItemRepository.save(extensionItem);
            } else {
                //找到对应机器和磁盘 做延期计费
                CloudServer server = imsInternalService.queryBySn(instance);
                OrderItem serverItem = orderItemRepository.queryLastSuccessBySn(instance);
                ServiceSku serverSku = serviceSkuRepository.query(serverItem.getSkuId());
                List<ElementModel> elementModels = JSONArray.parseArray(serverItem.getElements(), ElementModel.class);
                ElementModel element = elementModels.get(0);
                BigDecimal serverPrice = OrderPriceUtils.getUnitPrice(service, serverSku, element.getSpecs(), duration);
                OrderItem serverExtension = new OrderItem();
                serverExtension.setCatalog(OrderItem.Catalog.complex);
                serverExtension.setSkuId(server.getSkuId());
                serverExtension.setSn(server.getUuid());
                serverExtension.setServiceCode(serverSku.getServiceCode());
                serverExtension.setStatus(OrderItemStatus.Extensioning);
                serverExtension.setElements(serverItem.getElements());
                DurationModel durationModel = new DurationModel();
                durationModel.setMode(duration.getMode());
                serverExtension.setEffectiveTime(serverItem.getExpiredTime());
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(serverItem.getExpiredTime());
                if (duration.getMode().equals(DurationModel.Mode.Month)) {
                    calendar.add(Calendar.MONTH, extension);
                } else {
                    calendar.add(Calendar.YEAR, extension);
                }
                serverExtension.setExpiredTime(calendar.getTime());
                durationModel.setAmount(extension);
                serverExtension.setDuration(JSONObject.toJSONString(durationModel));
                serverExtension.setDeleted(false);
                serverExtension.setResources("[]");
                serverExtension.setBasicPrice(serverPrice);
                serverExtension.setFinalPrice(serverPrice.multiply(tenant.getDiscount()));
                serverExtension.setOrderId(extensionOrder.getId());
                orderItemRepository.save(serverExtension);
                // 查询磁盘列表
                List<Volume> volumes = imsInternalService.listByVmId(server.getId());
                for (Volume volume : volumes) {
                    if (null != volume.getSkuId() && 0L != volume.getSkuId()) {
                        OrderItem volumeItem = orderItemRepository.queryLastSuccessBySn(volume.getUuid());
                        CloudService cloudService = cloudServiceRepository.queryByCode(volumeItem.getServiceCode());
                        ServiceSku volumeSku = serviceSkuRepository.query(volumeItem.getSkuId());
                        List<ElementModel> volumeElements =
                            JSONArray.parseArray(volumeItem.getElements(), ElementModel.class);
                        ElementModel elementModel = volumeElements.get(0);
                        BigDecimal volumePrice =
                            OrderPriceUtils.getUnitPrice(cloudService, volumeSku, elementModel.getSpecs(), duration);
                        OrderItem volumeOrder = new OrderItem();
                        volumeOrder.setCatalog(OrderItem.Catalog.complex);
                        volumeOrder.setSkuId(volume.getSkuId());
                        volumeOrder.setSn(volume.getUuid());
                        volumeOrder.setElements(volumeItem.getElements());
                        volumeOrder.setServiceCode(volumeSku.getServiceCode());
                        volumeOrder.setStatus(OrderItemStatus.Extensioning);
                        volumeOrder.setDuration(JSONObject.toJSONString(durationModel));
                        volumeOrder.setEffectiveTime(serverExtension.getEffectiveTime());
                        volumeOrder.setExpiredTime(serverExtension.getExpiredTime());
                        volumeOrder.setDeleted(false);
                        volumeOrder.setResources("[]");
                        volumeOrder.setBasicPrice(volumePrice);
                        volumeOrder.setFinalPrice(volumePrice.multiply(tenant.getDiscount()));
                        volumeOrder.setOrderId(extensionOrder.getId());
                        orderItemRepository.save(volumeOrder);
                    }
                }
            }
            Long flowId = serviceFlowConfigRepository.getServiceFlowId(service.getId(), ApplyCategory.ExtensionOperate);
            if (null == flowId) {
                if (duration.getMode().equals(Mode.Year)) {
                    extensionParam.setExtension(extensionParam.getExtension() * 12);
                }
                extensionOrder.setApplyParam(JSONObject.toJSONString(extensionParam));
                serviceOrderRepository.update(extensionOrder);
                return new BocloudResult<>(true, extensionOrder.getSn(), "申请已提交,支付后即可进行资源延期操作");
            } else {
                List<OrderItem> extensionItems = orderItemRepository.listByOrderId(extensionOrder.getId());
                for (OrderItem exItem : extensionItems) {
                    exItem.setStatus(OrderItemStatus.Audit);
                    orderItemRepository.update(exItem);
                }
                extensionOrder.setOrderStatus(ChargeCommon.OrderStatus.audit);
                serviceOrderRepository.update(extensionOrder);
                Map<String, Object> content =
                    getParams(ApplyCategory.ExtensionOperate.name(), JSONObject.toJSONString(extensionParam),
                        "申请审核服务实例【" + extensionParam.getInstanceName() + "】延期", context);
                content.put("order", JSONObject.toJSONString(extensionOrder));
                content.put("requestContext", JSONObject.toJSONString(context));
                Execution execution = engine.startInstanceById(flowId, context, extensionOrder.getProjectId(), content);
                if (execution == null || execution.getFlowInstance() == null) {
                    return new BocloudResult<>(false, "服务实例延期审核流程启动失败");
                }
                return new BocloudResult<>(true, "服务实例延期审核流程启动成功");
            }
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "申请提交失败 : " + e.getMessage());
        }
    }

    @Override public Result cancelation(CancelationParam cancelationParam, RequestContext context) {
        try {
            // 服务资源实例id
            String instance = cancelationParam.getInstance();
            // 检测资源操作
            OrderItem item = orderItemRepository.queryBySn(instance);
            Assert.notNull(item, "订单明细未找到");
            ServiceOrder order = serviceOrderRepository.query(item.getOrderId());
            Assert.notNull(order, "订单未找到");
            String code = item.getServiceCode();
            if (null != item.getSkuId() && 0L != item.getSkuId()) {
                ServiceSku sku = serviceSkuRepository.query(item.getSkuId());
                Assert.notNull(sku, "订单明细规格未找到");
                CloudService service = cloudServiceRepository.query(sku.getServiceId());
                Assert.notNull(service, "订单明细关联的服务未找到");
                code = service.getCode();
            }
            AbstractResourceOperate resourceOperate = ResourceOperateUtils.getResourceOperate(code);
            Assert.notNull(resourceOperate, "该服务不支持此操作");
            cancelationParam.setCategory(resourceOperate.getCategory().name());
            cancelationParam.setService(code);
            cancelationParam.setContext(context);
            cancelationParam.setOldStatus(item.getStatus());
            logger.info("退订消息：{}", cancelationParam.toString());
            if (resourceOperate instanceof AbstractPaasResourceOperate) {
                amqpTemplate.convertAndSend("paas.resource.deletion.queue", cancelationParam);
            } else {
                if ("vmware.nsx.vpc".equals(code)) {
                    amqpTemplate.convertAndSend("network.resource.deletion.queue", cancelationParam);
                } else {

                    amqpTemplate.convertAndSend("iaas.resource.deletion.queue", cancelationParam);
                }
            }
            return new Result(true, "申请已提交");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new Result(false, "申请已提交: " + e.getMessage());
        }
    }

    /**
     * 扣除配额
     *
     * @param quotaMap
     * @param tenantId
     * @param isCheck
     * @return
     * @throws Exception
     */
    private Result handleQuota(Map<String, Integer> quotaMap, Long tenantId, boolean isCheck) throws Exception {
        Tenant tenant = tenantService.detail(tenantId);
        if (!tenant.getQuotaAble()) {
            return new Result(true, "租户未开启配额,不需要计算配额");
        }
        if (null == quotaMap || quotaMap.isEmpty()) {
            return new Result(true, "无配额信息需更新");
        }
        List<ResourceQuota> list = resourceQuotaRepository.listByTenantId(tenantId);
        Assert.notEmpty(list, "未查询到配额信息");
        Map<String, ResourceQuota> map = new HashMap<>(8);
        for (ResourceQuota quota : list) {
            map.put(quota.getCode(), quota);
        }
        List<ResourceQuota> quotas = Lists.newArrayList();
        for (Entry<String, Integer> entry : quotaMap.entrySet()) {
            String code = entry.getKey();
            if (!"vmware.storage.disk".equals(code)) {
                String quotaName;
                if (QuotaCategory.CPU.getCode().equals(code)) {
                    quotaName = QuotaCategory.CPU.getName();
                } else if (QuotaCategory.MEM.getCode().equals(code)) {
                    quotaName = QuotaCategory.MEM.getName();
                } else if (QuotaCategory.DISK.getCode().equals(code)) {
                    quotaName = QuotaCategory.DISK.getName();
                } else {
                    CloudService cloudService = cloudServiceRepository.queryByCode(code);
                    quotaName = cloudService.getName();
                }
                if (!map.containsKey(entry.getKey())) {
                    return new Result(false, "未查询到" + quotaName + "配额信息");
                }
                ResourceQuota quota = map.get(entry.getKey());
                if (entry.getValue() + quota.getUsed() > quota.getQuota()) {
                    return new Result(false, "剩余" + quotaName + "配额不足");
                }
                quota.setQuotaActionValue(entry.getValue().longValue());
                quotas.add(quota);
            }
        }
        if (isCheck) {
            return new Result(true, "配额信息校验成功");
        }
        Result result = resourceQuotaService.quotaAction(quotas, ResourceQuota.Action.Order, tenantId);
        if (result.isFailed()) {
            return result;
        }
        return new Result(true, "更新配额信息成功");
    }

    @Override public Result updateOrder(OrderMessage message) {
        try {
            if (null == message) {
                return new Result(false, "消息格式不正确");
            }
            message.setCategory(ResourceCategory.Kubernetes.name());
            logger.info("接收到k8s结果消息：{}", JSONObject.toJSON(message));
            OrderItemStatus status = message.getStatus();
            if (OrderItemStatus.OpenedSuccess == status || OrderItemStatus.OpenedFailure == status) {
                OrderItem item = orderItemRepository.queryBySn(message.getSn());
                if (null == item) {
                    return new Result(false, "订单明细不存在");
                }
                logger.info("k8s结果订单明细消息：{}", JSONObject.toJSON(item));
                ServiceOrder order = serviceOrderRepository.query(item.getOrderId());
                if (null == order) {
                    return new Result(false, "订单不存在");
                }
                logger.info("k8s结果订单消息：{}", JSONObject.toJSON(order));
                Long tenantId = order.getTenantId();
                Long target = order.getUserId();
                Catalog catalog = Catalog.User;
                if (null != tenantId) {
                    catalog = Catalog.Tenant;
                    target = tenantId;
                }
                RequestContext context = new RequestContext(catalog, target, tenantId, null, null);
                String duration = item.getDuration();
                if (!StringUtils.isNotEmpty(duration)) {
                    return new Result(false, "订购时长信息不存在");
                }
                DurationModel durationModel = JSONObject.parseObject(duration, DurationModel.class);
                DurationModel.Mode mode = durationModel.getMode();
                Integer month = durationModel.getAmount();
                Date effectiveTime = new Date();
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(effectiveTime);
                if (DurationModel.Mode.Hour == mode) {
                    calendar.setTime(DateTools.stringToDateTime("2099-12-31 00:00:00"));
                } else {
                    calendar.add(Calendar.MONTH, month);
                }
                message.setEffectiveTime(effectiveTime);
                message.setExpiredTime(calendar.getTime());
                message.setContext(context);
                if (OrderItemStatus.OpenedFailure == message.getStatus()) {
                    List<ResourceModel> resourceModels = message.getResources();
                    List<String> instanceIds = new ArrayList<>();
                    if (!ListTool.isEmpty(resourceModels)) {
                        for (ResourceModel resourceModel : resourceModels) {
                            instanceIds.add(resourceModel.getUuid());
                        }
                        imsInternalService.delete(instanceIds);
                    }
                }
            } else if (OrderItemStatus.ChangeSuccess == status || OrderItemStatus.ChangeFailure == status) {
                OrderItem item = orderItemRepository.queryBySn(message.getNewSn());
                if (null == item) {
                    return new Result(false, "订单明细不存在");
                }
                ServiceOrder order = serviceOrderRepository.query(item.getOrderId());
                if (null == order) {
                    return new Result(false, "订单不存在");
                }
                List<OrderItem> oldItems = orderItemRepository.listByOrderId(order.getId());
                OrderItem changingItem = null;
                for (OrderItem orderItem : oldItems) {
                    if (OrderItemStatus.Changing == orderItem.getStatus()) {
                        changingItem = orderItem;
                        break;
                    }
                }
                if (null == changingItem) {
                    return new Result(false, "变更前子订单未找到");
                }
                // 查询变更前账单
                Long tenantId = order.getTenantId();
                Long target = order.getUserId();
                Catalog catalog = Catalog.User;
                if (null != tenantId) {
                    catalog = Catalog.Tenant;
                    target = tenantId;
                }
                RequestContext context = new RequestContext(catalog, target, tenantId, null, null);
                message.setEffectiveTime(new Date());
                message.setExpiredTime(changingItem.getExpiredTime());
                message.setContext(context);
                message.setSn(changingItem.getSn());
                if (OrderItemStatus.ChangeFailure == message.getStatus()) {
                    List<ResourceModel> resourceModels = message.getResources();
                    List<String> instanceIds = new ArrayList<>();
                    if (!ListTool.isEmpty(resourceModels)) {
                        for (ResourceModel resourceModel : resourceModels) {
                            instanceIds.add(resourceModel.getUuid());
                        }
                        imsInternalService.delete(instanceIds);
                    }
                }
            } else {
                return new Result(false, "订单更新失败");
            }
            amqpTemplate.convertAndSend("resource.result.queue", message);
            return new Result(true, "订单更新成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new Result(false, "订单更新失败");
        }
    }

    @Override public Result k8sAlteration(AlterationParam params) {
        try {
            logger.info("接收到k8s缩容消息：{}", JSONObject.toJSONString(params));
            OrderItem item = orderItemRepository.queryBySn(params.getInstance());
            Assert.notNull(item, "子订单明细不存在");
            ServiceOrder order = serviceOrderRepository.query(item.getOrderId());
            Assert.notNull(order, "订单不存在");
            Catalog catalog = Catalog.Tenant;
            Long target = order.getTenantId();
            if (null != order.getUserId()) {
                catalog = Catalog.User;
                target = order.getUserId();
            }
            RequestContext context = new RequestContext(catalog, target, order.getTenantId(), null, null);
            return this.alteration(params, context);
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return new Result(false, e.getMessage());
        }
    }

    @Override @Transactional(rollbackFor = BoCloudDBException.class)
    public Result clone(CloneParams cloneParams, RequestContext context) {
        try {
            // 生成克隆参数
            logger.info("接收到克隆消息：{}", JSONObject.toJSON(cloneParams));
            Assert.notNull(cloneParams, "参数格式错误");
            Long templateId = cloneParams.getTemplateId();
            String instance = cloneParams.getInstance();
            JSONObject config = new JSONObject();
            config.put("templateId", templateId);
            config.put("instance", instance);
            Assert.notNull(instance, "未找到资源关联的子订单");
            OrderItem item = orderItemRepository.queryBySn(instance);
            Assert.notNull(item, "未找到资源关联的子订单");
            ServiceOrder order = serviceOrderRepository.query(item.getOrderId());
            Assert.notNull(order, "未找到资源关联的订单");
            ApplicationParam applicationParam = JSONObject.parseObject(order.getApplyParam(), ApplicationParam.class);
            BocloudResult serverResult = imsInternalService.serverDetail(context.getApiKey(), templateId);
            Assert.isTrue(serverResult.isSuccess(), serverResult.getMessage());
            CloudServer server =
                JSONObject.parseObject(JSONObject.toJSONString(serverResult.getData()), CloudServer.class);
            // 查询磁盘列表
            List<Volume> volumes = imsInternalService.listByVmId(server.getId());
            // 系统盘
            Volume sysDisk =
                volumes.stream().filter(volume -> "Hard disk 1".equalsIgnoreCase(volume.getName())).findFirst()
                    .orElse(null);
            Assert.notNull(sysDisk, "未查询到系统盘信息");
            Long skuId = sysDisk.getSkuId();
            if (null != skuId && 0L != skuId) {
                ServiceSku diskSku = serviceSkuRepository.query(skuId);
                Assert.notNull(diskSku, "虚拟机系统盘关联的产品SKU未找到");
                config.put("diskCategoryId", diskSku.getCategoryId());
            }
            List<NetworkCard> netWordCards = imsInternalService.listNetworkCards(server.getId());
            JSONArray networkCardConfigs = new JSONArray();
            // 原虚拟机配置
            for (NetworkCard card : netWordCards) {
                JSONObject networkCardConfig = new JSONObject();
                networkCardConfig.put("ipPoolId", cloneParams.getIpPoolId());
                networkCardConfig.put("ipPolicy", cloneParams.getIpPolicy());
                networkCardConfig.put("networkCardId", card.getId());
                networkCardConfig.put("portGroupId", cloneParams.getPortGroupId());
                networkCardConfig.put("address", cloneParams.getAddresses());
                networkCardConfigs.add(networkCardConfig);
            }
            ApplicationParam newApplication = new ApplicationParam();
            LocationModel locationModel = new LocationModel();
            locationModel.setVendorId(server.getVendorId());
            PoolGroup poolGroup = imsInternalService.getPoolGroup(server.getPoolGroupId());
            Assert.notNull(poolGroup, "没找相应的资源池");
            locationModel.setAz(poolGroup.getAvailablitiyZone());
            locationModel.setRegion(poolGroup.getRegion());
            locationModel.setPoolGroupId(poolGroup.getId());
            locationModel.setProjectId(server.getProjectId());
            locationModel.setBusinessId(server.getBusinessId());
            locationModel.setCreatorId(server.getCreatorId());
            locationModel.setMenderId(server.getMenderId());
            locationModel.setOwnerId(server.getOwnerId());
            locationModel.setRegion(server.getRegion());
            locationModel.setVendorId(server.getVendorId());
            locationModel.setTenantId(server.getTenantId());
            locationModel.setVendorType(server.getVendorType());
            newApplication.setLocation(locationModel);
            DurationModel duration = new DurationModel();
            duration.setMode(Mode.Hour);
            EmptionModel emption = new EmptionModel();
            emption.setCount(1);
            emption.setAutoRenew(false);
            emption.setDuration(duration);
            newApplication.setEmption(emption);
            newApplication.setService(item.getServiceCode());

            List<ElementModel> elementModels = new ArrayList<>();
            Integer cpu = server.getCpu();
            BigDecimal memory = server.getMemory();
            // cpu内存规格
            ElementModel element = new ElementModel();
            element.setServiceCode(newApplication.getService());
            List<SpecModel> specs = new ArrayList<>();
            specs.add(new SpecModel(cpu, memory));
            element.setSkuId(server.getSkuId());
            element.setCategoryId(server.getCategoryId());
            element.setSpecs(specs);
            elementModels.add(element);
            // 磁盘规格
            for (Volume volume : volumes) {
                ElementModel volumeElement = new ElementModel();
                Long volumeSkuId = volume.getSkuId();
                if (null != volumeSkuId && 0L != volumeSkuId) {
                    ServiceSku diskSku = serviceSkuRepository.query(volumeSkuId);
                    volumeElement.setSkuId(volumeSkuId);
                    volumeElement.setCategoryId(diskSku.getCategoryId());
                }
                volumeElement.setServiceCode("vmware.storage.disk");
                List<SpecModel> diskSpecs = new ArrayList<>();
                diskSpecs.add(new SpecModel(volume.getSize()));
                volumeElement.setSpecs(diskSpecs);
                elementModels.add(volumeElement);
            }
            newApplication.setElements(elementModels);
            config.put("name", cloneParams.getName());
            config.put("isAddShterm", cloneParams.getIsAddShterm());
            config.put("cpu", cpu);
            config.put("memory", memory);
            config.put("disk", sysDisk.getSize());
            config.put("skuId", item.getSkuId());
            config.put("category", ResourceCategory.Computer.name());
            config.put("networkCardConfigs", networkCardConfigs);
            if (StringUtils.isNotEmpty(server.getPswd())) {
                Encryptor encryptor = new AESEncrptor();
                config.put("password", encryptor.encrypt(server.getPswd(), null));
            }
            newApplication.setConfigs(config);
            newApplication.setContext(context);
            // 申请校验规格 校验服务是否存在等
            BocloudResult<ApplicationModel> result = applyCheck(newApplication, context, false);
            if (result.isFailed()) {
                return new BocloudResult<>(false, result.getMessage());
            }
            ApplicationModel model = result.getData();
            // 生成订单订单项
            Mode mode = duration.getMode();
            ChargeMode chargeMode = ChargeCommon.ChargeMode.PostCharge;
            if (mode != Mode.Hour) {
                chargeMode = ChargeCommon.ChargeMode.PreCharge;
            }
            // 下单
            ServiceOrder cloneOrder = new ServiceOrder();
            cloneOrder.setServiceCode(newApplication.getService());
            // 生成订单号，增加分布式锁
            String path = IdGenerator.class.getSimpleName() + "_ordersn";
            try (BocloudACLock lock = lockFactory.getACLock(path)) {
                Assert.isTrue(lock.acquire(1, TimeUnit.MINUTES), "生成订单号请求超时");
                cloneOrder.setSn(IdGenerator.getInstance().nextId());
            } catch (Exception e) {
                logger.error("Get error message：{}", e.getMessage());
                return new BocloudResult<>(false, "资源申请失败 : " + e.getMessage());
            }
            cloneOrder.setChargeMode(chargeMode);
            if (ChargeMode.PostCharge.equals(chargeMode)) {
                cloneOrder.setOrderStatus(ChargeCommon.OrderStatus.paid);
            } else {
                cloneOrder.setOrderStatus(ChargeCommon.OrderStatus.unpay);
            }
            cloneOrder.setType(ChargeCommon.Type.apply);
            cloneOrder.setGmtCreate(new Date());
            cloneOrder.setAutoRenew(false);
            LocationModel location = newApplication.getLocation();
            // 设置订单归属
            cloneOrder.setVendorId(location.getVendorId());
            cloneOrder.setBusinessId(location.getBusinessId());
            cloneOrder.setProjectId(location.getProjectId());
            cloneOrder.setUserId(location.getOwnerId());
            cloneOrder.setTenantId(context.getTenant());
            cloneOrder.setResourceStatus(ChargeCommon.ResourceStatus.creating);
            serviceOrderRepository.save(cloneOrder);
            // 生成新的订单项
            List<OrderItem> items = model.getOrderItems();
            Tenant tenant = tenantService.detail(cloneOrder.getTenantId());
            cloneOrder.setItems(items);
            if (!ListTool.isEmpty(items)) {
                for (OrderItem cloneItem : items) {
                    cloneItem.setOrderId(cloneOrder.getId());
                    List<ElementModel> cloneElement = JSONArray.parseArray(cloneItem.getElements(), ElementModel.class);
                    cloneElement.forEach(elementModel -> elementModel.setRatio(tenant.getDiscount()));
                    cloneItem.setElements(JSONObject.toJSONString(cloneElement));
                    cloneItem.setFinalPrice(cloneItem.getBasicPrice().multiply(tenant.getDiscount()));
                    orderItemRepository.save(cloneItem);
                }
            }
            cloneOrder.setApplyParam(JSONObject.toJSONString(newApplication));
            newApplication.setOrder(cloneOrder);
            logger.info("克隆消息{},发送成功", JSONObject.toJSONString(Lists.newArrayList(newApplication)));
            amqpTemplate.convertAndSend("iaas.resource.creation.queue", Lists.newArrayList(newApplication));
            return new Result(true, "克隆申请已提交");
        } catch (Exception e) {
            logger.error("Get error message:{}", e.getMessage());
            return new Result(false, e.getMessage());
        }
    }

    @Override public Result empty(RequestContext context) {
        try {
            List<CloudServer> servers = imsInternalService.listServers(context);
            logger.info("虚拟机列表：{}", JSONObject.toJSON(servers));
            Assert.notEmpty(servers, "回收站无云主机需要清空");
            for (CloudServer server : servers) {
                OrderItem item = orderItemRepository.queryBySn(server.getUuid());
                if (OrderItemStatus.Unsubscribing == item.getStatus()) {
                    continue;
                }
                CancelationParam params = new CancelationParam();
                params.setInstance(server.getUuid());
                this.cancelation(params, context);
            }
            return new Result(true, "清空回收站任务已下发，请等待执行结果...");
        } catch (Exception e) {
            logger.error("Get error message:{}", e.getMessage());
            return new Result(false, e.getMessage());
        }
    }

}
