package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.cos.entity.Paragraph;
import com.bocloud.cos.entity.ReportConfigTemplate;
import com.bocloud.cos.entity.ReportHistoryResult;
import com.bocloud.cos.entity.ReportTemplateHistory;
import com.bocloud.cos.enums.ReportFrequenceType;
import com.bocloud.cos.intf.StorageReportService;
import com.bocloud.cos.model.ReportPair;
import com.bocloud.cos.model.StorageInformationReport;
import com.bocloud.cos.repository.ParagraphRepository;
import com.bocloud.cos.repository.ReportConfigTemplateRepository;
import com.bocloud.cos.repository.ReportHistoryResultRepository;
import com.bocloud.cos.repository.StorageInformationReportRepository;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.util.*;

/**
 * @Author hufy
 */
@Service("storageReportService")
public class StorageReportServiceImpl extends AbstractReportService implements StorageReportService {
	private static final Logger logger = LoggerFactory.getLogger(TenantReportServiceImpl.class);
	@Autowired
	private ReportConfigTemplateRepository reportConfigTemplateRepository;
	@Autowired
	private ParagraphRepository paragraphRepository;
	@Autowired
	private StorageInformationReportRepository storageInformationReportRepository;

	@Autowired
	private ReportHistoryResultRepository reportHistoryResultRepository;

	@Override
	public BocloudResult<List<ReportPair<List<StorageInformationReport>>>> queryDetail(Long configTemplateId,
			ReportTemplateHistory history, Boolean isRealTime) {
		List<ReportPair<List<StorageInformationReport>>> resultList = new ArrayList<>();
		ReportConfigTemplate configTemplate = null;
		ReportFrequenceType reportFrequenceType = null;
		Map<String, Object> param = new HashMap<>(16);
		try {
			if (isRealTime != null && isRealTime) {
				// 查看实时数据
				configTemplate = reportConfigTemplateRepository.query(configTemplateId);
				String cycleType = configTemplate.getCycleType();
				reportFrequenceType = ReportFrequenceType.valueOf(cycleType);
				// 处理时间参数
				extractTimeParam(null, reportFrequenceType, configTemplate, param, true);
			} else if (history != null) {
				// 查看历史数据
				configTemplateId = history.getConfigTemplateId();
				configTemplate = reportConfigTemplateRepository.query(configTemplateId);
				String cycleType = configTemplate.getCycleType();
				reportFrequenceType = ReportFrequenceType.valueOf(cycleType);
				// 处理时间参数
				extractTimeParam(history.getTriggerTime(), reportFrequenceType, configTemplate, param, false);
			}
			try {
				List<Paragraph> paragraphs = paragraphRepository.getByConfigTemplate(configTemplateId);
				for (Paragraph paragraph : paragraphs) {
					String resourceIdStr = paragraph.getResourceIds();
					String[] resourceIds = resourceIdStr.split(",");
					// 先查询出表格数据
					List<StorageInformationReport> reports = getMainTableReport(reportFrequenceType, param, resourceIds,
							null, null, configTemplate, paragraph, history);
					resultList.add(new ReportPair<>(paragraph.getName(), reports));
				}
			} catch (Exception e) {
				logger.error(ExceptionUtils.getFullStackTrace(e));
				return new BocloudResult<>(false, "查询失败:"+e.getMessage());
			}
		} catch (Exception e) {
			logger.error(ExceptionUtils.getFullStackTrace(e));
			return new BocloudResult<>(false, "查询失败:"+e.getMessage());
		}
		return new BocloudResult<>(true, resultList, "查询成功");
	}

	@Override
	public InputStream getMainTableJasperStream() throws Exception {
		// TODO Auto-generated method stub
		return this.getClass().getClassLoader().getResourceAsStream("template/storage/main.jasper");
	}

	@Override
	public String getTitleName() {
		return "存储综合报表";
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	@Override
	public List<StorageInformationReport> getMainTableReport(ReportFrequenceType reportFrequenceType,
			Map<String, Object> param, String[] resourceIds, RequestContext context, Boolean isAdmin,
			ReportConfigTemplate configTemplate, Paragraph paragraph, ReportTemplateHistory history) {
		List list = Collections.emptyList();
		try {
			/*
			 * 查询缓存的结果
			 */
			ReportHistoryResult cacheResult = reportHistoryResultRepository.query(history.getId(), paragraph.getId(),
					0L);
			if (cacheResult == null) {
				/*
				 * 缓存为空,计算数据，存入缓存
				 */
				list = storageInformationReportRepository.queryStorageInformationReport(resourceIds,
						reportFrequenceType, param);
				cacheResult = new ReportHistoryResult();
				String content = JSONObject.toJSONString(list);
				cacheResult.setContent(content);
				cacheResult.setHistoryId(history.getId());
				cacheResult.setContentTemplateId(0L);
				cacheResult.setParagraphId(paragraph.getId());
				reportHistoryResultRepository.save(cacheResult);
			} else {
				list = JSONArray.parseArray(cacheResult.getContent(), StorageInformationReport.class);
			}

		} catch (Exception e) {
			logger.error("查询失败", e);
		}
		return list;
	}
}
