package com.bocloud.cos.service.jasper.handler.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.common.ReportConstants;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.enums.ReportFrequenceType;
import com.bocloud.cos.intf.PlatformInfoReportService;
import com.bocloud.cos.intf.ReportCreateService;
import com.bocloud.cos.model.GraphData;
import com.bocloud.cos.repository.ReportHistoryResultRepository;
import com.bocloud.cos.service.PlatformInfoReportServiceImpl;
import com.bocloud.cos.service.jasper.utils.XyLineChartReportBuilder;
import net.sf.jasperreports.engine.JasperPrint;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * @author hufy
 */
public class PlatformJasperHandler extends AbstractJasperHandler {

    public static final String VM_CREATE = "虚拟机创建趋势图";
    private static Logger logger = LoggerFactory.getLogger(PlatformJasperHandler.class);

    private final ReportHistoryResultRepository reportHistoryResultRepository;

    public PlatformJasperHandler(ReportCreateService service) {
        super(service);
        this.reportHistoryResultRepository = ((PlatformInfoReportServiceImpl) this.getService()).getReportHistoryResultRepository();
    }

    @Override
    public JasperPrint getJasperPrint(ReportTemplateHistory history, Map<String, Object> params,
                                      ContentTemplate contentTemplate, ReportFrequenceType frequenceType, String[] resourceIds,
                                      Paragraph paragraph, ReportConfigTemplate configTemplate) {
        PlatformInfoReportService platformInfoReportService = (PlatformInfoReportService) getService();
        JasperPrint jasperPrint = null;
        String chartName = (String) params.get(ReportConstants.CHART_NAME);
        List<GraphData> graphDatas = null;
        try {
            ReportHistoryResult cacheResult = reportHistoryResultRepository.query(history.getId(), paragraph.getId(),
                    contentTemplate.getId());
            if (cacheResult == null) {
                switch (chartName) {
                    case VM_CREATE:
                        graphDatas = platformInfoReportService.getVmLineChartByTime(frequenceType, params, resourceIds);
                        break;
                    default:
                        return null;

                }
                if (graphDatas == null) {
                    graphDatas = Collections.emptyList();
                }
                cacheResult = new ReportHistoryResult();
                String content = JSONObject.toJSONString(graphDatas);
                cacheResult.setContent(content);
                cacheResult.setHistoryId(history.getId());
                cacheResult.setContentTemplateId(contentTemplate.getId());
                cacheResult.setParagraphId(paragraph.getId());
                // 如果不存在则保存数据
                reportHistoryResultRepository.save(cacheResult);
            } else {
                graphDatas = JSONArray.parseArray(cacheResult.getContent(), GraphData.class);
            }
        } catch (Exception e) {
            logger.error(ExceptionUtils.getFullStackTrace(e));
        }
        switch (chartName) {
            // 查询虚拟机创建数量
            case VM_CREATE:
                jasperPrint = new XyLineChartReportBuilder(graphDatas, VM_CREATE, "时间点", "数量").build();
                break;
            default:
                return null;

        }
        return jasperPrint;
    }
}
