package com.bocloud.cos.service.jasper.utils;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.common.utils.ListTool;
import com.bocloud.cos.model.GraphData;
import com.bocloud.cos.model.GraphSeries;
import net.sf.dynamicreports.report.builder.chart.AbstractCategoryChartSerieBuilder;
import net.sf.dynamicreports.report.builder.column.TextColumnBuilder;
import net.sf.dynamicreports.report.builder.style.FontBuilder;
import net.sf.dynamicreports.report.datasource.DRDataSource;
import net.sf.dynamicreports.report.exception.DRException;
import net.sf.jasperreports.engine.JRDataSource;
import net.sf.jasperreports.engine.JasperPrint;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.*;

import static net.sf.dynamicreports.report.builder.DynamicReports.*;

/**
 * @author hufy
 */
public class XyLineChartReportBuilder {

	private static Logger logger = LoggerFactory.getLogger(XyLineChartReportBuilder.class);

	private List<GraphData> lineData;

	private String title;

	private String xLabel;

	private String yLabel;

	public XyLineChartReportBuilder(List<GraphData> lineData, String title, String xLabel, String yLabel) {
		this.lineData = lineData;
		this.title = title;
		this.xLabel = xLabel;
		this.yLabel = yLabel;
	}

	public JasperPrint build() {
		JasperPrint jasperPrint = null;
		try {
			jasperPrint = toJasperData(lineData, title, xLabel, yLabel);
		} catch (Exception e) {
			logger.error(ExceptionUtils.getFullStackTrace(e));
		}
		return jasperPrint;

	}

	@SuppressWarnings("unchecked")
	public JasperPrint toJasperData(List<GraphData> lineDatas, String title, String xLabel, String yLabel) {
		JasperPrint jasperPrint = null;
		if (ListTool.isEmpty(lineDatas)) {
			return jasperPrint;
		}
		/**
		 * 找出时间段最长的
		 */
		int maxLength = 0;
		List<String> maxLengthTimeList = Collections.emptyList();
		for (GraphData lineData : lineDatas) {
			if (lineData.getTimes() != null) {
				int length = lineData.getTimes().size();
				if (length > maxLength) {
					maxLengthTimeList = lineData.getTimes();
					maxLength = length;
				}
			}
		}
		// 说明没有查到数据
		if (maxLength == 0) {
			return null;
		}
		Map<String, YaxisLineData> yLineDataMap = new HashMap<>(16);
		if (!ListTool.isEmpty(maxLengthTimeList)) {
			for (int i = 0; i < maxLengthTimeList.size(); i++) {
				for (int j = 0; j < lineDatas.size(); j++) {
					GraphData graphData = lineDatas.get(j);
					String name = graphData.getName();
					YaxisLineData yLineData = null;
					if (yLineDataMap.containsKey(name)) {
						yLineData = yLineDataMap.get(name);
					} else {
						yLineData = new YaxisLineData();
						yLineData.setName(name);
						yLineDataMap.put(name, yLineData);
					}
					List<String> times = graphData.getTimes();
					List<GraphSeries> values = graphData.getSeries();
					if (i < times.size()) {
						String time = times.get(i);
						if (time.equals(maxLengthTimeList.get(i))) {
							yLineData.getValues().add(Double.valueOf(String.valueOf(values.get(0).getData().get(i))));
						} else {
							// 补y轴数据
							yLineData.getValues().add(null);
						}
					} else {
						// 补y轴数据
						yLineData.getValues().add(null);
					}
				}
			}
		}
		if (yLineDataMap.size() > 0) {
			// 设置x轴
			TextColumnBuilder<String> xColumn = col.column(xLabel, "x", type.stringType());
			// 设置y轴
			// 第0个是x轴
			@SuppressWarnings("rawtypes")
			TextColumnBuilder[] xAndYColumns = new TextColumnBuilder[yLineDataMap.size() + 1];
			xAndYColumns[0] = xColumn;
			// y轴数据
			@SuppressWarnings("rawtypes")
			AbstractCategoryChartSerieBuilder[] yBuilders = new AbstractCategoryChartSerieBuilder[yLineDataMap.size()];

			Set<String> keySet = yLineDataMap.keySet();
			List<String> subTitle = new ArrayList<>(keySet);

			for (int i = 1; i <= subTitle.size(); i++) {
				xAndYColumns[i] = col.column(subTitle.get(i - 1), "y" + i, type.doubleType());
				yBuilders[i - 1] = cht.serie(xAndYColumns[i]);
			}
			try {
				FontBuilder boldFont = stl.fontArialBold().setFontSize(13).setFontName("myfont");
				jasperPrint = report().setTemplate(DynamicTemplates.REPORT_TEMPLATE).columns(xAndYColumns)
						.title(DynamicTemplates.createTitleComponent(title))
						.summary(cht.lineChart().setTitle(title).setTitleFont(boldFont).setSubtitleFont(boldFont)
								.setLegendFont(boldFont).setCategory(xColumn).series(yBuilders)
								.setCategoryAxisFormat(cht.axisFormat().setLabel(xLabel).setLabelFont(boldFont).setTickLabelRotation(60d))
								.setValueAxisFormat(cht.axisFormat().setLabel(yLabel).setLabelFont(boldFont)))
						.setDataSource(createDataSource(subTitle, yLineDataMap, xAndYColumns, maxLengthTimeList))
						.toJasperPrint();
			} catch (DRException e) {
				logger.error(ExceptionUtils.getFullStackTrace(e));
			}
		}

		return jasperPrint;
	}

	private JRDataSource createDataSource(List<String> subTitle, Map<String, YaxisLineData> yLineDataListMap,
			@SuppressWarnings("rawtypes") TextColumnBuilder[] xyColumns, List<String> timeList) {
		String[] colums = new String[xyColumns.length];
		colums[0] = "x";
		for (int i = 1; i < xyColumns.length; i++) {
			colums[i] = "y" + i;
		}
		List<YaxisLineData> yLineDataList = new ArrayList<>();
		for (int i = 0; i < subTitle.size(); i++) {
			yLineDataList.add(yLineDataListMap.get(subTitle.get(i)));
		}
		DRDataSource dataSource = new DRDataSource(colums);
		for (int i = 0; i < timeList.size(); i++) {
			Object[] params = new Object[yLineDataList.size() + 1];
			params[0] = timeList.get(i);
			for (int j = 0; j < yLineDataList.size(); j++) {
				params[j + 1] = yLineDataList.get(j).getValues().get(i);
			}
			dataSource.add(params);
		}
		return dataSource;
	}

	public static void main(String[] args) {
		try {
			List<GraphData> graphData = JSONArray.parseArray(
					IOUtils.toString(new FileReader(new File(
							"D:\\new_repo\\bocloud.cos\\bocloud.cos.server\\src\\main\\resources\\test.json"))),
					GraphData.class);
			new XyLineChartReportBuilder(graphData, "haha", "ceshi", "nikan");
		} catch (IOException e) {
			logger.error(ExceptionUtils.getFullStackTrace(e));
		}

	}

	public static class YaxisLineData {

		private String name;
		private List<Double> values = new ArrayList<>();

		public String getName() {
			return name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public List<Double> getValues() {
			return values;
		}

		public void setValues(List<Double> values) {
			this.values = values;
		}
	}
}
