package com.bocloud.cos.service.resource.operate;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.OrderItem;
import com.bocloud.cos.enums.OrderItemStatus;
import com.bocloud.cos.enums.ResourceCategory;
import com.bocloud.cos.intf.OrderSnService;
import com.bocloud.cos.model.DurationModel;
import com.bocloud.cos.model.ElementModel;
import com.bocloud.cos.model.EmptionModel;
import com.bocloud.cos.model.ServicePrice;
import com.bocloud.cos.repository.CloudServiceRepository;
import com.bocloud.cos.repository.OrderItemRepository;
import com.bocloud.cos.repository.ServiceCategoryRepository;
import com.bocloud.cos.repository.ServiceSkuRepository;
import com.bocloud.cos.service.utils.IdGenerator;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author tyl
 * @since 2021/5/21 15:16
 */
public abstract class AbstractTaskResourceOperate extends AbstractResourceOperate {

    /**
     * 资源项
     */
    List<ElementModel> elements;
    /**
     * 时长 订购参数
     */
    EmptionModel emptionModel;
    /**
     * 服务编码
     */
    final String serviceCode;
    final Logger logger = LoggerFactory.getLogger(AbstractPaasResourceOperate.class);
    CloudServiceRepository serviceRepository;
    ServiceCategoryRepository categoryRepository;
    ServiceSkuRepository skuRepository;
    OrderItemRepository itemRepository;
    LockFactory lockFactory;
    OrderSnService orderSnService;

    public AbstractTaskResourceOperate(ResourceCategory category, List<ElementModel> elements,
        EmptionModel emptionModel, String serviceCode, CloudServiceRepository serviceRepository,
        ServiceSkuRepository skuRepository, OrderItemRepository itemRepository,
        ServiceCategoryRepository categoryRepository, LockFactory lockFactory, OrderSnService orderSnService) {
        super(category);
        this.elements = elements;
        this.emptionModel = emptionModel;
        this.serviceCode = serviceCode;
        this.serviceRepository = serviceRepository;
        this.skuRepository = skuRepository;
        this.itemRepository = itemRepository;
        this.categoryRepository = categoryRepository;
        this.lockFactory = lockFactory;
        this.orderSnService = orderSnService;
    }

    public AbstractTaskResourceOperate(ResourceCategory category, String serviceCode) {
        super(category);
        this.serviceCode = serviceCode;
    }

    @Override public List<OrderItem> buildApplyItem() {
        List<OrderItem> items = new ArrayList<>();
        try {
            DurationModel duration = emptionModel.getDuration();
            // 订购数量
            Integer count = emptionModel.getCount();
            Assert.isTrue(null != count && count > 0, "订购数量不能小于0");
            //构建新的订单项
            OrderItem orderItem = new OrderItem();
            orderItem.setSkuId(null);
            String itemSn = this.getItemSn(null);
            Assert.notNull(itemSn, "生成订购单号失败");
            orderItem.setSn(itemSn);
            orderItem.setServiceCode(serviceCode);
            orderItem.setStatus(OrderItemStatus.Opening);
            orderItem.setDuration(JSONObject.toJSONString(duration));
            orderItem.setDeleted(false);
            orderItem.setResources("[]");
            orderItem.setElements("[]");
            CloudService cloudService = serviceRepository.queryByCode(serviceCode);
            BigDecimal basicPrice = getBasicPrice(duration, cloudService);
            orderItem.setBasicPrice(basicPrice);
            items.add(orderItem);
        } catch (Exception e) {
            logger.error("build  apply item error", e);
        }
        return items;
    }

    private BigDecimal getBasicPrice(DurationModel duration, CloudService cloudService) {
        List<ServicePrice> servicePrices = JSONArray.parseArray(cloudService.getBasicPrice(), ServicePrice.class);
        if (!cloudService.getBillable() || servicePrices == null || servicePrices.isEmpty()) {
            return BigDecimal.ZERO;
        }
        Map<String, ServicePrice> specBasicMap = servicePrices.stream().collect(Collectors.toMap(ServicePrice::getSpecName, Function.identity()));
        ServicePrice servicePrice = null;
        for (Map.Entry<String, ServicePrice> entry : specBasicMap.entrySet()) {
            servicePrice = entry.getValue();
            break;
        }
        if (servicePrice == null){
            return BigDecimal.ZERO;
        }
        DurationModel.Mode mode = duration.getMode();
        BigDecimal basicPrice;
        switch (mode) {
            case Year:
                basicPrice = servicePrice.getYearPrice();
                break;
            case Month:
                basicPrice = servicePrice.getMonthPrice();
                break;
            default:
                basicPrice = BigDecimal.ZERO;
                break;
        }
        return basicPrice;
    }

    @Override public List<OrderItem> buildAlterationItem() {
        return null;
    }

    public String getItemSn(Long skuId) {
        if (null == skuId || 0L == skuId) {
            // 生成订单号，增加分布式锁
            String itemPath = IdGenerator.class.getSimpleName() + "_order_item_sn";
            try (BocloudACLock lock = lockFactory.getACLock(itemPath)) {
                Assert.isTrue(lock.acquire(1, TimeUnit.MINUTES), "生成订单号请求超时");
                return IdGenerator.getInstance().nextId();
            } catch (Exception e) {
                logger.error("Get error message：{}", e.getMessage());
                return null;
            }
        } else {
            BocloudResult<String> snResult = orderSnService.generate(skuId);
            Assert.isTrue(snResult.isSuccess(), snResult.getMessage());
            return snResult.getData();
        }
    }
}
