package com.bocloud.cos.service.resource.operate;

import com.bocloud.cos.enums.ResourceCategory;
import com.bocloud.cos.intf.OrderSnService;
import com.bocloud.cos.model.AlterationParam;
import com.bocloud.cos.model.ElementModel;
import com.bocloud.cos.model.EmptionModel;
import com.bocloud.cos.repository.*;
import com.bocloud.service.lock.LockFactory;

import java.util.Date;
import java.util.List;

/**
 * @author tyl
 * @since 2021/5/21 14:48
 */
public class ResourceOperateUtils {

    /**
     * 获取资源申请操作对象
     * 1
     *
     * @param serviceCode       申请资源服务编码
     * @param emptionModel      订购参数（时间 数量）
     * @param elements          订购规格
     * @param skuRepository
     * @param itemRepository
     * @param serviceRepository
     * @return
     */
    public static AbstractResourceOperate getApplyOperate(String serviceCode, EmptionModel emptionModel,
        List<ElementModel> elements, CloudServiceRepository serviceRepository, ServiceSkuRepository skuRepository,
        OrderItemRepository itemRepository, ServiceCategoryRepository categoryRepository, LockFactory lockFactory,
        OrderSnService orderSnService) {
        if (serviceCode.startsWith("operations.")) {
            return new CosOperate(ResourceCategory.Operations, elements, emptionModel, serviceCode, serviceRepository,
                skuRepository, itemRepository, categoryRepository, lockFactory, orderSnService);
        } else if (serviceCode.startsWith("fabric.")) {
            return new RosOperate(ResourceCategory.Fabric, elements, emptionModel, serviceCode, serviceRepository,
                skuRepository, itemRepository, categoryRepository, lockFactory, orderSnService);
        }

        switch (serviceCode) {
            case "rds.mysql":
                return new MySQLOperate(serviceCode, ResourceCategory.MySQL, emptionModel, elements, serviceRepository,
                    skuRepository, categoryRepository, lockFactory, orderSnService);
            case "rds.sqlserver":
                return new SQLServerOperate(serviceCode, ResourceCategory.SqlServer, emptionModel, elements,
                    serviceRepository, skuRepository, categoryRepository, lockFactory, orderSnService);
            case "kvs.redis":
                return new RedisOperate(serviceCode, ResourceCategory.Redis, emptionModel, elements, serviceRepository,
                    skuRepository, categoryRepository, lockFactory, orderSnService);
            case "kvs.mongodb":
                return new MongoDBOperate(serviceCode, ResourceCategory.MongoDB, emptionModel, elements,
                    serviceRepository, skuRepository, categoryRepository, lockFactory, orderSnService);
            case "cms.kafka":
                return new KafkaOperate(serviceCode, ResourceCategory.Kafka, emptionModel, elements, serviceRepository,
                    skuRepository, categoryRepository, lockFactory, orderSnService);
            case "cms.rabbitmq":
                return new RabbitMQOperate(serviceCode, ResourceCategory.RabbitMQ, emptionModel, elements,
                    serviceRepository, skuRepository, categoryRepository, lockFactory, orderSnService);
            case "rds.postgresql":
                return new PostgreSQLOperate(serviceCode, ResourceCategory.PostgreSQL, emptionModel, elements,
                    serviceRepository, skuRepository, categoryRepository, lockFactory, orderSnService);
            case "vmware.nsx.vpc":
            case "openstack.standard.vpc":
            case "tce.standard.vpc":
            case "manageone.standard.vpc":
            case "huawei.standard.vpc":
            case "tencent.standard.vpc":
            case "aliyun.standard.vpc":
            case "azure.standard.vpc":
            case "aws.standard.vpc":
                return new VpcOperate(serviceCode, ResourceCategory.Vpc, emptionModel, elements, serviceRepository,
                    skuRepository, categoryRepository, lockFactory, orderSnService);
            case "network.aliyun.slb":
            case "network.aws.slb":
                return new SlbOperate(serviceCode, ResourceCategory.LoadBalance, emptionModel, elements,
                    serviceRepository, skuRepository, categoryRepository, lockFactory, orderSnService);
            case "manageone.standard.eip":
            case "openstack.standard.eip":
            case "fusioncloud.standard.eip":
            case "network.aliyun.eip":
            case "network.huawei.eip":
            case "network.tencent.eip":
            case "network.tce.eip":
            case "network.qcloud.eip":
            case "network.aws.eip":
                return new EipOperate(serviceCode, ResourceCategory.FloatingIp, emptionModel, elements,
                    serviceRepository, skuRepository, categoryRepository, lockFactory, orderSnService);
            case "vmware.k8s.cluster":
                return new K8sOperate(serviceCode, ResourceCategory.Kubernetes, emptionModel, elements,
                    serviceRepository, skuRepository, categoryRepository, lockFactory, orderSnService);
            case "vmware.standard.server":
            case "openstack.standard.server":
            case "aliyun.standard.server":
            case "huawei.standard.server":
            case " fusioncloud.standard.server":
            case "manageone.standard.server":
            case "tce.standard.server":
            case "tencent.standard.server":
            case "azure.standard.server":
            case "aws.standard.server":
                return new EcsOperate(serviceCode, ResourceCategory.Computer, emptionModel, elements, serviceRepository,
                    skuRepository, categoryRepository, lockFactory, orderSnService);
            case "openstack.standard.volume":
            case "manageone.standard.volume":
            case "fusioncloud.standard.volume":
            case "aliyun.standard.volume":
            case "tce.standard.cbs":
            case "huawei.standard.volume":
            case "tencent.standard.volume":
            case "azure.standard.volume":
            case "aws.standard.volume":
                return new StorageOperate(serviceCode, ResourceCategory.Storage, emptionModel, elements,
                    serviceRepository, skuRepository, categoryRepository, lockFactory, orderSnService);
            default:
                break;
        }
        return null;
    }

    /**
     * 获取资源变更操作对象
     *
     * @param serviceCode       变更资源服务编码
     * @param instance          变更实例id
     * @param elements          变更规格
     * @param operation         变更类型
     * @param skuRepository
     * @param itemRepository
     * @param serviceRepository
     * @return
     */
    public static AbstractResourceOperate getAlterationOperate(String serviceCode, String instance,
        AlterationParam.Operation operation, List<ElementModel> elements, Date expiredTime, CloudServiceRepository serviceRepository,
        ServiceSkuRepository skuRepository, OrderItemRepository itemRepository, LockFactory lockFactory,
        OrderSnService orderSnService, ServiceCategoryRepository categoryRepository) {
        switch (serviceCode) {
            case "rds.mysql":
                return new MySQLOperate(serviceCode, instance, ResourceCategory.MySQL, operation, elements,expiredTime,
                    serviceRepository, skuRepository, itemRepository, lockFactory, orderSnService, categoryRepository);
            case "rds.sqlserver":
                return new SQLServerOperate(serviceCode, instance, ResourceCategory.SqlServer, operation, elements,expiredTime,
                    serviceRepository, skuRepository, itemRepository, lockFactory, orderSnService, categoryRepository);
            case "kvs.redis":
                return new RedisOperate(serviceCode, instance, ResourceCategory.Redis, operation, elements,expiredTime,
                    serviceRepository, skuRepository, itemRepository, lockFactory, orderSnService, categoryRepository);
            case "kvs.mongodb":
                return new MongoDBOperate(serviceCode, instance, ResourceCategory.MongoDB, operation, elements,expiredTime,
                    serviceRepository, skuRepository, itemRepository, lockFactory, orderSnService, categoryRepository);
            case "cms.kafka":
                return new KafkaOperate(serviceCode, instance, ResourceCategory.Kafka, operation, elements,expiredTime,
                    serviceRepository, skuRepository, itemRepository, lockFactory, orderSnService, categoryRepository);
            case "cms.rabbitmq":
                return new RabbitMQOperate(serviceCode, instance, ResourceCategory.RabbitMQ, operation, elements,expiredTime,
                    serviceRepository, skuRepository, itemRepository, lockFactory, orderSnService, categoryRepository);
            case "rds.postgresql":
                return new PostgreSQLOperate(serviceCode, instance, ResourceCategory.PostgreSQL, operation, elements,expiredTime,
                    serviceRepository, skuRepository, itemRepository, lockFactory, orderSnService, categoryRepository);
            case "vmware.k8s.cluster":
                return new K8sOperate(serviceCode, instance, ResourceCategory.Kubernetes, operation, elements,expiredTime,
                    serviceRepository, skuRepository, itemRepository, lockFactory, orderSnService, categoryRepository);
            case "vmware.standard.server":
            case "openstack.standard.server":
            case "aliyun.standard.server":
            case "huawei.standard.server":
            case " fusioncloud.standard.server":
            case "manageone.standard.server":
            case "tce.standard.server":
            case "tencent.standard.server":
            case "azure.standard.server":
            case "aws.standard.server":
                return new EcsOperate(serviceCode, instance, ResourceCategory.Computer, operation, elements,expiredTime,
                    serviceRepository, skuRepository, itemRepository, lockFactory, orderSnService, categoryRepository);
            case "openstack.standard.volume":
            case "manageone.standard.volume":
            case "fusioncloud.standard.volume":
            case "aliyun.standard.volume":
            case "tce.standard.cbs":
            case "huawei.standard.volume":
            case "tencent.standard.volume":
            case "azure.standard.volume":
            case "aws.standard.volume":
                return new StorageOperate(serviceCode, instance, ResourceCategory.Storage, operation, elements,expiredTime,
                    serviceRepository, skuRepository, itemRepository, lockFactory, orderSnService, categoryRepository);
            default:
                break;
        }
        return null;
    }

    /**
     * 获取资源延期操作对象
     *
     * @param serviceCode
     * @param instance
     * @param extension
     * @param itemRepository
     * @param orderRepository
     * @return
     */
    public static AbstractResourceOperate getExtensionOperate(String serviceCode, String instance, Integer extension,
        OrderItemRepository itemRepository, ServiceOrderRepository orderRepository, LockFactory lockFactory,
        OrderSnService orderSnService) {

        switch (serviceCode) {
            case "rds.mysql":
                return new MySQLOperate(serviceCode, instance, extension, ResourceCategory.MySQL, itemRepository,
                    orderRepository, lockFactory, orderSnService);
            case "rds.sqlserver":
                return new SQLServerOperate(serviceCode, instance, extension, ResourceCategory.SqlServer,
                    itemRepository, orderRepository, lockFactory, orderSnService);
            case "rds.postgresql":
                return new PostgreSQLOperate(serviceCode, instance, extension, ResourceCategory.PostgreSQL,
                    itemRepository, orderRepository, lockFactory, orderSnService);
            case "kvs.redis":
                return new RedisOperate(serviceCode, instance, extension, ResourceCategory.Redis, itemRepository,
                    orderRepository, lockFactory, orderSnService);
            case "kvs.mongodb":
                return new MongoDBOperate(serviceCode, instance, extension, ResourceCategory.MongoDB, itemRepository,
                    orderRepository, lockFactory, orderSnService);
            case "cms.kafka":
                return new KafkaOperate(serviceCode, instance, extension, ResourceCategory.Kafka, itemRepository,
                    orderRepository, lockFactory, orderSnService);
            case "cms.rabbitmq":
                return new RabbitMQOperate(serviceCode, instance, extension, ResourceCategory.RabbitMQ, itemRepository,
                    orderRepository, lockFactory, orderSnService);
            case "vmware.nsx.vpc":
            case "openstack.standard.vpc":
            case "tce.standard.vpc":
            case "manageone.standard.vpc":
            case "huawei.standard.vpc":
            case "tencent.standard.vpc":
            case "aliyun.standard.vpc":
            case "azure.standard.vpc":
            case "aws.standard.vpc":
                return new VpcOperate(serviceCode, instance, extension, ResourceCategory.Vpc, itemRepository,
                    orderRepository, lockFactory, orderSnService);
            case "manageone.standard.eip":
            case "openstack.standard.eip":
            case "fusioncloud.standard.eip":
            case "network.aliyun.eip":
            case "network.huawei.eip":
            case "network.tencent.eip":
            case "network.tce.eip":
            case "network.qcloud.eip":
            case "network.aws.eip":
                return new EipOperate(serviceCode, instance, extension, ResourceCategory.FloatingIp, itemRepository,
                    orderRepository, lockFactory, orderSnService);
            case "vmware.k8s.cluster":
                return new K8sOperate(serviceCode, instance, extension, ResourceCategory.Kubernetes, itemRepository,
                    orderRepository, lockFactory, orderSnService);
            case "vmware.standard.server":
            case "openstack.standard.server":
            case "aliyun.standard.server":
            case "huawei.standard.server":
            case " fusioncloud.standard.server":
            case "manageone.standard.server":
            case "tce.standard.server":
            case "tencent.standard.server":
            case "azure.standard.server":
            case "aws.standard.server":
                return new EcsOperate(serviceCode, instance, extension, ResourceCategory.Computer, itemRepository,
                    orderRepository, lockFactory, orderSnService);
            case "openstack.standard.volume":
            case "manageone.standard.volume":
            case "fusioncloud.standard.volume":
            case "aliyun.standard.volume":
            case "tce.standard.cbs":
            case "huawei.standard.volume":
            case "tencent.standard.volume":
            case "azure.standard.volume":
            case "aws.standard.volume":
                return new StorageOperate(serviceCode, instance, extension, ResourceCategory.Storage, itemRepository,
                    orderRepository, lockFactory, orderSnService);
            default:
                break;
        }
        return null;
    }

    public static AbstractResourceOperate getResourceOperate(String serviceCode) {
        if (serviceCode.startsWith("operations.")) {
            return new CosOperate(ResourceCategory.Operations, serviceCode);
        } else if (serviceCode.startsWith("fabric.")) {
            return new RosOperate(ResourceCategory.Fabric, serviceCode);
        }
        switch (serviceCode) {
            case "rds.mysql":
                return new MySQLOperate(ResourceCategory.MySQL, serviceCode);
            case "rds.sqlserver":
                return new SQLServerOperate(ResourceCategory.SqlServer, serviceCode);
            case "kvs.redis":
                return new RedisOperate(ResourceCategory.Redis, serviceCode);
            case "kvs.mongodb":
                return new MongoDBOperate(ResourceCategory.MongoDB, serviceCode);
            case "cms.kafka":
                return new KafkaOperate(ResourceCategory.Kafka, serviceCode);
            case "cms.rabbitmq":
                return new RabbitMQOperate(ResourceCategory.RabbitMQ, serviceCode);
            case "rds.postgresql":
                return new PostgreSQLOperate(ResourceCategory.PostgreSQL, serviceCode);
            case "vmware.nsx.vpc":
            case "openstack.standard.vpc":
            case "tce.standard.vpc":
            case "manageone.standard.vpc":
            case "huawei.standard.vpc":
            case "tencent.standard.vpc":
            case "aliyun.standard.vpc":
            case "azure.standard.vpc":
            case "aws.standard.vpc":
                return new VpcOperate(ResourceCategory.Vpc, serviceCode);
            case "network.aliyun.slb":
            case "network.aws.slb":
                return new SlbOperate(ResourceCategory.LoadBalance, serviceCode);
            case "manageone.standard.eip":
            case "openstack.standard.eip":
            case "fusioncloud.standard.eip":
            case "network.aliyun.eip":
            case "network.huawei.eip":
            case "network.tencent.eip":
            case "network.tce.eip":
            case "network.qcloud.eip":
            case "network.aws.eip":
                return new EipOperate(ResourceCategory.FloatingIp, serviceCode);
            case "vmware.k8s.cluster":
                return new K8sOperate(ResourceCategory.Kubernetes, serviceCode);
            case "vmware.standard.server":
            case "openstack.standard.server":
            case "aliyun.standard.server":
            case "huawei.standard.server":
            case " fusioncloud.standard.server":
            case "manageone.standard.server":
            case "tce.standard.server":
            case "tencent.standard.server":
            case "azure.standard.server":
            case "aws.standard.server":
                return new EcsOperate(ResourceCategory.Computer, serviceCode);
            case "openstack.standard.volume":
            case "manageone.standard.volume":
            case "fusioncloud.standard.volume":
            case "aliyun.standard.volume":
            case "tce.standard.cbs":
            case "huawei.standard.volume":
            case "tencent.standard.volume":
            case "azure.standard.volume":
            case "aws.standard.volume":
                return new StorageOperate(ResourceCategory.Storage, serviceCode);
            default:
                break;
        }
        return null;
    }
}
