package com.bocloud.cos.service.resource.operate;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.OrderItem;
import com.bocloud.cos.entity.ServiceCategory;
import com.bocloud.cos.entity.ServiceSku;
import com.bocloud.cos.enums.OrderItemStatus;
import com.bocloud.cos.enums.ResourceCategory;
import com.bocloud.cos.intf.OrderSnService;
import com.bocloud.cos.model.DurationModel;
import com.bocloud.cos.model.ElementModel;
import com.bocloud.cos.model.EmptionModel;
import com.bocloud.cos.repository.CloudServiceRepository;
import com.bocloud.cos.repository.ServiceCategoryRepository;
import com.bocloud.cos.repository.ServiceSkuRepository;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * @author tyl
 * @since 2021/5/21 10:40
 */
public class SlbOperate extends AbstractIaasResourceOperate {

    public SlbOperate(String serviceCode, ResourceCategory category, EmptionModel emptionModel,
        List<ElementModel> elements, CloudServiceRepository serviceRepository, ServiceSkuRepository skuRepository,
        ServiceCategoryRepository categoryRepository, LockFactory lockFactory, OrderSnService orderSnService) {
        super(serviceCode, category, emptionModel, elements, serviceRepository, skuRepository, categoryRepository,
            lockFactory, orderSnService);
    }

    public SlbOperate(ResourceCategory category, String serviceCode) {
        super(category, serviceCode);
    }

    @Override public List<OrderItem> buildApplyItem() {
        List<OrderItem> items = new ArrayList<>();
        try {
            DurationModel duration = emptionModel.getDuration();
            CloudService orderService = serviceRepository.queryByCode(serviceCode);
            Assert.notEmpty(elements, "订购内容不能为空");
            // 订购数量
            Integer count = emptionModel.getCount();
            Assert.isTrue(null != count && count > 0, "订购数量不能小于0");
            for (int i = 0; i < count; i++) {
                for (ElementModel element : elements) {
                    String elementServiceCode = element.getServiceCode();
                    CloudService elementService = serviceRepository.queryByCode(element.getServiceCode());
                    Long skuId = element.getSkuId();
                    ServiceSku sku = skuRepository.query(skuId);
                    BigDecimal price = OrderPriceUtils.getUnitPrice(elementService, sku, element.getSpecs(), duration);
                    element.setPrice(price);
                    if (null != element.getCategoryId() && 0 != element.getCategoryId()) {
                        ServiceCategory category = categoryRepository.query(element.getCategoryId());
                        element.setCategory(category.getName());
                    } else {
                        element.setCategory(elementService.getName());
                    }
                    // 不开启计费则子订单计费为0
                    if (!orderService.getBillable()) {
                        price = BigDecimal.ZERO;
                    }
                    //构建新的订单项
                    OrderItem orderItem = new OrderItem();
                    orderItem.setSkuId(skuId);
                    String itemSn = super.getItemSn(skuId);
                    Assert.notNull(itemSn, "生成订单号失败");
                    orderItem.setSn(itemSn);
                    orderItem.setServiceCode(elementServiceCode);
                    orderItem.setStatus(OrderItemStatus.Opening);
                    orderItem.setDuration(JSONObject.toJSONString(duration));
                    orderItem.setDeleted(false);
                    orderItem.setResources("[]");
                    orderItem.setElements(JSONObject.toJSONString(Lists.newArrayList(element)));
                    orderItem.setBasicPrice(price);
                    items.add(orderItem);
                }
            }
        } catch (Exception e) {
            logger.error("build  apply item error", e);
        }
        return items;
    }
}
