package com.bocloud.cos.service.resource.operate;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.OrderItem;
import com.bocloud.cos.entity.ServiceCategory;
import com.bocloud.cos.entity.ServiceSku;
import com.bocloud.cos.enums.OrderItemStatus;
import com.bocloud.cos.enums.ResourceCategory;
import com.bocloud.cos.intf.OrderSnService;
import com.bocloud.cos.model.AlterationParam;
import com.bocloud.cos.model.DurationModel;
import com.bocloud.cos.model.ElementModel;
import com.bocloud.cos.model.EmptionModel;
import com.bocloud.cos.repository.*;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author tyl
 * @since 2021/5/21 10:40
 */
public class StorageOperate extends AbstractIaasResourceOperate {

    public StorageOperate(String serviceCode, ResourceCategory category, EmptionModel emptionModel,
        List<ElementModel> elements, CloudServiceRepository serviceRepository, ServiceSkuRepository skuRepository,
        ServiceCategoryRepository categoryRepository, LockFactory lockFactory, OrderSnService orderSnService) {
        super(serviceCode, category, emptionModel, elements, serviceRepository, skuRepository, categoryRepository,
            lockFactory, orderSnService);
    }

    public StorageOperate(String serviceCode, String instance, ResourceCategory category,
        AlterationParam.Operation operation, List<ElementModel> elements, Date expiredTime,
        CloudServiceRepository serviceRepository, ServiceSkuRepository skuRepository,
        OrderItemRepository itemRepository, LockFactory lockFactory, OrderSnService orderSnService,
        ServiceCategoryRepository categoryRepository) {
        super(serviceCode, instance, category, operation, elements, expiredTime, serviceRepository, skuRepository,
            itemRepository, lockFactory, orderSnService, categoryRepository);
    }

    public StorageOperate(String serviceCode, String instance, Integer extension, ResourceCategory category,
        OrderItemRepository itemRepository, ServiceOrderRepository orderRepository, LockFactory lockFactory,
        OrderSnService orderSnService) {
        super(serviceCode, instance, extension, category, itemRepository, orderRepository, lockFactory, orderSnService);
    }

    public StorageOperate(ResourceCategory category, String serviceCode) {
        super(category, serviceCode);
    }

    @Override public List<OrderItem> buildApplyItem() {
        List<OrderItem> items = new ArrayList<>();
        try {
            DurationModel duration = emptionModel.getDuration();
            Assert.notEmpty(elements, "订购内容不能为空");
            // 订购数量
            Integer count = emptionModel.getCount();
            Assert.isTrue(null != count && count > 0, "订购数量不能小于0");
            for (int i = 0; i < count; i++) {
                for (ElementModel element : elements) {
                    String elementServiceCode = element.getServiceCode();
                    CloudService elementService = serviceRepository.queryByCode(element.getServiceCode());
                    Long skuId = element.getSkuId();
                    ServiceSku sku = skuRepository.query(skuId);
                    BigDecimal price = OrderPriceUtils.getUnitPrice(elementService, sku, element.getSpecs(), duration);
                    element.setPrice(price);
                    if (null != element.getCategoryId() && 0 != element.getCategoryId()) {
                        ServiceCategory category = categoryRepository.query(element.getCategoryId());
                        element.setCategory(category.getName());
                    } else {
                        element.setCategory(elementService.getName());
                    }
                    //构建新的订单项
                    OrderItem orderItem = new OrderItem();
                    orderItem.setSkuId(skuId);
                    String addItemSn = this.getItemSn(skuId);
                    Assert.notNull(addItemSn, "生成订购单号失败");
                    orderItem.setSn(addItemSn);
                    orderItem.setServiceCode(elementServiceCode);
                    orderItem.setStatus(OrderItemStatus.Opening);
                    orderItem.setDuration(JSONObject.toJSONString(duration));
                    orderItem.setDeleted(false);
                    orderItem.setResources("[]");
                    orderItem.setElements(JSONObject.toJSONString(Lists.newArrayList(element)));
                    orderItem.setBasicPrice(price);
                    items.add(orderItem);
                }
            }
        } catch (Exception e) {
            logger.error("build  apply item error", e);
        }
        return items;
    }

    @Override public List<OrderItem> buildAlterationItem() {
        //查询旧的订单
        OrderItem oldItem = itemRepository.queryLastSuccessBySn(instance);
        DurationModel duration = JSONObject.parseObject(oldItem.getDuration(), DurationModel.class);
        Assert.notEmpty(elements, "变更内容不能为空");
        //变更的服务
        try {
            switch (operation) {
                case AddDisk:
                    return this.handlerAddDiskItems(oldItem, duration, elements);
                case EditDisk:
                    OrderItem orderItem = this.handlerEditItem(oldItem, duration, elements);
                    return Lists.newArrayList(orderItem);
                default:
                    break;
            }
            return null;
        } catch (Exception e) {
            logger.error("build alteration item error,", e);
            return null;
        }
    }

    private List<OrderItem> handlerAddDiskItems(OrderItem oldItem, DurationModel duration,
        List<ElementModel> elements) {
        List<OrderItem> items = new ArrayList<>();
        for (ElementModel element : elements) {
            Long elementSkuId = element.getSkuId();
            element.setCategory(element.getCategory());
            CloudService elementService = serviceRepository.queryByCode(element.getServiceCode());
            ServiceSku sku = skuRepository.query(elementSkuId);
            BigDecimal price = OrderPriceUtils.getUnitPrice(elementService, sku, element.getSpecs(), duration);
            element.setPrice(price);
            // 构建新添加磁盘的订单项
            OrderItem addDiskItem = new OrderItem();
            addDiskItem.setSkuId(elementSkuId);
            String addItemSn = this.getItemSn(elementSkuId);
            Assert.notNull(addItemSn, "生成订购单号失败");
            addDiskItem.setSn(addItemSn);
            element.setUuid(addItemSn);
            addDiskItem.setOrderId(oldItem.getOrderId());
            addDiskItem.setServiceCode(element.getServiceCode());
            addDiskItem.setStatus(OrderItemStatus.Changing);
            addDiskItem.setDuration(oldItem.getDuration());
            addDiskItem.setDeleted(false);
            addDiskItem.setResources("[]");
            addDiskItem.setElements(JSONObject.toJSONString(Lists.newArrayList(element)));
            addDiskItem.setEffectiveTime(new Date());
            addDiskItem.setExpiredTime(oldItem.getExpiredTime());
            addDiskItem.setBasicPrice(price);
            items.add(addDiskItem);
        }
        return items;
    }

    private OrderItem handlerEditItem(OrderItem oldItem, DurationModel duration, List<ElementModel> elements) {
        ElementModel element = elements.get(0);
        element.setCategory(element.getCategory());
        //变更的服务
        Long elementSkuId = element.getSkuId();
        CloudService alteraService = serviceRepository.queryByCode(element.getServiceCode());
        ServiceSku sku = skuRepository.query(elementSkuId);
        BigDecimal price = OrderPriceUtils.getUnitPrice(alteraService, sku, element.getSpecs(), duration);
        element.setPrice(price);
        // 不开启计费则子订单计费为0
        if (!alteraService.getBillable()) {
            price = BigDecimal.ZERO;
        }
        //构建新添加磁盘的订单项
        OrderItem orderItem = new OrderItem();
        orderItem.setSkuId(elementSkuId);
        String itemSn = super.getItemSn(elementSkuId);
        Assert.notNull(itemSn, "生成订单号失败");
        orderItem.setSn(itemSn);
        orderItem.setServiceCode(element.getServiceCode());
        orderItem.setStatus(OrderItemStatus.Opening);
        orderItem.setDuration(oldItem.getDuration());
        orderItem.setOrderId(orderItem.getOrderId());
        orderItem.setDeleted(false);
        orderItem.setResources("[]");
        orderItem.setElements(JSONObject.toJSONString(Lists.newArrayList(elements)));
        orderItem.setBasicPrice(price);
        return orderItem;
    }
}
