package com.bocloud.cos.service.task;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import com.bocloud.common.model.Param;
import com.bocloud.common.utils.ListTool;
import com.bocloud.cos.entity.Cos;
import com.bocloud.cos.entity.Ros;
import com.bocloud.cos.repository.CosRepository;
import com.bocloud.cos.repository.RosRepository;
import com.bocloud.cos.service.internal.SmsSystemConfigService;
import com.bocloud.service.core.CurrentService;

/**
 * 资源到期处理
 *
 * @author weiwei
 */
@Component
public class ResourceExpireScheduler {

	private static Logger logger = LoggerFactory.getLogger(ResourceExpireScheduler.class);
	@Autowired
	private CurrentService service;
	@Autowired
	private CosRepository cosRepository;
	@Autowired
	private RosRepository rosRepository;
	@Autowired
	private SmsSystemConfigService smsSystemConfigService;
	@Autowired
	private AmqpTemplate amqpTemplate;

	@Scheduled(cron = "0 0 0 * * ?")
	protected void warn() {
		logger.info("当前服务是否是leader：{}", service.isLeader());
		if (service.isLeader()) {
			List<Param> params = new ArrayList<>();
			List<Cos> coss = null;
			List<Ros> ross = null;
			try {
				coss = cosRepository.list(1, Integer.MAX_VALUE, null, params);
				ross = rosRepository.list(1, Integer.MAX_VALUE, null, params);
			} catch (Exception e) {
				return;
			}
			if (!ListTool.isEmpty(coss)) {
				for (Cos cos : coss) {
					AbstractExpiredTask task = new CosExpiredTask(cos, smsSystemConfigService, amqpTemplate,
							cosRepository);
					task.step();
				}
			}
			if (!ListTool.isEmpty(ross)) {
				for (Ros ros : ross) {
					AbstractExpiredTask task = new RosExpiredTask(ros, smsSystemConfigService, amqpTemplate,
							rosRepository);
					task.step();
				}
			}
		}
	}

}
