package com.bocloud.cos.service.utils;

import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.streaming.SXSSFRow;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFChart;
import org.openxmlformats.schemas.drawingml.x2006.chart.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * excel 导出工具类
 *
 * @author lyy
 * @version 1.0
 * @date 2020/10/12 14:07
 */
public class ExcelPoiHelper {
    private static Logger logger = LoggerFactory.getLogger(ExcelPoiHelper.class);
    private static SXSSFWorkbook workbook;

    public static SXSSFWorkbook create(JSONObject date) {
        try {
            workbook = new SXSSFWorkbook();
            SXSSFSheet sheet = workbook.createSheet("数据页");
            createSurface(sheet, date);
//            Boolean line = null == date.getBoolean("line") ? false : date.getBoolean("line");
//            Boolean bar = null == date.getBoolean("bar") ? false : date.getBoolean("bar");
//            if (!line && !bar) {
//                return workbook;
//            }
//            //处理图表数据
//            List<JSONObject> values = date.getJSONArray("values").toJavaList(JSONObject.class);
//            List<String> keys = date.getJSONArray("keys").toJavaList(String.class);
//            List<String> fldNameArr = Lists.newArrayList();
//            List<Map<String, Object>> dataList = Lists.newArrayList();
//            for (JSONObject value : values) {
//                fldNameArr.add(String.valueOf(value.getString("name")));
//            }
//            int index = 0;
//            for (String key : keys) {
//                Map<String, Object> dataMap = Maps.newHashMap();
//                dataMap.put("value1", key);
//                int num = 2;
//                for (JSONObject value : values) {
//                    List<Double> time = value.getJSONArray("data").toJavaList(Double.class);
//                    dataMap.put("value" + num, time.get(index));
//                    num++;
//                }
//                dataList.add(dataMap);
//                index++;
//            }
//            //创建图表引用数据
//            SXSSFSheet hiddenSheet = drawHiddenSheetTable(fldNameArr, dataList);
//            if (line) {
//                //创建折线图
//                createTimeXYChar(hiddenSheet, fldNameArr, dataList);
//            }
//            if (bar) {
//                //创建柱状图
//                createBarChart(hiddenSheet, fldNameArr, dataList);
//            }
            workbook.setSheetHidden(1, true);
            return workbook;
        } catch (Exception e) {
            logger.error("创建excel失败{}", e.getStackTrace());
            return workbook;
        }
    }

    /**
     * 生成折线图
     *
     * @param sheet
     * @param type
     * @param fldNameArr
     * @param dataList
     * @return
     */
    private static void drawLineSheetMap(SXSSFSheet hiddenSheet, SXSSFSheet sheet, String type, List<String> fldNameArr,
                                         List<Map<String, Object>> dataList) {
        // 获取隐藏页sheet名称
        String sheetName = hiddenSheet.getSheetName();
        //写入标题
//        drawTitle(sheet, titleName, dataList);
        // 创建一个画布
        Drawing<?> drawing = sheet.createDrawingPatriarch();
        // 画一个图区域
        // 前四个默认0，从第5行到第30行,从第0列到第18列的区域
        ClientAnchor anchor = drawing.createAnchor(0, 0, 0, 0, 0, 4, 18, 30);
        // 创建一个chart对象
        Chart chart = drawing.createChart(anchor);
        CTChart ctChart = ((XSSFChart) chart).getCTChart();
        CTPlotArea ctPlotArea = ctChart.getPlotArea();
        // 折线图
        CTLineChart ctLineChart = ctPlotArea.addNewLineChart();
        CTBoolean ctBoolean = ctLineChart.addNewVaryColors();
        ctLineChart.addNewGrouping().setVal(STGrouping.STANDARD);
        // 创建序列,并且设置选中区域
        for (int i = 0; i < fldNameArr.size(); i++) {
            CTLineSer ctLineSer = ctLineChart.addNewSer();
            CTSerTx ctSerTx = ctLineSer.addNewTx();
            // 图例区
            CTStrRef ctStrRef = ctSerTx.addNewStrRef();
            // 选定区域第0行,第1,2,3等等列标题作为图例
            String legendDataRange = new CellRangeAddress(0, 0, i + 1, i + 1).formatAsString(sheetName, true);
            ctStrRef.setF(legendDataRange);
            ctLineSer.addNewIdx().setVal(i);
            // 横坐标区
            CTAxDataSource cttAxDataSource = ctLineSer.addNewCat();
            ctStrRef = cttAxDataSource.addNewStrRef();
            // 选第0列,第1-n行作为横坐标区域
            String axisDataRange = new CellRangeAddress(1, dataList.size(), 0, 0).formatAsString(sheetName, true);
            ctStrRef.setF(axisDataRange);
            // 数据区域
            CTNumDataSource ctNumDataSource = ctLineSer.addNewVal();
            CTNumRef ctNumRef = ctNumDataSource.addNewNumRef();
            // 选择数据区域
            String numDataRange = new CellRangeAddress(1, dataList.size(), i + 1, i + 1).formatAsString(sheetName,
                    true);
            ctNumRef.setF(numDataRange);
            // 设置标签格式
            ctBoolean.setVal(false);
            CTDLbls newDLbls = ctLineSer.addNewDLbls();
            newDLbls.setShowLegendKey(ctBoolean);
            ctBoolean.setVal(true);
            newDLbls.setShowVal(ctBoolean);
            ctBoolean.setVal(false);
            newDLbls.setShowCatName(ctBoolean);
            newDLbls.setShowSerName(ctBoolean);
            newDLbls.setShowPercent(ctBoolean);
            newDLbls.setShowBubbleSize(ctBoolean);
            newDLbls.setShowLeaderLines(ctBoolean);
            // 是否是平滑曲线
            CTBoolean addNewSmooth = ctLineSer.addNewSmooth();
            addNewSmooth.setVal(false);
            // 是否是堆积曲线
            CTMarker addNewMarker = ctLineSer.addNewMarker();
            CTMarkerStyle addNewSymbol = addNewMarker.addNewSymbol();
            addNewSymbol.setVal(STMarkerStyle.NONE);
        }
        // telling the BarChart that it has axes and giving them Ids
        ctLineChart.addNewAxId().setVal(123456);
        ctLineChart.addNewAxId().setVal(123457);
        // cat axis
        CTCatAx ctCatAx = ctPlotArea.addNewCatAx();
        ctCatAx.addNewAxId().setVal(123456); // id of the cat axis
        CTScaling ctScaling = ctCatAx.addNewScaling();
        ctScaling.addNewOrientation().setVal(STOrientation.MIN_MAX);
        ctCatAx.addNewAxPos().setVal(STAxPos.B);
        ctCatAx.addNewCrossAx().setVal(123457); // id of the val axis
        ctCatAx.addNewTickLblPos().setVal(STTickLblPos.NEXT_TO);
        // val axis
        CTValAx ctValAx = ctPlotArea.addNewValAx();
        ctValAx.addNewAxId().setVal(123457); // id of the val axis
        ctScaling = ctValAx.addNewScaling();
        ctScaling.addNewOrientation().setVal(STOrientation.MIN_MAX);
        ctValAx.addNewAxPos().setVal(STAxPos.L);
        ctValAx.addNewCrossAx().setVal(123456); // id of the cat axis
        ctValAx.addNewTickLblPos().setVal(STTickLblPos.NEXT_TO);
        // 是否删除主左边轴
        ctValAx.addNewDelete().setVal(false);
        // 是否删除横坐标
        if (type.equals("line-bar")) {
            ctCatAx.addNewDelete().setVal(true);
        }
        CTLegend ctLegend = ctChart.addNewLegend();
        ctLegend.addNewLegendPos().setVal(STLegendPos.B);
        ctLegend.addNewOverlay().setVal(false);
    }

    /**
     * 生成柱状图
     *
     * @param sheet
     * @param group
     * @param fldNameArr
     * @param dataList
     * @return
     */
    private static void drawSheetMap(SXSSFSheet hiddenSheet, SXSSFSheet sheet, STBarGrouping.Enum group, List<String> fldNameArr,
                                     List<Map<String, Object>> dataList) {
        // 获取隐藏sheet名称
        String sheetName = hiddenSheet.getSheetName();
        //写入标题
//        drawTitle(sheet, titleName, dataList);
        // 创建一个画布
        Drawing<?> drawing = sheet.createDrawingPatriarch();
        // 画一个图区域
        // 前四个默认0，从第5行到第30行,从第0列到第18列的区域
        ClientAnchor anchor = drawing.createAnchor(0, 0, 0, 0, 0, 4, 18, 30);
        // 创建一个chart对象
        Chart chart = drawing.createChart(anchor);
        CTChart ctChart = ((XSSFChart) chart).getCTChart();
        CTPlotArea ctPlotArea = ctChart.getPlotArea();
        // 创建柱状图模型
        CTBarChart ctBarChart = ctPlotArea.addNewBarChart();
        CTBoolean ctBoolean = ctBarChart.addNewVaryColors();
        ctBarChart.getVaryColors().setVal(true);
        // 设置图类型
        ctBarChart.addNewGrouping().setVal(group);
        ctBoolean.setVal(true);
        ctBarChart.addNewBarDir().setVal(STBarDir.COL);
        // 是否添加左侧坐标轴
        ctChart.addNewDispBlanksAs().setVal(STDispBlanksAs.ZERO);
        ctChart.addNewShowDLblsOverMax().setVal(true);
        // 设置这两个参数是为了在STACKED模式下生成堆积模式；(standard)标准模式时需要将这两行去掉
        if ("stacked".equals(group.toString()) || "percentStacked".equals(group.toString())) {
            ctBarChart.addNewGapWidth().setVal(150);
            ctBarChart.addNewOverlap().setVal((byte) 100);
        }
        // 创建序列,并且设置选中区域
        for (int i = 0; i < fldNameArr.size(); i++) {
            CTBarSer ctBarSer = ctBarChart.addNewSer();
            CTSerTx ctSerTx = ctBarSer.addNewTx();
            // 图例区
            CTStrRef ctStrRef = ctSerTx.addNewStrRef();
            // 选定区域第0行,第1,2,3等等列标题作为图例
            String legendDataRange = new CellRangeAddress(0, 0, i + 1, i + 1).formatAsString(sheetName, true);
            ctStrRef.setF(legendDataRange);
            ctBarSer.addNewIdx().setVal(i);
            // 横坐标区
            CTAxDataSource cttAxDataSource = ctBarSer.addNewCat();
            ctStrRef = cttAxDataSource.addNewStrRef();
            // 选第0列,第1-n行作为横坐标区域
            String axisDataRange = new CellRangeAddress(1, dataList.size(), 0, 0).formatAsString(sheetName, true);
            ctStrRef.setF(axisDataRange);
            // 数据区域
            CTNumDataSource ctNumDataSource = ctBarSer.addNewVal();
            CTNumRef ctNumRef = ctNumDataSource.addNewNumRef();
            // 选择数据区域
            String numDataRange = new CellRangeAddress(1, dataList.size(), i + 1, i + 1).formatAsString(sheetName,
                    true);
            ctNumRef.setF(numDataRange);
            // 添加柱状边框线
            ctBarSer.addNewSpPr().addNewLn().addNewSolidFill().addNewSrgbClr().setVal(new byte[]{0, 0, 0});
            // 设置负轴颜色不是白色
            ctBarSer.addNewInvertIfNegative().setVal(false);
            // 设置标签格式
            ctBoolean.setVal(false);
            CTDLbls newDLbls = ctBarSer.addNewDLbls();
            newDLbls.setShowLegendKey(ctBoolean);
            ctBoolean.setVal(true);
            newDLbls.setShowVal(ctBoolean);
            ctBoolean.setVal(false);
            newDLbls.setShowCatName(ctBoolean);
            newDLbls.setShowSerName(ctBoolean);
            newDLbls.setShowPercent(ctBoolean);
            newDLbls.setShowBubbleSize(ctBoolean);
            newDLbls.setShowLeaderLines(ctBoolean);
        }
        // 告诉BarChart它有坐标轴，并给它们id
        ctBarChart.addNewAxId().setVal(123456);
        ctBarChart.addNewAxId().setVal(123457);
        // 横坐标
        CTCatAx ctCatAx = ctPlotArea.addNewCatAx();
        ctCatAx.addNewAxId().setVal(123456); // id of the cat axis
        CTScaling ctScaling = ctCatAx.addNewScaling();
        ctScaling.addNewOrientation().setVal(STOrientation.MIN_MAX);
        ctCatAx.addNewAxPos().setVal(STAxPos.B);
        ctCatAx.addNewCrossAx().setVal(123457); // id of the val axis
        ctCatAx.addNewTickLblPos().setVal(STTickLblPos.NEXT_TO);
        // 纵坐标
        CTValAx ctValAx = ctPlotArea.addNewValAx();
        ctValAx.addNewAxId().setVal(123457); // id of the val axis
        ctScaling = ctValAx.addNewScaling();
        ctScaling.addNewOrientation().setVal(STOrientation.MIN_MAX);
        // 设置位置
        ctValAx.addNewAxPos().setVal(STAxPos.L);
        ctValAx.addNewCrossAx().setVal(123456); // id of the cat axis
        ctValAx.addNewTickLblPos().setVal(STTickLblPos.NEXT_TO);
        // 是否删除主左边轴
        ctValAx.addNewDelete().setVal(false);
        // 是否删除横坐标
        ctCatAx.addNewDelete().setVal(false);
        // legend图注
        // if(true){
        CTLegend ctLegend = ctChart.addNewLegend();
        ctLegend.addNewLegendPos().setVal(STLegendPos.B);
        ctLegend.addNewOverlay().setVal(false);
        // }
    }

    /**
     * 创建柱状图
     *
     * @param hiddenSheet
     * @param fldNameArr
     * @param dataList
     * @throws Exception
     */
    public static void createBarChart(SXSSFSheet hiddenSheet, List<String> fldNameArr, List<Map<String, Object>> dataList) throws Exception {
        // 创建一个sheet页
        SXSSFSheet barSheet = workbook.createSheet("柱状图图表页");
        // drawSheet0Table(sheet,titleArr,fldNameArr,dataList);
        // 堆积=STBarGrouping.STACKED 多组=STBarGrouping.CLUSTERED
        drawSheetMap(hiddenSheet, barSheet, STBarGrouping.CLUSTERED, fldNameArr, dataList);
    }

    /**
     * 创建折线图
     *
     * @param fldNameArr
     * @param dataList
     */
    public static void createTimeXYChar(SXSSFSheet hiddenSheet, List<String> fldNameArr, List<Map<String, Object>> dataList) {
        // 创建一个sheet页
        SXSSFSheet lineSheet = workbook.createSheet("折线图图表页");
        // 第二个参数折线图类型:line=普通折线图,line-bar=折线+柱状图
        drawLineSheetMap(hiddenSheet, lineSheet, "line", fldNameArr, dataList);
    }

    /**
     * 生成图表所需数据
     *
     * @param fldNameArr
     * @param dataList
     * @return
     */
    private static SXSSFSheet drawHiddenSheetTable(List<String> fldNameArr,
                                                   List<Map<String, Object>> dataList) {
        SXSSFSheet hiddenSheet = workbook.createSheet("资源数值隐藏页");
        // 测试时返回值
        // 初始化表格样式
        List<CellStyle> styleList = tableStyle();
        // 根据数据创建excel第一行标题行
        SXSSFRow row0 = hiddenSheet.createRow(0);
        row0.createCell(0).setCellValue("时间点");
        row0.getCell(0).setCellStyle(styleList.get(0));
        for (int i = 1; i < fldNameArr.size() + 1; i++) {
            // 设置标题
            row0.createCell(i).setCellValue(fldNameArr.get(i - 1));
            // 设置标题行样式
            row0.getCell(i).setCellStyle(styleList.get(0));
        }
        // 填充数据
        for (int i = 0; i < dataList.size(); i++) {
            // 获取每一项的数据
            Map<String, Object> data = dataList.get(i);
            // 设置每一行的字段标题和数据
            SXSSFRow rowi = hiddenSheet.createRow(i + 1);
            for (int j = 0; j < data.size(); j++) {
                // 判断是否是标题字段列
                if (j == 0) {
                    rowi.createCell(j).setCellValue((String) data.get("value" + (j + 1)));
                    // 设置左边字段样式
                    hiddenSheet.getRow(i + 1).getCell(j).setCellStyle(styleList.get(0));
                } else {
                    rowi.createCell(j).setCellValue((Double) data.get("value" + (j + 1)));
                    // 设置数据样式
                    hiddenSheet.getRow(i + 1).getCell(j).setCellStyle(styleList.get(2));
                }
            }
        }
        return hiddenSheet;
    }

    private static void createSurface(SXSSFSheet sheet, JSONObject date) {
        List<String> title = date.getJSONObject("data").getJSONArray("title").toJavaList(String.class);
        List<String> value = date.getJSONObject("data").getJSONArray("value").toJavaList(String.class);
        List<Map<String, String>> valueList = Lists.newArrayList();
        int num = 0;
        while (true) {
            if (value.size() > num) {
                Map<String, String> map = Maps.newHashMap();
                map.put("value" + (num + 1), value.get(num));
                num++;
            } else {
                break;
            }
        }
        drawSheetSurface(sheet, title, valueList);
    }

    private static void drawSheetSurface(SXSSFSheet sheet, List<String> title, List<Map<String, String>> valueList) {
        // 初始化表格样式
        List<CellStyle> styleList = tableStyle();
        // 根据数据创建excel第一行标题行
        SXSSFRow row0 = sheet.createRow(0);
        int number = -1;
        for (String titleValue : title) {
            number++;
            row0.createCell(number).setCellValue(titleValue);
            sheet.getRow(0).getCell(number).setCellStyle(styleList.get(0));
            sheet.setColumnWidth(number, 15 * 256);
            sheet.addMergedRegion(new CellRangeAddress(0, 1, number, number));
        }
        // 填充数据
        for (int i = 1; i < valueList.size() + 1; i++) {
            // 获取每一项的数据
            Map<String, String> data = valueList.get(i - 1);
            // 设置每一行的字段标题和数据
            SXSSFRow rowi = sheet.createRow(i + 1);
            for (int j = 0; j < data.size(); j++) {
                rowi.createCell(j).setCellValue(data.get("value" + (j + 1)));
                // 设置左边字段样式
                sheet.getRow(i + 1).getCell(j).setCellStyle(styleList.get(1));
            }
        }
    }

    /**
     * 生成样式
     *
     * @return
     */
    private static List<CellStyle> tableStyle() {
        List<CellStyle> cellStyleList = new ArrayList<CellStyle>();
        // 样式准备
        // 标题样式
        Font font = workbook.createFont();
        font.setColor(HSSFColor.HSSFColorPredefined.WHITE.getIndex());
        CellStyle style = workbook.createCellStyle();
        style.setFillForegroundColor(IndexedColors.ROYAL_BLUE.getIndex());
        style.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        style.setBorderBottom(BorderStyle.THIN); // 下边框
        style.setBorderLeft(BorderStyle.THIN);// 左边框
        style.setBorderTop(BorderStyle.THIN);// 上边框
        style.setBorderRight(BorderStyle.THIN);// 右边框
        style.setAlignment(HorizontalAlignment.LEFT);
        style.setWrapText(true);
        style.setFont(font);
        cellStyleList.add(style);
        CellStyle style1 = workbook.createCellStyle();
        style1.setBorderBottom(BorderStyle.THIN); // 下边框
        style1.setBorderLeft(BorderStyle.THIN);// 左边框
        style1.setBorderTop(BorderStyle.THIN);// 上边框
        style1.setBorderRight(BorderStyle.THIN);// 右边框
        style1.setAlignment(HorizontalAlignment.CENTER);
        cellStyleList.add(style1);
        CellStyle cellStyle = workbook.createCellStyle();
        cellStyle.setBorderTop(BorderStyle.THIN);// 上边框
        cellStyle.setBorderBottom(BorderStyle.THIN); // 下边框
        cellStyle.setBorderLeft(BorderStyle.THIN);// 左边框
        cellStyle.setBorderRight(BorderStyle.THIN);// 右边框
        cellStyle.setAlignment(HorizontalAlignment.CENTER);// 水平对齐方式
        // cellStyle.setVerticalAlignment(VerticalAlignment.TOP);//垂直对齐方式
        cellStyleList.add(cellStyle);
        return cellStyleList;
    }
}
